@@ -15,9 +15,10 @@ SUBROUTINE init_run()
   !
   USE kinds,                    ONLY : DP
   USE control_flags,            ONLY : nbeg, nomore, lwf, iverbosity, iprint, &
-                                       ndr, ndw, tfor, tprnfor, tpre, force_pairing
+                                       ndr, ndw, tfor, tprnfor, tpre, ts_vdw, &
+                                       force_pairing
   USE cp_electronic_mass,       ONLY : emass, emass_cutoff
-  USE ions_base,                ONLY : na, nax, nat, nsp, iforce, amass, ityp, cdms
+  USE ions_base,                ONLY : na, nax, nat, nsp, iforce, amass, cdms
   USE ions_positions,           ONLY : tau0, taum, taup, taus, tausm, tausp, &
                                        vels, velsm, velsp, fion, fionm
   USE gvecw,                    ONLY : ngw, ngw_g, ggp
@@ -63,13 +64,13 @@ SUBROUTINE init_run()
   USE cg_module,                ONLY : allocate_cg
   USE wannier_module,           ONLY : allocate_wannier  
   USE io_files,                 ONLY : tmp_dir, prefix
-  USE io_global,                ONLY : ionode, stdout, io_global_start
+  USE io_global,                ONLY : ionode, stdout
   USE printout_base,            ONLY : printout_base_init
   USE wave_types,               ONLY : wave_descriptor_info
   USE xml_io_base,              ONLY : restart_dir, create_directory, change_directory
   USE orthogonalize_base,       ONLY : mesure_diag_perf, mesure_mmul_perf
   USE ions_base,                ONLY : ions_reference_positions, cdmi
-  USE mp_global,                ONLY : nimage, my_image_id, nbgrp, me_image, intra_image_comm, root_image
+  USE mp_bands,                 ONLY : nbgrp
   USE mp,                       ONLY : mp_barrier
   USE wrappers
   USE ldaU_cp
@@ -77,6 +78,7 @@ SUBROUTINE init_run()
   USE wavefunctions_module,     ONLY : cv0          ! Lingzhu Kong
   USE wannier_base,             ONLY : vnbsp        ! Lingzhu Kong
   USE cp_restart,               ONLY : cp_read_wfc_Kong  ! Lingzhu Kong
+  USE tsvdw_module,             ONLY : tsvdw_initialize
   !
   IMPLICIT NONE
   !
@@ -90,19 +92,10 @@ SUBROUTINE init_run()
   !
   ! ... initialize directories
   !
-  IF( nimage > 1 ) THEN
-     !
-     ! ... When bgrps are used, open a directory for each one
-     !
-     WRITE( dirname, FMT = '( I5.5 )' ) my_image_id
-     tmp_dir = TRIM( tmp_dir ) // '/' // TRIM( dirname )
-     IF( nbeg < 0 ) THEN
-        CALL create_directory( tmp_dir )
-     END IF
-     CALL io_global_start( me_image, root_image )
-     !
-  END IF
-
+  IF( nbeg < 0 ) THEN
+     CALL create_directory( tmp_dir )
+  END IF 
+  !
   CALL plugin_initialization()
 
   IF( nbgrp > 1 .AND. force_pairing ) &
@@ -149,6 +142,12 @@ SUBROUTINE init_run()
                          dfftp%nr2x, dfftp%npl, dfftp%nnr, dffts%nnr, nat, nax, nsp,   &
                          nspin, nbsp, nbspx, nupdwn, nkb, gstart, nudx, &
                          tpre, nbspx_bgrp )
+  ! 
+  !=======================================================================
+  !     Initialization of the TS-vdW code (RAD)
+  !=======================================================================
+  !
+  IF (ts_vdw) CALL tsvdw_initialize()
   !
   !  initialize wave functions descriptors and allocate wf
   !
