@@ -123,7 +123,6 @@ MODULE input
      USE io_global,     ONLY : stdout
      USE autopilot,     ONLY : auto_check
      USE autopilot,     ONLY : restart_p
-     USE control_flags, ONLY : ldamped
      USE control_flags, ONLY : ndw_        => ndw, &
                                ndr_        => ndr, &
                                iprint_     => iprint, &
@@ -158,8 +157,7 @@ MODULE input
                                tnosee_        => tnosee
      USE control_flags, ONLY : tdampions_ => tdampions, &
                                tfor_      => tfor, &
-                               tsdp_      => tsdp, &
-                               lfixatom, tconvthrs
+                               tsdp_      => tsdp
      USE control_flags, ONLY : tnosep_ => tnosep, &
                                tcap_   => tcap, &
                                tcp_    => tcp, &
@@ -188,8 +186,8 @@ MODULE input
      USE control_flags, ONLY : textfor
      USE control_flags, ONLY : do_makov_payne, twfcollect
      USE control_flags, ONLY : lwf, lwfnscf, lwfpbe0, lwfpbe0nscf ! Lingzhu Kong
-     USE control_flags, ONLY : lneb, lsmd, lpath
-     USE control_flags, ONLY : llondon, smallmem
+     USE control_flags, ONLY : smallmem
+     USE control_flags, ONLY : tconvthrs
      !
      ! ...  Other modules
      !
@@ -245,17 +243,13 @@ MODULE input
      forc_conv_thr_ = forc_conv_thr
      ekin_maxiter_  = electron_maxstep
      iesr           = iesr_inp
-     llondon        = london
-     !
      remove_rigid_rot_ = remove_rigid_rot
+     !
+     ! ... define memory- and disk-related internal switches
+     !
+     smallmem = ( TRIM( memory ) == 'small' )
      twfcollect = wf_collect
      !
-     ! ... define memory related internal switches
-     !
-     IF( TRIM( memory ) == 'small' ) THEN
-        smallmem = .TRUE.
-     END IF
-     !
      ! Options for isolated system
      SELECT CASE( TRIM( assume_isolated ) )
          !
@@ -272,7 +266,6 @@ MODULE input
          do_makov_payne = .FALSE.
          !
      END SELECT
-      !
      !
      tefield_  = tefield
      epol_     = epol
@@ -290,13 +283,6 @@ MODULE input
      !
      emass_ = emass
      emaec_ = emass_cutoff
-     ! no longer implemented!
-     lneb = ( TRIM( calculation ) == 'neb' )
-     IF ( lneb ) CALL errore ( 'iosys', &
-                 'NEB no longer implemented, use "neb.x" instead', 1)
-     lsmd = ( TRIM( calculation ) == 'smd' )
-     IF ( lsmd ) CALL errore ( 'iosys', 'SMD no longer implemented', 1)
-     lpath = lneb .OR. lsmd
 !====================================================================
 !Lingzhu Kong
      lwf = ( TRIM( calculation ) == 'cp-wf'      .OR. &
@@ -570,7 +556,6 @@ MODULE input
         CASE ('cg')       ! Conjugate Gradient minimization for ions
           CALL errore( "iosys ", " ion_dynamics = '//TRIM(ion_dynamics)//' not yet implemented ", 1 )
         CASE ('damp')
-          ldamped    = .TRUE.
           tsdp_      = .FALSE.
           tfor_      = .TRUE.
           tdampions_ = .TRUE.
@@ -592,10 +577,6 @@ MODULE input
       !
       IF ( ANY( rd_for(:,1:nat) /= 0.0_DP ) ) textfor = .TRUE.
 
-      ! some atoms are kept fixed
-      !
-      IF ( ANY( if_pos(:,1:nat) == 0 ) ) lfixatom = .TRUE.
-
       ! ... Ionic Temperature
 
       tcp_      = .FALSE.
@@ -618,23 +599,20 @@ MODULE input
 
       ! ... Starting/Restarting ionic velocities
 
-      tcap_         = .FALSE.
+      tzerop_= .FALSE.
+      tv0rd_ = .FALSE.
+      tcap_  = .FALSE.
       SELECT CASE ( TRIM(ion_velocities) )
         CASE ('default')
-          tzerop_ = .FALSE.
-          tv0rd_ = .FALSE.
-          tcap_ = .FALSE.
+          CONTINUE
         CASE ('change_step')
-          tzerop_ = .FALSE.
-          tv0rd_ = .FALSE.
-          tcap_ = .FALSE.
           dt_old_ = tolp
         CASE ('zero')
-          tzerop_ = .TRUE.
-          tv0rd_ = .FALSE.
+          tzerop_= .TRUE.
         CASE ('from_input')
-          tzerop_ = .TRUE.
           tv0rd_  = .TRUE.
+          IF( .NOT. tavel  ) CALL errore(' iosys ', &
+                               ' ION_VELOCITIES not present in stdin ', 1 )
         CASE ('random')
           tcap_ = .TRUE.
         CASE DEFAULT
@@ -752,9 +730,6 @@ MODULE input
       IF( .NOT. trd_ht .AND. ibrav == 0 ) &
         CALL errore(' iosys ',' ibrav = 0 but CELL_PARAMETERS not present in stdin ', 1 )
 
-      IF( .NOT. tavel .AND. TRIM(ion_velocities)=='from_input' ) &
-        CALL errore(' iosys ',' ION_VELOCITIES not present in stdin ', 1 )
-
       RETURN
    END SUBROUTINE set_control_flags
    !
@@ -762,10 +737,6 @@ MODULE input
    SUBROUTINE modules_setup()
      !-------------------------------------------------------------------------
      !
-     USE control_flags,    ONLY : lconstrain, tpre, thdyn, tksw
-
-     USE constants,        ONLY : amu_au, pi
-     !
      USE input_parameters, ONLY: ibrav , celldm , trd_ht, dt,                 &
            rd_ht, a, b, c, cosab, cosac, cosbc, ntyp , nat ,                  &
            na_inp , sp_pos , rd_pos , rd_vel, atom_mass, atom_label, if_pos,  &
@@ -805,8 +776,11 @@ MODULE input
                                   axis
      USE input_parameters, ONLY : lda_plus_u, Hubbard_U
      USE input_parameters, ONLY : step_pen, A_pen, alpha_pen, sigma_pen
-     USE input_parameters, ONLY : london, london_s6, london_rcut
+     USE input_parameters, ONLY : vdw_corr, london, london_s6, london_rcut, &
+                                  ts_vdw, ts_vdw_isolated, ts_vdw_econv_thr
      !
+     USE constants,        ONLY : amu_au, pi
+     USE control_flags,    ONLY : lconstrain, tpre, thdyn, tksw
      USE ions_base,        ONLY : zv
      USE cell_base,        ONLY : cell_base_init, cell_dyn_init, at, cell_alat
      USE cell_nose,        ONLY : cell_nose_init
@@ -828,16 +802,18 @@ MODULE input
      USE ensemble_dft,     ONLY : ensemble_initval,tens
      USE wannier_base,     ONLY : wannier_init
      USE efield_module,    ONLY : tefield
-     USE funct,            ONLY : dft_is_nonlocc
+     USE funct,            ONLY : dft_is_nonlocc, get_inlc
      USE kernel_table,     ONLY : vdw_table_name_ => vdw_table_name, &
                                   initialize_kernel_table
+     USE control_flags,    ONLY : llondon, ts_vdw_ => ts_vdw
      USE london_module,    ONLY : init_london, scal6, lon_rcut
+     USE tsvdw_module,     ONLY : vdw_isolated, vdw_econv_thr
      !
      IMPLICIT NONE
      !
      REAL(DP) :: alat_ , massa_totale
      ! ...   DIIS
-     INTEGER :: ia, iss
+     INTEGER :: ia, iss, inlc
      LOGICAL :: ltest
      !
      !   Subroutine Body
@@ -865,7 +841,10 @@ MODULE input
      ! ...   Set Values for the cutoff
 
      CALL ecutoffs_setup( ecutwfc, ecutrho, ecfixed, qcutz, q2sigma, refg )
-
+     if (.not. allocated(xk)) then
+       allocate(xk(3,1))
+       xk = 0.d0
+     endif
      CALL gcutoffs_setup( alat_ , tk_inp, nkstot, xk )
 
      ! ... 
@@ -960,17 +939,52 @@ MODULE input
      CALL ldaU_init0 ( ntyp, lda_plus_u, Hubbard_U )
      CALL ldaUpen_init( SIZE(sigma_pen), step_pen, sigma_pen, alpha_pen, A_pen )
      !
-     IF ( london) THEN
+     !  ... initialize variables for vdW (dispersions) corrections
+     !
+     SELECT CASE( TRIM( vdw_corr ) )
+         !
+       CASE( 'grimme-d2', 'Grimme-D2', 'DFT-D', 'dft-d' )
+         !
+         llondon= .TRUE.
+         ts_vdw_= .FALSE.
+         !
+       CASE( 'TS', 'ts', 'ts-vdw', 'ts-vdW', 'tkatchenko-scheffler' )
+         !
+         llondon= .FALSE.
+         ts_vdw_= .TRUE.
+         !
+       CASE DEFAULT
+         !
+         llondon= .FALSE.
+         ts_vdw_= .FALSE.
+         !
+     END SELECT
+     ! 
+     IF ( ts_vdw ) THEN
+        CALL infomsg("iosys","ts_vdw is obsolete, use ''vdw_corr='ts-vdw''' instead")
+        ts_vdw_ = .TRUE.
+     END IF
+     IF ( london ) THEN
+        CALL infomsg("iosys","london is obsolete, use ''vdw_corr='grimme-d2''' instead")
+        llondon = .TRUE.
+     END IF
+     IF (ts_vdw_.AND.llondon) CALL errore("iosys", &
+                                    "must choose a unique vdW correction!", 1)
+     IF ( llondon) THEN
         lon_rcut    = london_rcut
         scal6       = london_s6
         CALL init_london ( )
+     ELSE IF ( ts_vdw_ ) THEN
+        vdw_isolated = ts_vdw_isolated
+        vdw_econv_thr= ts_vdw_econv_thr
      END IF
      !
      ! ... initialize kernel table for nonlocal functionals
      !
      IF ( dft_is_nonlocc( ) ) THEN
         vdw_table_name_ = vdw_table_name
-        CALL initialize_kernel_table()
+        inlc = get_inlc()
+        call initialize_kernel_table(inlc)
      ENDIF
      !
      RETURN
@@ -1012,8 +1026,6 @@ MODULE input
 505 FORMAT(   3X,'MD Simulation time step            = ',F10.2)
 510 FORMAT(   3X,'Electronic fictitious mass (emass) = ',F10.2)
 511 FORMAT(   3X,'emass cut-off                      = ',F10.2)
-509 FORMAT(   3X,'Verlet algorithm for electron dynamics')
-502 FORMAT(   3X,'An initial quench is performed')
 
     RETURN
   END SUBROUTINE input_info
