@@ -23,7 +23,7 @@
                                     printout_pos, printout_cell, printout_stress
       USE constants,         ONLY : au_gpa, bohr_radius_cm, amu_au, &
                                     BOHR_RADIUS_ANGS, pi
-      USE ions_base,         ONLY : na, nsp, nat, ind_bck, atm, ityp, amass, cdmi, &
+      USE ions_base,         ONLY : na, nsp, nat, ind_bck, atm, amass, cdmi, &
                                     ions_cofmass, ions_displacement, label_srt
       USE cell_base,         ONLY : s_to_r, get_volume
       USE efield_module,     ONLY : tefield, pberryel, pberryion, &
@@ -41,6 +41,8 @@
       USE io_global,         ONLY : ionode, ionode_id, stdout
       USE control_flags,     ONLY : lwfpbe0, lwfpbe0nscf  ! Lingzhu Kong
       USE energies,          ONLY : exx  ! Lingzhu Kong
+      USE control_flags,     ONLY : ts_vdw
+      USE tsvdw_module,      ONLY : EtsvdW
       !
       IMPLICIT NONE
       !
@@ -257,7 +259,17 @@
       IF( ( MOD( nfi, iprint_stdout ) == 0 ) .OR. tfirst )  THEN
          !
          WRITE( stdout, * )
-         WRITE( stdout, 1947)
+         !
+         IF (ts_vdw) THEN
+           !
+           WRITE( stdout, 19470)
+           !
+         ELSE
+           !
+           WRITE( stdout, 1947)
+           !
+         END IF
+         !
          IF ( abivol .AND. pvar ) write(stdout,*) 'P = ', P_ext*au_gpa
          !
       END IF
@@ -276,8 +288,19 @@
          ' # of electrons within the isosurface = ', n_ele
 
       IF( .not. tcg ) THEN
-         WRITE( stdout, 1948 ) nfi, ekinc, temphc, tempp, etot, enthal, &
+         !
+         IF (ts_vdw) THEN
+           !
+           WRITE(stdout,19480) nfi, ekinc, temphc, tempp, etot, enthal, &
+                      econs, econt, vnhh(3,3), xnhh0(3,3), vnhp(1), xnhp0(1), EtsvdW
+           !
+         ELSE
+           !
+           WRITE(stdout, 1948) nfi, ekinc, temphc, tempp, etot, enthal, &
                       econs, econt, vnhh(3,3), xnhh0(3,3), vnhp(1),  xnhp0(1)
+           !
+         END IF
+         !
       ELSE
          IF ( MOD( nfi, iprint ) == 0 .OR. tfirst ) THEN
             !
@@ -299,16 +322,22 @@
       !
       !
 255   FORMAT( '     ',A5,A8,3(1X,A12),A6 )
-256   FORMAT( 'Step ',I5,1X,I7,1X,F12.5,1X,F12.5,1X,F12.5,1X,I5 )
+256   FORMAT( 'Step ',I5,1X,I7,1X,F13.6,1X,F13.6,1X,F13.6,1X,I5 )
 1000  FORMAT(/,3X,'Center of mass square displacement (a.u.): ',F10.6,/)
 1944  FORMAT(//'   Partial temperatures (for each ionic specie) ', &
              /,'   Species  Temp (K)   Mean Square Displacement (a.u.)')
-1945  FORMAT(3X,I6,1X,F10.2,1X,F10.4)
+1945  FORMAT(3X,I6,1X,ES10.2,1X,ES14.4)
+!1947  FORMAT( 2X,'nfi',5X,'ekinc',11X,'temph',7X,'tempp',7X,'etot',12X,'enthal', &
+!           & 10X,'econs',11X,'econt',11X,'vnhh',11X,'xnhh0',10X,'vnhp',11X,'xnhp0' )
+!1948  FORMAT( I5,1X,ES15.5,1X,ES11.1,1X,ES11.1,4(1X,ES15.5),4(1X,ES14.4) )
 1947  FORMAT( 2X,'nfi',4X,'ekinc',2X,'temph',2X,'tempp',11X,'etot',9X,'enthal', &
            & 10X,'econs',10X,'econt',4X,'vnhh',3X,'xnhh0',4X,'vnhp',3X,'xnhp0' )
-1948  FORMAT( I5,1X,F8.5,1X,F6.1,1X,F6.1,4(1X,F14.5),4(1X,F7.4) )
-2948  FORMAT( I6,1X,F8.5,1X,F6.1,1X,F6.1,4(1X,F14.5),F10.2, F8.2, F8.5 )
-2949  FORMAT( I6,1X,4(1X,F7.4), F8.5 )
+19470 FORMAT( 2X,'nfi',4X,'ekinc',2X,'temph',2X,'tempp',11X,'etot',9X,'enthal', &
+           & 10X,'econs',10X,'econt',4X,'vnhh',3X,'xnhh0',4X,'vnhp',3X,'xnhp0',5X,'evdw')  ! GGA+TS-vdW
+1948  FORMAT( I5,1X,F8.5,1X,F6.1,1X,F6.1,4(1X,F14.6),4(1X,F7.4) )
+19480 FORMAT( I5,1X,F8.5,1X,F6.1,1X,F6.1,4(1X,F14.5),4(1X,F7.4),2X,F14.5 ) ! GGA+TS-vdW
+2948  FORMAT( I6,1X,ES18.10,1X,ES18.10,1X,ES18.10,4(1X,ES18.10),ES18.10, ES18.10, ES18.10 )
+2949  FORMAT( I6,1X,4(1X,ES18.10), ES18.10 )
       !
       RETURN
    END SUBROUTINE printout_new_x
@@ -380,7 +409,7 @@
         WRITE( stdout,1959) avgs(9), avgs_run(9)
         WRITE( stdout,1990)
  1949   FORMAT(//,3X,'Averaged Physical Quantities',/ &
-              ,3X,'                  ',' accomulated','      this run')
+              ,3X,'                  ',' accumulated','      this run')
  1951   FORMAT(3X,'ekinc         : ',F14.5,F14.5,' (AU)')
  1952   FORMAT(3X,'ekin          : ',F14.5,F14.5,' (AU)')
  1953   FORMAT(3X,'epot          : ',F14.5,F14.5,' (AU)')
