@@ -25,6 +25,7 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
                                   tprnfor, iesr, textfor
       USE io_global,        ONLY: stdout
       USE ions_base,        ONLY: nsp, na, nat, rcmax, compute_eextfor
+      USE ions_base,        ONLY: ind_srt, ind_bck
       USE gvecs
       USE gvect,            ONLY: ngm, nl, nlm
       USE cell_base,        ONLY: omega, r_to_s
@@ -41,8 +42,9 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
                                   detot6, dekin6, dps6, dh6, dsr6, dxc6, denl6
       USE mp,               ONLY: mp_sum
       USE mp_global,        ONLY: intra_bgrp_comm
-      USE funct,            ONLY: dft_is_meta, dft_is_nonlocc, nlc
+      USE funct,            ONLY: dft_is_meta, dft_is_nonlocc, nlc, get_inlc
       USE vdW_DF,           ONLY: stress_vdW_DF
+      use rVV10,            ONLY: stress_rVV10 
       USE pres_ai_mod,      ONLY: abivol, abisur, v_vol, P_ext, volclu,  &
                                   Surf_t, surfclu
       USE fft_interfaces,   ONLY: fwfft, invfft
@@ -52,7 +54,10 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
                                   add_drhoph, stress_local, force_loc, self_vofhar
       USE fft_base,         ONLY: dfftp, dffts
       USE ldaU_cp,          ONLY: e_hubbard
-      USE step_penalty,     ONLY: e_pen
+      USE control_flags,    ONLY: ts_vdw
+      USE tsvdw_module,     ONLY: tsvdw_calculate
+      USE tsvdw_module,     ONLY: EtsvdW,UtsvdW,FtsvdW,HtsvdW
+      USE mp_global,        ONLY: me_image
 
       IMPLICIT NONE
 !
@@ -67,7 +72,7 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
       COMPLEX(DP) :: sfac(:,:)
       INTEGER     :: irb(:,:)
       !
-      INTEGER iss, isup, isdw, ig, ir, i, j, k, ij, is, ia
+      INTEGER iss, isup, isdw, ig, ir, i, j, k, ij, is, ia, inlc
       REAL(DP) :: vtxc, vave, ebac, wz, eh, ehpre, enlc
       COMPLEX(DP)  fp, fm, ci, drhop, zpseu, zh
       COMPLEX(DP), ALLOCATABLE :: rhotmp(:), vtemp(:)
@@ -98,6 +103,20 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
 
       CALL start_clock( 'vofrho' )
 
+      !
+      !     TS-vdW calculation (RAD)
+      !
+      IF (ts_vdw) THEN
+        !
+        CALL start_clock( 'ts_vdw' )
+        ALLOCATE (stmp(3,nat))
+        stmp(:,:) = tau0(:,ind_bck(:))
+        CALL tsvdw_calculate(stmp,rhor)
+        DEALLOCATE (stmp)
+        CALL stop_clock( 'ts_vdw' )
+        !
+      END IF
+      !
       ci = ( 0.0d0, 1.0d0 )
       !
       !     wz = factor for g.neq.0 because of c*(g)=c(-g)
@@ -329,8 +348,8 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
          !
          ! ... UGLY HACK WARNING: nlc adds nonlocal term (Ry) to input energy
          !
-	 enlc = 0.0_dp
-         CALL nlc( rhosave, rhocsave, enlc, vtxc, rhor )
+         enlc = 0.0_dp
+         CALL nlc( rhosave, rhocsave, nspin, enlc, vtxc, rhor )
          CALL mp_sum( enlc, intra_bgrp_comm )
          exc = exc + enlc / e2
          !
@@ -339,7 +358,16 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
          !
          IF ( tpre ) THEN
              denlc(:,:) = 0.0_dp
-             CALL stress_vdW_DF( rhosave, rhocsave, denlc )
+             inlc = get_inlc()
+
+             if (inlc==1 .or. inlc==2) then
+               if (nspin>2) call errore('stres_vdW_DF', 'vdW+DF non implemented in spin polarized calculations',1)
+               CALL stress_vdW_DF(rhosave, rhocsave, nspin, denlc )
+             elseif (inlc == 3) then
+               if (nspin>2) call errore('stress_rVV10', 'rVV10 non implemented with nspin>2',1)
+               CALL stress_rVV10(rhosave, rhocsave, nspin, denlc )
+             end if
+
              CALL mp_sum( denlc, intra_bgrp_comm )
              dxc(:,:) = dxc(:,:) - omega/e2 * MATMUL(denlc,TRANSPOSE(ainv))
          END IF
@@ -348,6 +376,39 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
          denlc(:,:) = 0.0_dp
       END IF
       !
+      !     Add TS-vdW wavefunction forces to rhor here... (RAD)
+      !
+      IF (ts_vdw) THEN
+        !
+        IF (dffts%npp(me_image+1).NE.0) THEN
+          !
+          IF (nspin.EQ.1) THEN
+            !
+!$omp parallel do
+            DO ir=1,dffts%npp(me_image+1)*dfftp%nr1*dfftp%nr2
+              !
+              rhor(ir,1)=rhor(ir,1)+UtsvdW(ir)
+              !
+            END DO
+!$omp end parallel do
+            !
+          ELSE IF (nspin.EQ.2) THEN
+            !
+!$omp parallel do
+            DO ir=1,dffts%npp(me_image+1)*dfftp%nr1*dfftp%nr2
+              !
+              rhor(ir,1)=rhor(ir,1)+UtsvdW(ir)
+              rhor(ir,2)=rhor(ir,2)+UtsvdW(ir)
+              !
+            END DO
+!$omp end parallel do
+            !
+          END IF
+          !
+        END IF
+        !
+      END IF
+!
 !     rhor contains the xc potential in r-space
 !
 !     ===================================================================
@@ -425,7 +486,14 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
          !    add g-space ionic and core correction contributions to fion
          !
          fion = fion + fion1
-
+         !
+         !    Add TS-vdW ion forces to fion here... (RAD)
+         !
+         IF (ts_vdw) THEN
+            fion1(:,:) = FtsvdW(:,ind_srt(:))
+            fion = fion + fion1
+            !fion=fion+FtsvdW
+         END IF
       END IF
 
       DEALLOCATE( fion1 )
@@ -535,7 +603,11 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
       !
       !     etot is the total energy ; ekin, enl were calculated in rhoofr
       !
-      etot = ekin + eht + epseu + enl + exc + ebac +e_hubbard + eextfor + e_pen
+      etot = ekin + eht + epseu + enl + exc + ebac +e_hubbard + eextfor
+      !
+      !     Add TS-vdW energy to etot here... (RAD)
+      !
+      IF (ts_vdw)  etot=etot+EtsvdW
       !
       if (abivol) etot = etot + P_ext*volclu
       if (abisur) etot = etot + Surf_t*surfclu
@@ -555,6 +627,10 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
          !
          detot = detot + denl + dxc
          !
+         !     Add TS-vdW cell derivatives to detot here... (RAD)
+         !
+         IF (ts_vdw) detot = detot + HtsvdW
+         !
       END IF
       !
       CALL stop_clock( 'vofrho' )
@@ -611,6 +687,12 @@ SUBROUTINE vofrho_x( nfi, rhor, drhor, rhog, drhog, rhos, rhoc, tfirst, &
             WRITE( stdout,*) "kbar"
             detmp = -1.0d0 * MATMUL( dxc, TRANSPOSE( h ) ) / omega * au_gpa * 10.0d0
             WRITE( stdout,5555) ((detmp(i,j),j=1,3),i=1,3)
+            !
+            WRITE( stdout,*) "derivative of e(TS-vdW)"
+            WRITE( stdout,5555) ((HtsvdW(i,j),j=1,3),i=1,3)
+            WRITE( stdout,*) "kbar"
+            detmp = -1.0d0 * MATMUL( HtsvdW, TRANSPOSE( h ) ) / omega * au_gpa * 10.0d0
+            WRITE( stdout,5555) ((detmp(i,j),j=1,3),i=1,3)
          ENDIF
       ENDIF
 
