@@ -2,175 +2,119 @@
 # Associate images original text with physical files.
 
 
-$key = q/displaystyletau_{2}^{};MSF=1.6;LFS=12;AAT/;
+$key = q/psi_{{j+1}}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img14.png"
- ALT="$\displaystyle \tau_{2}^{}$">|; 
+ WIDTH="41" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img6.png"
+ ALT="$ \psi_{{j+1}}^{}$">|; 
 
 $key = q/Phi;MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="18" HEIGHT="16" ALIGN="BOTTOM" BORDER="0"
- SRC="|."$dir".q|img5.png"
+ SRC="|."$dir".q|img4.png"
  ALT="$ \Phi$">|; 
 
 $key = q/displaystyletau_{1}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img12.png"
+ SRC="|."$dir".q|img11.png"
  ALT="$\displaystyle \tau_{1}^{}$">|; 
 
-$key = q/psi_{{j+1}}^{};MSF=1.6;LFS=12;AAT/;
+$key = q/psi^{*}_{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="41" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img7.png"
- ALT="$ \psi_{{j+1}}^{}$">|; 
+ WIDTH="25" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img3.png"
+ ALT="$ \psi^{*}_{}$">|; 
 
-$key = q/phi_{i}^{};MSF=1.6;LFS=12;AAT/;
+$key = q/tau_{i}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="21" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img26.png"
- ALT="$ \phi_{i}^{}$">|; 
+ WIDTH="18" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img16.png"
+ ALT="$ \tau_{i}^{}$">|; 
 
-$key = q/bf{G;MSF=1.6;LFS=12;AAT/;
+$key = q/displaystyletau_{3}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="22" HEIGHT="20" ALIGN="BOTTOM" BORDER="0"
- SRC="|."$dir".q|img24.png"
- ALT="$ \bf G$">|; 
-
-$key = q/displaystyle{frac{{i-1}}{{nr1}}};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="48" HEIGHT="61" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img11.png"
- ALT="$\displaystyle {\frac{{i-1}}{{nr1}}}$">|; 
-
-$key = q/bf{k;MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="16" HEIGHT="20" ALIGN="BOTTOM" BORDER="0"
- SRC="|."$dir".q|img23.png"
- ALT="$ \bf k$">|; 
-
-$key = q/framebox{{sum_{bfk}equiv{sumoverlocal{bfk}-points+{ttmp_sum}on{ttinter_pool_comm}};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="523" HEIGHT="28" ALIGN="BOTTOM" BORDER="0"
- SRC="|."$dir".q|img21.png"
- ALT="\framebox{$\sum_{\bf k}\equiv$ sum over local {\bf k}-points +
-{\tt mp\_sum} on {\tt inter \_pool\_comm}}">|; 
+ WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img15.png"
+ ALT="$\displaystyle \tau_{3}^{}$">|; 
 
 $key = q/tau_{3}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img20.png"
+ SRC="|."$dir".q|img19.png"
  ALT="$ \tau_{3}^{}$">|; 
 
 $key = q/displaystyle{frac{{j-1}}{{nr2}}};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="50" HEIGHT="61" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img13.png"
+ SRC="|."$dir".q|img12.png"
  ALT="$\displaystyle {\frac{{j-1}}{{nr2}}}$">|; 
 
+$key = q/displaystyle{frac{{k-1}}{{nr3}}};MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="52" HEIGHT="61" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img14.png"
+ ALT="$\displaystyle {\frac{{k-1}}{{nr3}}}$">|; 
+
+$key = q/{Phi(G)-Phi^*(-G)over2i};MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="98" HEIGHT="47" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img8.png"
+ ALT="$ {\Phi(G) - \Phi^*(-G)\over 2i}$">|; 
+
+$key = q/displaystyletau_{2}^{};MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img13.png"
+ ALT="$\displaystyle \tau_{2}^{}$">|; 
+
 $key = q/psi;MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="17" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img3.png"
+ SRC="|."$dir".q|img2.png"
  ALT="$ \psi$">|; 
 
-$key = q/langle;MSF=1.6;LFS=12;AAT/;
+$key = q/displaystyle{frac{{i-1}}{{nr1}}};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="12" HEIGHT="39" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img25.png"
- ALT="$ \langle$">|; 
-
-$key = q/tau_{i}^{};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="18" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img17.png"
- ALT="$ \tau_{i}^{}$">|; 
-
-$key = q/psi^{*}_{};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="25" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img4.png"
- ALT="$ \psi^{*}_{}$">|; 
-
-$key = q/rangle;MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="12" HEIGHT="39" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img28.png"
- ALT="$ \rangle$">|; 
+ WIDTH="48" HEIGHT="61" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img10.png"
+ ALT="$\displaystyle {\frac{{i-1}}{{nr1}}}$">|; 
 
 $key = q/{Phi(G)+Phi^*(-G)over2};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="98" HEIGHT="47" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img8.png"
+ SRC="|."$dir".q|img7.png"
  ALT="$ {\Phi(G) + \Phi^*(-G)\over 2}$">|; 
 
 $key = q/Gamma;MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="16" HEIGHT="16" ALIGN="BOTTOM" BORDER="0"
- SRC="|."$dir".q|img2.png"
+ SRC="|."$dir".q|img1.png"
  ALT="$ \Gamma$">|; 
 
 $key = q/le;MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="19" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img10.png"
- ALT="$ \le$">|; 
-
-$key = q/tau_{2}^{};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img19.png"
- ALT="$ \tau_{2}^{}$">|; 
-
-$key = q/{Phi(G)-Phi^*(-G)over2i};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="98" HEIGHT="47" ALIGN="MIDDLE" BORDER="0"
  SRC="|."$dir".q|img9.png"
- ALT="$ {\Phi(G) - \Phi^*(-G)\over 2i}$">|; 
-
-$key = q/displaystyle{frac{{k-1}}{{nr3}}};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="52" HEIGHT="61" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img15.png"
- ALT="$\displaystyle {\frac{{k-1}}{{nr3}}}$">|; 
-
-$key = q/framebox{{sum_iequiv{sumoverlocalbands+{ttmp_sum}on{ttinter_bgrp_comm}};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="489" HEIGHT="28" ALIGN="BOTTOM" BORDER="0"
- SRC="|."$dir".q|img22.png"
- ALT="\framebox{$\sum_i\equiv$ sum over local bands + {\tt mp\_sum} on
-{\tt inter\_bgrp\_comm}}">|; 
+ ALT="$ \le$">|; 
 
 $key = q/psi_{j}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="24" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img6.png"
+ SRC="|."$dir".q|img5.png"
  ALT="$ \psi_{j}^{}$">|; 
 
-$key = q/phi_{j}^{};MSF=1.6;LFS=12;AAT/;
+$key = q/tau_{2}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="23" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img27.png"
- ALT="$ \phi_{j}^{}$">|; 
+ WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img18.png"
+ ALT="$ \tau_{2}^{}$">|; 
 
 $key = q/tau_{1}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img18.png"
+ SRC="|."$dir".q|img17.png"
  ALT="$ \tau_{1}^{}$">|; 
 
-$key = q/framebox{{sum_{bfG}equiv{{ttmp_sum}on{ttintra_bgrp_comm}};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="298" HEIGHT="28" ALIGN="BOTTOM" BORDER="0"
- SRC="|."$dir".q|img29.png"
- ALT="\framebox{$\sum_{\bf G} \equiv$ {\tt mp\_sum} on {\tt intra\_bgrp\_comm}}">|; 
-
-$key = q/displaystyletau_{3}^{};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img16.png"
- ALT="$\displaystyle \tau_{3}^{}$">|; 
-
 1;
 
