@@ -2,11 +2,15 @@
 # Associate internals original text with physical files.
 
 
-$key = q/Sec:git/;
+$key = q/SubSec:Inst/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:Contrib/;
+$key = q/SubSec:propedit/;
+$ref_files{$key} = "$dir".q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:conf/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
@@ -14,11 +18,11 @@ $key = q/SubSec:Conflicts/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:BeDev/;
+$key = q/SubSec:Bugs/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:conf/;
+$key = q/SubSec:CPP/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
@@ -30,25 +34,5 @@ $key = q/SubSec:Merge/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:Bugs/;
-$ref_files{$key} = "$dir".q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:propedit/;
-$ref_files{$key} = "$dir".q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:Inst/;
-$ref_files{$key} = "$dir".q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:qeforge/;
-$ref_files{$key} = "$dir".q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:CPP/;
-$ref_files{$key} = "$dir".q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
 1;
 
