@@ -2,43 +2,11 @@
 # Associate labels original text with physical files.
 
 
-$key = q/SubSec:propedit/;
-$external_labels{$key} = "$URL/" . q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:Bugs/;
-$external_labels{$key} = "$URL/" . q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
 $key = q/SubSec:Inst/;
 $external_labels{$key} = "$URL/" . q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:qeforge/;
-$external_labels{$key} = "$URL/" . q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/Sec:SVN/;
-$external_labels{$key} = "$URL/" . q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:Merge/;
-$external_labels{$key} = "$URL/" . q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:CPP/;
-$external_labels{$key} = "$URL/" . q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/Sec:git/;
-$external_labels{$key} = "$URL/" . q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:Contrib/;
-$external_labels{$key} = "$URL/" . q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:BeDev/;
+$key = q/SubSec:propedit/;
 $external_labels{$key} = "$URL/" . q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
@@ -50,6 +18,22 @@ $key = q/SubSec:Conflicts/;
 $external_labels{$key} = "$URL/" . q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
+$key = q/SubSec:Bugs/;
+$external_labels{$key} = "$URL/" . q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:CPP/;
+$external_labels{$key} = "$URL/" . q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/Sec:SVN/;
+$external_labels{$key} = "$URL/" . q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Merge/;
+$external_labels{$key} = "$URL/" . q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
 1;
 
 
@@ -57,38 +41,14 @@ $noresave{$key} = "$nosave";
 # labels from external_latex_labels array.
 
 
-$key = q/SubSec:CPP/;
-$external_latex_labels{$key} = q|5.1.1|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:qeforge/;
-$external_latex_labels{$key} = q|3.1|; 
-$noresave{$key} = "$nosave";
-
 $key = q/SubSec:Inst/;
 $external_latex_labels{$key} = q|5.1|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:Bugs/;
-$external_latex_labels{$key} = q|3.5|; 
-$noresave{$key} = "$nosave";
-
 $key = q/SubSec:propedit/;
 $external_latex_labels{$key} = q|11.4|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:Merge/;
-$external_latex_labels{$key} = q|11.3|; 
-$noresave{$key} = "$nosave";
-
-$key = q/Sec:SVN/;
-$external_latex_labels{$key} = q|11|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:BeDev/;
-$external_latex_labels{$key} = q|3|; 
-$noresave{$key} = "$nosave";
-
 $key = q/SubSec:conf/;
 $external_latex_labels{$key} = q|5.1.2|; 
 $noresave{$key} = "$nosave";
@@ -97,16 +57,20 @@ $key = q/SubSec:Conflicts/;
 $external_latex_labels{$key} = q|11.2|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:Contrib/;
-$external_latex_labels{$key} = q|3.3|; 
+$key = q/SubSec:Bugs/;
+$external_latex_labels{$key} = q|3.5|; 
 $noresave{$key} = "$nosave";
 
-$key = q/_oldnewlabel/;
-$external_latex_labels{$key} = q|<|; 
+$key = q/SubSec:CPP/;
+$external_latex_labels{$key} = q|5.1.1|; 
 $noresave{$key} = "$nosave";
 
-$key = q/Sec:git/;
-$external_latex_labels{$key} = q|12|; 
+$key = q/Sec:SVN/;
+$external_latex_labels{$key} = q|11|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Merge/;
+$external_latex_labels{$key} = q|11.3|; 
 $noresave{$key} = "$nosave";
 
 1;
