@@ -22,7 +22,11 @@
     !  celldm: old-style parameters of the simulation cell (se latgen.f90)
     REAL(DP) :: celldm(6) = (/ 0.0_DP,0.0_DP,0.0_DP,0.0_DP,0.0_DP,0.0_DP /)
     !  traditional crystallographic cell parameters (alpha=cosbc and so on)
+    
     REAL(DP) :: a, b, c, cosab, cosac, cosbc
+    ! format of input cell parameters:
+    ! 'alat','bohr','angstrom'
+    CHARACTER(len=80) :: cell_units
     !  alat: lattice parameter - often used to scale quantities, or
     !  in combination to other parameters/constants to define new units
     REAL(DP) :: alat = 0.0_DP
@@ -99,7 +103,7 @@
 !------------------------------------------------------------------------------!
 !
   SUBROUTINE cell_base_init( ibrav_, celldm_, a_, b_, c_, cosab_, cosac_, &
-               cosbc_, trd_ht, rd_ht, cell_units )
+               cosbc_, trd_ht, rd_ht, cell_units_ )
     !
     ! ... initialize cell_base module variables, set up crystal lattice
     !
@@ -109,20 +113,22 @@
     REAL(DP), INTENT(IN) :: celldm_ (6)
     LOGICAL, INTENT(IN) :: trd_ht
     REAL(DP), INTENT(IN) :: rd_ht (3,3)
-    CHARACTER(LEN=*), INTENT(IN) :: cell_units
+    CHARACTER(LEN=*), INTENT(IN) :: cell_units_
     REAL(DP), INTENT(IN) :: a_ , b_ , c_ , cosab_, cosac_, cosbc_
 
     REAL(DP) :: units
     !
     IF ( ibrav_ == 0 .and. .not. trd_ht ) THEN
        CALL errore('cell_base_init', 'ibrav=0: must read cell parameters', 1)
-    ELSE IF ( ibrav /= 0 .and. trd_ht ) THEN
+    ELSE IF ( ibrav_ /= 0 .and. trd_ht ) THEN
        CALL errore('cell_base_init', 'redundant data for cell parameters', 2)
     END IF
     !
     ibrav  = ibrav_
     celldm = celldm_
     a = a_ ; b = b_ ; c = c_ ; cosab = cosab_ ; cosac = cosac_ ; cosbc = cosbc_
+    cell_units = cell_units_
+    units = 0.0_DP
     !
     IF ( trd_ht ) THEN
       !
@@ -131,20 +137,38 @@
       SELECT CASE ( TRIM( cell_units ) )
         CASE ( 'bohr' )
           IF( celldm( 1 ) /= 0.0_DP .OR. a /= 0.0_dp ) CALL errore &
-              ('cell_base_init','lattice vectors in Bohr or in a0 units?',1)
+              ('cell_base_init','lattice parameter specified twice',1)
           units = 1.0_DP
         CASE ( 'angstrom' )
           IF( celldm( 1 ) /= 0.0_DP .OR. a /= 0.0_dp ) CALL errore &
-              ('cell_base_init','lattice vectors in A or in a0 units?',2)
+              ('cell_base_init','lattice parameter specified twice',2)
           units = 1.0_DP / bohr_radius_angs
-        CASE DEFAULT
+        CASE ( 'alat' ) 
           IF( celldm( 1 ) /= 0.0_DP ) THEN
              units = celldm( 1 )
           ELSE IF ( a /= 0.0_dp ) THEN
              units = a / bohr_radius_angs
           ELSE
-             units = 1.0_DP
+            CALL errore ('cell_base_init', &
+                         'lattice parameter not specified',1) 
           END IF
+          ! following case is deprecated and should be removed
+        CASE ( 'none' ) 
+          ! cell_units is 'none' if nothing was specified
+          IF( celldm( 1 ) /= 0.0_DP ) THEN
+             units = celldm( 1 )
+             cell_units = 'alat'
+          ELSE IF ( a /= 0.0_dp ) THEN
+             units = a / bohr_radius_angs
+             cell_units = 'alat'
+          ELSE
+             units = 1.0_DP
+             cell_units = 'bohr'
+          END IF
+          !
+        CASE DEFAULT
+          CALL errore ('cell_base_init', &
+                       'unexpected cell_units '//TRIM(cell_units),1) 
      END SELECT
      !
      ! ... Beware the transpose operation between matrices ht and at!
@@ -174,6 +198,9 @@
         celldm(1) = a / bohr_radius_angs
         celldm(2) = b / a
         celldm(3) = c / a
+        IF ( (ABS(cosab) > 1.0_dp) .OR. (ABS(cosac) > 1.0_dp) .OR. &
+             (ABS(cosbc) > 1.0_dp) ) CALL errore ('cell_base_init',&
+                         'incorrect values for cosab, cosac, cosbc',1)
         !
         IF ( ibrav == 14 ) THEN
            !
@@ -530,6 +557,26 @@
         RETURN
       END SUBROUTINE pbcs_vectors
 
+!------------------------------------------------------------------------------!
+
+  SUBROUTINE set_h_ainv()
+    !
+    ! CP-PW compatibility: align CP arrays H and ainv to at and bg
+    !
+    IMPLICIT NONE
+    !
+    !write(stdout,*) 'alat=',alat
+    !write(stdout,*) 'at=',at
+    !write(stdout,*) 'bg=',bg
+    !
+    h(:,:) = at(:,:)*alat
+    !
+    ainv(1,:) = bg(:,1)/alat
+    ainv(2,:) = bg(:,2)/alat
+    ainv(3,:) = bg(:,3)/alat
+    !
+  END SUBROUTINE set_h_ainv
+
 !------------------------------------------------------------------------------!
 
   SUBROUTINE cell_dyn_init( trd_ht, rd_ht, wc_ , total_ions_mass , press_ , &
@@ -737,7 +784,6 @@
       WRITE( stdout, 340 ) omega
     END IF
 
-300 FORMAT( 3X, 'ibrav = ',I4)
 305 FORMAT( 3X, 'alat  = ',F14.8)
 310 FORMAT( 3X, 'a1    = ',3F14.8)
 320 FORMAT( 3X, 'a2    = ',3F14.8)
