@@ -650,7 +650,9 @@ CONTAINS
 
   SUBROUTINE pdsyevd_drv( tv, n, nb, s, lds, w, ortho_cntx )
      USE kinds,     ONLY : DP
-     USE mp_global,   ONLY: nproc_bgrp, me_bgrp, intra_bgrp_comm,root_bgrp,ortho_comm
+     USE mp_bands,  ONLY: nproc_bgrp, me_bgrp, intra_bgrp_comm, root_bgrp
+     USE mp_diag,   ONLY: ortho_comm
+     USE mp,        ONLY: mp_comm_free
 #ifdef __ELPA
      USE elpa1
 #endif
@@ -686,7 +688,7 @@ CONTAINS
         ALLOCATE( vv( SIZE( s, 1 ), SIZE( s, 2 ) ) )
         jobv = 'V'
      ELSE
-        CALL errore( ' pdsyevd_drv ', ' PDSYEVD does not compute eigenvalue only ', ABS( info ) )
+        CALL errore('pdsyevd_drv','PDSYEVD does not compute eigenvalue only',1)
      END IF
 
      CALL descinit( desch, n, n, nb, nb, 0, 0, ortho_cntx, SIZE( s, 1 ) , info )
@@ -700,10 +702,13 @@ CONTAINS
 
 #ifdef __ELPA
      CALL BLACS_Gridinfo(ortho_cntx,nprow, npcol, my_prow,my_pcol)
-     call GET_ELPA_ROW_COL_COMMS(ortho_comm, my_prow, my_pcol,mpi_comm_rows, mpi_comm_cols)
+     CALL GET_ELPA_ROW_COL_COMMS(ortho_comm, my_prow, my_pcol,mpi_comm_rows, mpi_comm_cols)
      CALL SOLVE_EVP_REAL(n,  n,   s, lds,    w,  vv, lds     ,nb  ,mpi_comm_rows, mpi_comm_cols)
-     s = vv
+     IF( tv )  s = vv
      IF( ALLOCATED( vv ) ) DEALLOCATE( vv )
+
+     CALL mp_comm_free ( mpi_comm_rows )
+     CALL mp_comm_free ( mpi_comm_cols )
 #else
      CALL PDSYEVD( jobv, 'L', n, s, 1, 1, desch, w, vv, 1, 1, desch, rtmp, lwork, itmp, liwork, info )
 
