@@ -49,7 +49,7 @@
         PUBLIC :: cgather_sym, cgather_smooth, cgather_custom
         PUBLIC :: cscatter_sym, cscatter_smooth, cscatter_custom
         PUBLIC :: gather_smooth, scatter_smooth
-        PUBLIC :: tg_gather
+        PUBLIC :: tg_gather, tg_cgather
 
 
 
@@ -209,8 +209,9 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
      !
      IF( isgn == 1 ) THEN
 
-!$omp parallel default(none) private(ip,ioff,i,mc,it,j) shared(dfft,nppx,sendsiz,me,f_in,f_aux)
-!$omp do
+!!$omp parallel default(none) private(ip,ioff,i,mc,it,j) shared(dfft,nppx,sendsiz,me,f_in,f_aux)
+!!$omp do
+
         DO ip = 1, dfft%nproc
            ioff = dfft%iss( ip )
            DO i = 1, dfft%nsp( ip )
@@ -221,8 +222,9 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
               ENDDO
            ENDDO
         ENDDO
-!$omp end do
-!$omp end parallel
+
+!!$omp end do
+!!$omp end parallel
 
      ELSE
 
@@ -234,8 +236,8 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
            nnp  = dfft%nnp
         ENDIF
         !
-!$omp parallel default(none) private(ip,ioff,i,mc,it,j,gproc,ii) shared(dfft,nppx,npp,nnp,sendsiz,use_tg_,f_in,f_aux)
-!$omp do
+!!$omp parallel default(none) private(ip,ioff,i,mc,it,j,gproc,ii) shared(dfft,nppx,npp,nnp,sendsiz,use_tg_,f_in,f_aux)
+!!$omp do
         DO ip = 1, dfft%nproc
 
            IF( use_tg_ ) THEN
@@ -263,8 +265,8 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
            ENDDO
            !
         ENDDO
-!$omp end do
-!$omp end parallel
+!!$omp end do
+!!$omp end parallel
 
      END IF
 
@@ -440,6 +442,8 @@ SUBROUTINE grid_gather( f_in, f_out )
   !
   CALL stop_clock( 'gather' )
   !
+#else
+  CALL errore('grid_gather', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
@@ -506,6 +510,8 @@ SUBROUTINE grid_scatter( f_in, f_out )
   !
   CALL stop_clock( 'scatter' )
   !
+#else
+  CALL errore('grid_scatter', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
@@ -565,6 +571,8 @@ SUBROUTINE cgather_sym( f_in, f_out )
   !
   CALL stop_clock( 'cgather' )
   !
+#else
+  CALL errore('cgather_sym', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
@@ -624,6 +632,8 @@ SUBROUTINE cgather_smooth ( f_in, f_out )
   !
   CALL stop_clock( 'gather' )
   !
+#else
+  CALL errore('cgather_smooth', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
@@ -684,6 +694,8 @@ SUBROUTINE cgather_custom ( f_in, f_out, dfftt )
   !
   CALL stop_clock( 'gather' )
   !
+#else
+  CALL errore('cgather_custom', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
@@ -745,6 +757,8 @@ SUBROUTINE cscatter_sym( f_in, f_out )
   !
   CALL stop_clock( 'cscatter_sym' )
   !
+#else
+  CALL errore('cscatter_sym', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
@@ -805,6 +819,8 @@ SUBROUTINE cscatter_smooth( f_in, f_out )
   !
   CALL stop_clock( 'scatter' )
   !
+#else
+  CALL errore('cscatter_smooth', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
@@ -866,6 +882,8 @@ SUBROUTINE cscatter_custom( f_in, f_out, dfftt )
   !
   CALL stop_clock( 'scatter' )
   !
+#else
+  CALL errore('cscatter_custom', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
@@ -925,6 +943,8 @@ SUBROUTINE gather_smooth ( f_in, f_out )
   !
   CALL stop_clock( 'gather' )
   !
+#else
+  CALL errore('gather_smooth', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
@@ -985,6 +1005,8 @@ SUBROUTINE scatter_smooth( f_in, f_out )
   !
   CALL stop_clock( 'scatter' )
   !
+#else
+  CALL errore('scatter_smooth', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
@@ -1051,6 +1073,73 @@ SUBROUTINE tg_gather( dffts, v, tg_v )
 #endif
 
 END SUBROUTINE tg_gather
+!
+!  Complex version of previous routine
+!
+SUBROUTINE tg_cgather( dffts, v, tg_v )
+   !
+   USE parallel_include
+   !
+   USE fft_types,      ONLY : fft_dlay_descriptor
+
+   ! T.G.
+   ! NOGRP:      Number of processors per orbital task group
+
+   IMPLICIT NONE
+
+   TYPE(fft_dlay_descriptor), INTENT(in) :: dffts
+
+   COMPLEX(DP) :: v(:)
+   COMPLEX(DP) :: tg_v(:)
+
+   INTEGER :: nsiz, i, ierr, nsiz_tg
+   INTEGER :: recv_cnt( dffts%nogrp ), recv_displ( dffts%nogrp )
+
+   nsiz_tg = dffts%tg_nnr * dffts%nogrp
+
+   IF( size( tg_v ) < nsiz_tg ) &
+      CALL errore( ' tg_gather ', ' tg_v too small ', ( nsiz_tg - size( tg_v ) ) )
+
+   nsiz = dffts%npp( dffts%mype+1 ) * dffts%nr1x * dffts%nr2x
+
+   IF( size( v ) < nsiz ) &
+      CALL errore( ' tg_gather ', ' v too small ',  ( nsiz - size( v ) ) )
+
+   !
+   !  The potential in v is distributed across all processors
+   !  We need to redistribute it so that it is completely contained in the
+   !  processors of an orbital TASK-GROUP
+   !
+   recv_cnt(1)   = dffts%npp( dffts%nolist(1) + 1 ) * dffts%nr1x * dffts%nr2x
+   recv_displ(1) = 0
+   DO i = 2, dffts%nogrp
+      recv_cnt(i) = dffts%npp( dffts%nolist(i) + 1 ) * dffts%nr1x * dffts%nr2x
+      recv_displ(i) = recv_displ(i-1) + recv_cnt(i-1)
+   ENDDO
+
+   ! clean only elements that will not be overwritten
+   !
+   DO i = recv_displ(dffts%nogrp) + recv_cnt( dffts%nogrp ) + 1, size( tg_v )
+      tg_v( i ) = (0.0d0,0.0d0)
+   ENDDO
+   !
+   ! The quantities are complex, multiply the cunters by 2 and gather
+   ! real numbers
+   !
+   nsiz = 2 * nsiz
+   recv_cnt = 2 * recv_cnt
+   recv_displ = 2 * recv_displ
+#if defined (__MPI)
+
+   CALL MPI_Allgatherv( v(1), nsiz, MPI_DOUBLE_PRECISION, &
+        tg_v(1), recv_cnt, recv_displ, MPI_DOUBLE_PRECISION, dffts%ogrp_comm, IERR)
+
+   IF( ierr /= 0 ) &
+      CALL errore( ' tg_cgather ', ' MPI_Allgatherv ', abs( ierr ) )
+
+#endif
+
+END SUBROUTINE tg_cgather
 
 !=----------------------------------------------------------------------=!
    END MODULE fft_base
