@@ -20,20 +20,108 @@ MODULE generate_function
  
 CONTAINS
 !----------------------------------------------------------------------
-      SUBROUTINE generate_gaussian( nnr, charge, spread, pos, rho )
+      SUBROUTINE planar_average( nnr, naxis, axis, shift, reverse, f, f1d )
 !----------------------------------------------------------------------
       !
       USE kinds,            ONLY : DP
-      USE constants,        ONLY : sqrtpi
-      USE cell_base,        ONLY : at, bg, alat
+      USE io_global,        ONLY : stdout
       USE fft_base,         ONLY : dfftp
-      USE mp_global,        ONLY : me_bgrp, intra_bgrp_comm
+      USE mp,               ONLY : mp_sum
+      USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
       !
       IMPLICIT NONE
       !
       ! ... Declares variables
       !
-      INTEGER, INTENT(IN)       :: nnr
+      INTEGER, INTENT(IN)       :: nnr, naxis, axis, shift
+      LOGICAL, INTENT(IN)       :: reverse
+      REAL( DP ), INTENT(INOUT) :: f( nnr )
+      REAL( DP ), INTENT(INOUT) :: f1d( naxis )
+      !
+      ! ... Local variables
+      !
+      INTEGER                   :: i, j, k, ir, ir_end
+      INTEGER                   :: index, index0, narea
+      !
+      REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
+      !
+      inv_nr1 = 1.D0 / DBLE( dfftp%nr1 )
+      inv_nr2 = 1.D0 / DBLE( dfftp%nr2 )
+      inv_nr3 = 1.D0 / DBLE( dfftp%nr3 )
+      !
+      index0 = 0
+      ir_end = nnr
+      !
+#if defined (__MPI)
+      DO i = 1, me_bgrp
+        index0 = index0 + dfftp%nr1x*dfftp%nr2x*dfftp%npp(i)
+      END DO
+      ir_end = MIN(nnr,dfftp%nr1x*dfftp%nr2x*dfftp%npp(me_bgrp+1))
+#endif  
+      !
+      narea = dfftp%nr1*dfftp%nr2*dfftp%nr3 / naxis
+      !
+      IF ( reverse ) THEN
+        f = 0.D0
+      ELSE
+        f1d = 0.D0
+      END IF
+      !
+      DO ir = 1, ir_end
+         !
+         ! ... find the index along the selected axis
+         !
+         i = index0 + ir - 1
+         index = i / (dfftp%nr1x*dfftp%nr2x)
+         IF ( axis .LT. 3 ) THEN 
+           i = i - (dfftp%nr1x*dfftp%nr2x)*index
+           index = i / dfftp%nr1x
+         END IF 
+         IF ( axis .EQ. 1 ) index = i - dfftp%nr1x*index
+         !
+         index = index + 1 + shift
+         !
+         IF ( index .GT. naxis ) THEN 
+           index = index - naxis
+         ELSE IF (index .LE. 0 ) THEN
+           index = index + naxis
+         ENDIF           
+         !
+         IF ( reverse ) THEN
+           f(ir) = f1d(index)
+         ELSE
+           f1d(index) = f1d(index) + f(ir)
+         END IF 
+         !      
+      END DO
+      !
+      IF ( .NOT. reverse ) THEN 
+        CALL mp_sum( f1d(:), intra_bgrp_comm )
+        f1d = f1d / DBLE(narea)
+      END IF
+      !
+      RETURN
+      !
+!----------------------------------------------------------------------
+      END SUBROUTINE planar_average
+!----------------------------------------------------------------------
+!----------------------------------------------------------------------
+      SUBROUTINE generate_gaussian( nnr, dim, axis, charge, spread, pos, rho )
+!----------------------------------------------------------------------
+      !
+      USE kinds,            ONLY : DP
+      USE constants,        ONLY : sqrtpi
+      USE io_global,        ONLY : stdout
+      USE cell_base,        ONLY : at, bg, alat, omega
+      USE fft_base,         ONLY : dfftp
+      USE mp,               ONLY : mp_sum
+      USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
+      !
+      IMPLICIT NONE
+      !
+      ! ... Declares variables
+      !
+      INTEGER, INTENT(IN)       :: nnr, dim, axis
       REAL( DP ), INTENT(IN)    :: charge, spread
       REAL( DP ), INTENT(IN)    :: pos( 3 )
       REAL( DP ), INTENT(INOUT) :: rho( nnr )
@@ -41,10 +129,10 @@ CONTAINS
       ! ... Local variables
       !
       INTEGER                   :: i, j, k, ir, ir_end, ip
-      INTEGER                   :: index, index0
+      INTEGER                   :: index0
       !
       REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
-      REAL( DP )                :: scale, spr2, dist
+      REAL( DP )                :: scale, spr2, dist, length
       REAL( DP )                :: r( 3 ), s( 3 )
       REAL( DP ), ALLOCATABLE   :: rholocal ( : )
       !
@@ -66,7 +154,19 @@ CONTAINS
       ir_end = nnr
 #endif  
       !
-      scale = charge / ( sqrtpi * spread )**3
+      IF (axis.LT.1.OR.axis.GT.3) &
+           WRITE(stdout,*)'WARNING: wrong axis in generate_gaussian'
+      IF ( dim .EQ. 0 ) THEN
+        scale = charge / ( sqrtpi * spread )**3
+      ELSE IF ( dim .EQ. 1 ) THEN
+        length = at(axis,axis) * alat
+        scale = charge / length / ( sqrtpi * spread )**2
+      ELSE IF ( dim .EQ. 2 ) THEN
+        length = at(axis,axis) * alat
+        scale = charge * length / omega / ( sqrtpi * spread )
+      ELSE 
+        WRITE(stdout,*)'WARNING: wrong dim in generate_gaussian'
+      ENDIF
       spr2 = ( spread / alat )**2
       ALLOCATE( rholocal( nnr ) )
       rholocal = 0.D0
@@ -75,12 +175,11 @@ CONTAINS
          !
          ! ... three dimensional indexes
          !
-         index = index0 + ir - 1
-         k     = index / (dfftp%nr1x*dfftp%nr2x)
-         index = index - (dfftp%nr1x*dfftp%nr2x)*k
-         j     = index / dfftp%nr1x
-         index = index - dfftp%nr1x*j
-         i     = index
+         i = index0 + ir - 1
+         k = i / (dfftp%nr1x*dfftp%nr2x)
+         i = i - (dfftp%nr1x*dfftp%nr2x)*k
+         j = i / dfftp%nr1x
+         i = i - dfftp%nr1x*j
          !
          DO ip = 1, 3
             r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -90,6 +189,16 @@ CONTAINS
          !
          r(:) = pos(:) - r(:) 
          !
+         !  ... possibly 2D or 1D gaussians
+         !
+         IF ( dim .EQ. 1) THEN
+           r(axis) = 0.D0
+         ELSE IF ( dim .EQ. 2 ) THEN
+           DO i = 1, 3
+             IF ( i .NE. axis ) r(i) = 0.D0
+           ENDDO
+         END IF
+         !
          ! ... minimum image convention
          !
          s(:) = MATMUL( r(:), bg(:,:) )
@@ -111,20 +220,21 @@ CONTAINS
       END SUBROUTINE generate_gaussian
 !----------------------------------------------------------------------
 !----------------------------------------------------------------------
-      SUBROUTINE generate_gradgaussian( nnr, charge, spread, pos, gradrho )
+      SUBROUTINE generate_gradgaussian( nnr, dim, axis, charge, spread, pos, gradrho )
 !----------------------------------------------------------------------
       !
-      USE kinds,            ONLY: DP
-      USE constants,        ONLY: sqrtpi
-      USE cell_base,        ONLY : at, bg, alat
+      USE kinds,            ONLY : DP
+      USE constants,        ONLY : sqrtpi
+      USE io_global,        ONLY : stdout
+      USE cell_base,        ONLY : at, bg, alat, omega
       USE fft_base,         ONLY : dfftp
-      USE mp_global,        ONLY : me_bgrp, intra_bgrp_comm
+      USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
       !
       IMPLICIT NONE
       !
       ! ... Declares variables
       !
-      INTEGER, INTENT(IN)       :: nnr
+      INTEGER, INTENT(IN)       :: nnr, dim, axis
       REAL( DP ), INTENT(IN)    :: charge, spread
       REAL( DP ), INTENT(IN)    :: pos( 3 )
       REAL( DP ), INTENT(INOUT) :: gradrho( 3, nnr )
@@ -132,10 +242,10 @@ CONTAINS
       ! ... Local variables
       !
       INTEGER                   :: i, j, k, ir, ir_end, ip
-      INTEGER                   :: index, index0
+      INTEGER                   :: index0
       !
       REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
-      REAL( DP )                :: scale, spr2, dist
+      REAL( DP )                :: scale, spr2, dist, length
       REAL( DP )                :: r( 3 ), s( 3 )
       REAL( DP ), ALLOCATABLE   :: gradrholocal ( :, : )
       !
@@ -157,7 +267,19 @@ CONTAINS
       ir_end = nnr
 #endif  
       !
-      scale = 2.d0 * charge / sqrtpi**3 / spread**5
+      IF (axis.LT.1.OR.axis.GT.3) &
+           WRITE(stdout,*)'WARNING: wrong axis in generate_gaussian'
+      IF ( dim .EQ. 0 ) THEN
+        scale = charge / ( sqrtpi * spread )**3
+      ELSE IF ( dim .EQ. 1 ) THEN
+        length = at(axis,axis) * alat
+        scale = charge / length / ( sqrtpi * spread )**2
+      ELSE IF ( dim .EQ. 2 ) THEN
+        length = at(axis,axis) * alat
+        scale = charge * length / omega / ( sqrtpi * spread )
+      ELSE 
+        WRITE(stdout,*)'WARNING: wrong dim in generate_gaussian'
+      ENDIF
       spr2 = ( spread / alat )**2
       ALLOCATE( gradrholocal( 3, nnr ) )
       gradrholocal = 0.D0
@@ -166,12 +288,11 @@ CONTAINS
          !
          ! ... three dimensional indexes
          !
-         index = index0 + ir - 1
-         k     = index / (dfftp%nr1x*dfftp%nr2x)
-         index = index - (dfftp%nr1x*dfftp%nr2x)*k
-         j     = index / dfftp%nr1x
-         index = index - dfftp%nr1x*j
-         i     = index
+         i = index0 + ir - 1
+         k = i / (dfftp%nr1x*dfftp%nr2x)
+         i = i - (dfftp%nr1x*dfftp%nr2x)*k
+         j = i / dfftp%nr1x
+         i = i - dfftp%nr1x*j
          !
          DO ip = 1, 3
             r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -181,6 +302,16 @@ CONTAINS
          !
          r(:) = pos(:) - r(:) 
          !
+         !  ... possibly 2D or 1D gaussians
+         !
+         IF ( dim .EQ. 1) THEN
+           r(axis) = 0.D0
+         ELSE IF ( dim .EQ. 2 ) THEN
+           DO i = 1, 3
+             IF ( i .NE. axis ) r(i) = 0.D0
+           ENDDO
+         END IF
+         !
          ! ... minimum image convention
          !
          s(:) = MATMUL( r(:), bg(:,:) )
@@ -208,7 +339,7 @@ CONTAINS
       USE kinds,            ONLY : DP
       USE cell_base,        ONLY : at, bg, alat
       USE fft_base,         ONLY : dfftp
-      USE mp_global,        ONLY : me_bgrp, intra_bgrp_comm
+      USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
       !
       IMPLICIT NONE
       !
@@ -222,7 +353,7 @@ CONTAINS
       ! ... Local variables
       !
       INTEGER                   :: i, j, k, ir, ir_end, ip
-      INTEGER                   :: index, index0
+      INTEGER                   :: index0
       !
       REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
       REAL( DP )                :: dist, arg
@@ -255,12 +386,11 @@ CONTAINS
          !
          ! ... three dimensional indexes
          !
-         index = index0 + ir - 1
-         k     = index / (dfftp%nr1x*dfftp%nr2x)
-         index = index - (dfftp%nr1x*dfftp%nr2x)*k
-         j     = index / dfftp%nr1x
-         index = index - dfftp%nr1x*j
-         i     = index
+         i = index0 + ir - 1
+         k = i / (dfftp%nr1x*dfftp%nr2x)
+         i = i - (dfftp%nr1x*dfftp%nr2x)*k
+         j = i / dfftp%nr1x
+         i = i - dfftp%nr1x*j
          r = 0.D0
          !
          DO ip = 1, 3
@@ -303,7 +433,7 @@ CONTAINS
       USE kinds,            ONLY : DP
       USE cell_base,        ONLY : at, bg, alat
       USE fft_base,         ONLY : dfftp
-      USE mp_global,        ONLY : me_bgrp, intra_bgrp_comm
+      USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
       !
       IMPLICIT NONE
       !
@@ -317,7 +447,7 @@ CONTAINS
       ! ... Local variables
       !
       INTEGER                   :: i, j, k, ir, ir_end, ip
-      INTEGER                   :: index, index0
+      INTEGER                   :: index0
       !
       REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
       REAL( DP )                :: dist, arg
@@ -350,12 +480,11 @@ CONTAINS
          !
          ! ... three dimensional indexes
          !
-         index = index0 + ir - 1
-         k     = index / (dfftp%nr1x*dfftp%nr2x)
-         index = index - (dfftp%nr1x*dfftp%nr2x)*k
-         j     = index / dfftp%nr1x
-         index = index - dfftp%nr1x*j
-         i     = index
+         i = index0 + ir - 1
+         k = i / (dfftp%nr1x*dfftp%nr2x)
+         i = i - (dfftp%nr1x*dfftp%nr2x)*k
+         j = i / dfftp%nr1x
+         i = i - dfftp%nr1x*j
          !
          DO ip = 1, 3
             r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -395,14 +524,14 @@ CONTAINS
    USE kinds,            ONLY : DP
    USE cell_base,        ONLY : at, bg, alat
    USE fft_base,         ONLY : dfftp
-   USE mp_global,        ONLY : me_bgrp, intra_bgrp_comm
+   USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
   !
   INTEGER, INTENT(IN) :: nnr
   INTEGER, INTENT(IN) :: icor
   REAL(DP), INTENT(IN) :: pos(3)
   REAL(DP), INTENT(OUT) :: axis( dfftp%nnr )
   !
-  INTEGER  :: i, j, k, ir, ir_end, ip, index, index0
+  INTEGER  :: i, j, k, ir, ir_end, ip, index0
   REAL(DP) :: inv_nr1, inv_nr2, inv_nr3
   REAL(DP) :: r(3), s(3)
   !
@@ -428,12 +557,11 @@ CONTAINS
      !
      ! ... three dimensional indexes
      !
-     index = index0 + ir - 1
-     k     = index / (dfftp%nr1x*dfftp%nr2x)
-     index = index - (dfftp%nr1x*dfftp%nr2x)*k
-     j     = index / dfftp%nr1x
-     index = index - dfftp%nr1x*j
-     i     = index
+     i = index0 + ir - 1
+     k = i / (dfftp%nr1x*dfftp%nr2x)
+     i = i - (dfftp%nr1x*dfftp%nr2x)*k
+     j = i / dfftp%nr1x
+     i = i - dfftp%nr1x*j
      !
      DO ip = 1, 3
         r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -468,13 +596,13 @@ CONTAINS
    USE kinds,            ONLY : DP
    USE cell_base,        ONLY : at, bg, alat
    USE fft_base,         ONLY : dfftp
-   USE mp_global,        ONLY : me_bgrp, intra_bgrp_comm
+   USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
   !
   INTEGER, INTENT(IN) :: nnr
   REAL(DP), INTENT(IN) :: pos(3)
   REAL(DP), INTENT(OUT) :: distance( 3, dfftp%nnr )
   !
-  INTEGER  :: i, j, k, ir, ir_end, ip, index, index0
+  INTEGER  :: i, j, k, ir, ir_end, ip, index0
   REAL(DP) :: inv_nr1, inv_nr2, inv_nr3
   REAL(DP) :: r(3), s(3)
   !
@@ -500,12 +628,11 @@ CONTAINS
      !
      ! ... three dimensional indexes
      !
-     index = index0 + ir - 1
-     k     = index / (dfftp%nr1x*dfftp%nr2x)
-     index = index - (dfftp%nr1x*dfftp%nr2x)*k
-     j     = index / dfftp%nr1x
-     index = index - dfftp%nr1x*j
-     i     = index
+     i = index0 + ir - 1
+     k = i / (dfftp%nr1x*dfftp%nr2x)
+     i = i - (dfftp%nr1x*dfftp%nr2x)*k
+     j = i / dfftp%nr1x
+     i = i - dfftp%nr1x*j
      !
      DO ip = 1, 3
         r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
