@@ -211,6 +211,12 @@ MODULE input_parameters
         LOGICAL :: lberry = .false.
           ! if .TRUE., use modern theory of the polarization
 
+        LOGICAL :: lcalc_z2 = .false.
+          ! if .TRUE., calculate Z2 without inversion symmetry
+
+        REAL(DP) :: z2_m_threshold = 0.8d0, z2_z_threshold = 0.05d0
+          ! threshold for realizing the parallel transport gauge
+
         INTEGER :: gdir = 0
           ! G-vector for polarization calculation ( related to lberry )
           ! only used in PW
@@ -249,8 +255,12 @@ MODULE input_parameters
           ! if true symmetry in scf run is neglected for RPA Ec calculation
           ! 
 
+        LOGICAL :: tqmmm = .FALSE.    ! QM/MM coupling. enabled if .true.
+
         CHARACTER(len=256) :: vdw_table_name = ' '
 
+        CHARACTER(len=10) :: point_label_type='SC'
+
         CHARACTER(len=80) :: memory = 'default' 
           ! controls memory usage 
         CHARACTER(len=80) :: memory_allowed(3)
@@ -260,23 +270,14 @@ MODULE input_parameters
           ! if memory = 'large' then QE tries to use (when implemented) algorithms using more memory
           !                     to enhance performance.
 
-#if defined (__MS2)
-        LOGICAL :: MS2_enabled = .false.       ! Enable the shared memory exchange in MS2
-        CHARACTER(len=256) :: MS2_handler = ' '! Name for the shared memory handler in MS2
-#endif
-
         NAMELIST / control / title, calculation, verbosity, restart_mode, &
           nstep, iprint, isave, tstress, tprnfor, dt, ndr, ndw, outdir,   &
           prefix, wfcdir, max_seconds, ekin_conv_thr, etot_conv_thr,      &
           forc_conv_thr, pseudo_dir, disk_io, tefield, dipfield, lberry,  &
           gdir, nppstr, wf_collect, printwfc, lelfield, nberrycyc, refg,  &
           tefield2, saverho, tabps, lkpoint_dir, use_wannier, lecrpa,     &
-          vdw_table_name, lorbm, memory
-
-
-#if defined ( __MS2)
-        NAMELIST / control / MS2_enabled, MS2_handler
-#endif
+          tqmmm, vdw_table_name, lorbm, memory, point_label_type,         &
+          lcalc_z2, z2_m_threshold, z2_z_threshold
 
 !
 !=----------------------------------------------------------------------------=!
@@ -401,11 +402,14 @@ MODULE input_parameters
         REAL(DP) :: alpha_pen(10) = 0.0_DP
 
           ! next group of variables PWSCF ONLY
-          !
-        INTEGER  :: nqx1 = 1, nqx2 = 1, nqx3=1
+          ! 
           !
         REAL(DP) :: exx_fraction = -1.0_DP      ! if negative, use defaults
         REAL(DP) :: screening_parameter = -1.0_DP
+        INTEGER  :: nqx1 = 0, nqx2 = 0, nqx3=0  ! use the same values as nk1, nk2, nk3
+        !gau-pbe in
+        REAL(DP) :: gau_parameter = -1.0_DP
+        !gau-pbe out
           !
         CHARACTER(len=80) :: exxdiv_treatment = 'gygi-baldereschi'
           ! define how ro cure the Coulomb divergence in EXX
@@ -457,20 +461,43 @@ MODULE input_parameters
         LOGICAL :: one_atom_occupations=.false.
 
         CHARACTER(len=80) :: assume_isolated = 'none'
-          ! used to define corrections for isolated systems
-          ! other possibilities:
-          !  'makov-payne', 'martyna-tuckerman`, `dcc`, 'esm'
-!
+          ! possible corrections for isolated systems:
+          !  'none', 'makov-payne', 'martyna-tuckerman', 'esm'
+          ! plus ENVIRON-specific:
+          !  'slabx', 'slaby', 'slabz', 'pcc'
+
+        CHARACTER(len=80) :: vdw_corr = 'none'
+          ! semi-empirical van der Waals corrections
+          ! (not to be confused with nonlocal functionals,
+          !  specified in input_dft!). Default is 'none', allowed values:
+          !  'dft-d' or 'grimme-d2' [S.Grimme, J.Comp.Chem. 27, 1787 (2006)]
+          !  'ts', 'ts-vdW', 'tkatchenko-scheffler'
+          !  (Tkatchenko & Scheffler, Phys. Rev. Lett. 102, 073005 (2009))
+          !  'xdm' (Otero de la Roza and Johnson, J. Chem. Phys. 136 (2012) 174109)
+
         LOGICAL   :: london = .false.
-          ! if .true. compute semi-empirical dispersion term ( C6_ij / R_ij**6 )
-          ! other DFT-D parameters ( see PW/mm_dispersion.f90 )
-        REAL ( DP ) :: london_s6   =   0.75_DP , & ! default global scaling parameter for PBE
+          ! OBSOLESCENT: same as vdw_corr='grimme-d2'
+          ! other DFT-D parameters ( see Modules/mm_dispersion.f90 )
+          ! london_s6 = default global scaling parameter for PBE
+        REAL ( DP ) :: london_s6   =   0.75_DP , &
                        london_rcut = 200.00_DP
-#ifdef __ENVIRON
-!
-        LOGICAL   :: do_environ = .false.
-#endif
-!
+
+        LOGICAL   :: ts_vdw = .false.
+          ! OBSOLESCENT: same as vdw_corr='Tkatchenko-Scheffler'
+        LOGICAL :: ts_vdw_isolated = .FALSE.
+          ! if .TRUE., TS-vdW correction for isolated system
+          ! if .FALSE., TS-vdW correction for periodic system
+        REAL(DP) :: ts_vdw_econv_thr = 1.0E-6_DP
+          ! convergence criterion for TS-vdW energy for periodic system 
+          !
+
+        LOGICAL :: xdm = .FALSE.
+          ! OBSOLESCENT: same as vdw_corr='xdm'
+        REAL(DP) :: xdm_a1 = 0.6836_DP
+        REAL(DP) :: xdm_a2 = 1.5045_DP
+          ! xdm_a1 and xdm_a2 -- parameters for the BJ damping function
+          ! The default values are for the b86bpbe functional.
+          !
         CHARACTER(LEN=3) :: esm_bc = 'pbc'
           ! 'pbc': ordinary calculation with periodic boundary conditions
           ! 'bc1': vacuum-slab-vacuum
@@ -507,176 +534,16 @@ MODULE input_parameters
              nqx1, nqx2, nqx3, ecutfock,                                      &
              exxdiv_treatment, x_gamma_extrapolation, yukawa, ecutvcut,       &
              exx_fraction, screening_parameter,                               &
-#ifdef __ENVIRON
-             do_environ,                                                      &
-#endif
              noncolin, lspinorb, starting_spin_angle, lambda, angle1, angle2, &
              report,              &
              constrained_magnetization, B_field, fixed_magnetization,         &
              sic, sic_epsilon, force_pairing, sic_alpha,                      &
-             tot_charge, tot_magnetization,                                   &
-             spline_ps, one_atom_occupations, london, london_s6, london_rcut, &
+             tot_charge, tot_magnetization, spline_ps, one_atom_occupations,  &
+             vdw_corr, london, london_s6, london_rcut,                        &
+             ts_vdw, ts_vdw_isolated, ts_vdw_econv_thr,                       &
+             xdm, xdm_a1, xdm_a2,                                             &
              step_pen, A_pen, sigma_pen, alpha_pen, no_t_rev,                 &
              esm_bc, esm_efield, esm_w, esm_nfit, esm_debug, esm_debug_gpmax
-#ifdef __ENVIRON
-!
-!=----------------------------------------------------------------------------=!
-!  ENVIRON Namelist Input Parameters
-!=----------------------------------------------------------------------------=!
-!
-! Global parameters
-!
-        INTEGER  :: verbose = 0
-        ! verbosity  0: only prints summary of polarization charge calculation; 
-        !    1: prints an extra file with details of iterative convergence;
-        !    2: prints 3D cube files of physical properties
-        REAL(DP) :: environ_thr = 1.d-1
-        ! how early in scf should the corrective pot start being calculated
-        CHARACTER( LEN = 256 ) :: environ_type = 'input'
-        ! keyword to set up all the environment parameters at once to a specific set
-        ! vacuum = all the flags are off (perm=1.d0, surf=0.0, pres=0.0)
-        ! water = parameters optimized for water solutions in Andreussi et al. 
-        !         J. Chem. Phys. 136, 064102 (perm=78, surf=50, pres=-0.35)
-        ! input = do not use any predefined set, use paramters from input
-!
-! Switching function parameters
-!
-        REAL(DP) :: stype = 1
-        ! type of switching functions used in the solvation models
-        !    0: original Fattebert-Gygi
-        !    1: ultrasoft dielectric function as defined in Andreussi et al.
-        REAL(DP) :: rhomax = 0.005
-        ! first parameter of the sw function, roughly corresponding 
-        ! to the density threshold of the solvation model
-        REAL(DP) :: rhomin = 0.0001
-        ! second parameter of the sw function when stype=1
-        REAL(DP) :: tbeta = 4.8
-        ! second parameter of the sw function when stype=0
-!
-! Dielectric solvent parameters
-!
-        REAL(DP) :: env_static_permittivity = 1.D0
-        ! static dielectric permittivity of the solvation model. If set equal
-        ! to one (=vacuum) no dielectric effects
-        CHARACTER( LEN = 256 ) :: eps_mode = 'electronic'
-        !  eps_mode method for calculating the density that sets 
-        !  the dielectric constant
-        !  electronic = dielectric depends self-consist. on electronic density
-        !  ionic = dielectric defined on a fictitious ionic density, generated
-        !          as the sum of exponential functions centered on atomic 
-        !          positions of width specified in input by solvationrad(ityp)
-        !  full  = similar to electronic, but an extra density is added to 
-        !          represent the core electrons and the nuclei. This extra 
-        !          density is defined as the sum of gaussian functions centered
-        !          on atomic positions of width equal to atomicspread(ityp)
-        REAL(DP) :: solvationrad(nsx) = 3.D0
-        ! solvationrad radius of the solvation shell for each species when the
-        ! ionic dielectric function is adopted
-        REAL(DP) :: atomicspread(nsx) = 0.5D0
-        ! gaussian spreads of the atomic density of charge
-        LOGICAL :: add_jellium = .false.
-        ! depending on periodic boundary corrections, one may need to explicitly
-        ! polarize the compensatinig jellium background
-!
-! Numerical differentiators paramters
-!
-        INTEGER  :: ifdtype = 1 
-        ! type of numerical differentiator: 1=central differences, 
-        ! 2=low-noise lanczos (m=2), 3=low-noise lanczos (m=4), 
-        ! 4=smooth noise-robust (n=2), 5=smooth noise-robust (n=4)
-        INTEGER  :: nfdpoint = 1
-        ! number of points used in the numerical differentiator 
-        ! N = 2*nfdpoint+1
-!
-! Iterative solver parameters
-!
-        CHARACTER( LEN=256 ) :: mixtype = 'linear'
-        ! mixing method for iterative calculation of polarization charges
-        ! 'linear', 'anderson', 'diis', 'broyden'
-        REAL(DP) :: mixrhopol = 0.5D0
-        ! mixing param to be used in iter calculation of polarization charges
-        REAL(DP) :: tolrhopol = 1.D-10
-        ! convergence threshold for polarization charges in iterative procedure
-        INTEGER :: ndiis=1
-        ! order of DIIS interpolation of iterative polarization charge
-!
-! Cavitation energy parameters
-!
-        REAL(DP) :: env_surface_tension = 0.D0
-        ! solvent surface tension, if equal to zero no cavitation term 
-        REAL(DP) :: delta = 0.00001D0
-        ! finite difference parameter to compute the molecular surface
-!
-! PV energy parameters
-!
-        REAL(DP) :: env_pressure = 0.D0
-        ! external pressure for PV energy, if equal to zero no pressure term 
-!
-! Ionic countercharge parameters
-!
-        REAL(DP) :: env_ioncc_concentration = 0.D0
-        ! molar concentration of ionic countercharge (M=mol/L)
-        REAL(DP) :: zion = 1.D0
-        ! valence of ionic countercharge
-        REAL(DP) :: rhopb = 0.0001D0
-        ! density threshold for the onset of ionic countercharge
-        REAL(DP) :: solvent_temperature = 300.D0
-        ! temperature of the solution
-
-        NAMELIST / environ /                                           &
-             verbose, environ_thr, environ_type,                       &
-             stype, rhomax, rhomin, tbeta,                             &
-             env_static_permittivity, eps_mode,                        &
-             solvationrad, atomicspread, add_jellium,                  &
-             ifdtype, nfdpoint,                                        &
-             mixtype, ndiis, mixrhopol, tolrhopol,                     &
-             env_surface_tension, delta,                               &
-             env_pressure,                                             &
-             env_ioncc_concentration, zion, rhopb,                     &
-             solvent_temperature
-#endif
-!
-!=----------------------------------------------------------------------------=!
-!  EE Namelist Input Parameters
-!=----------------------------------------------------------------------------=!
-!
-! kinetic energy cutoff for the coarse (MultiGrid) grid
-        REAL(DP) :: ecutcoarse = 100.0d0
-! amount of "new" correction introduced when mixing
-        REAL(DP) :: mixing_charge_compensation = 1.0
-! error tolerance for the multigrid solver
-        REAL(DP) :: errtol = 1.d-22
-! how early in scf itarations should the corrective pot start being calculated
-        REAL(DP) :: comp_thr = 1.d-2
-! nlev number of grid levels in the multigrid solver
-        INTEGER :: nlev = 2
-! itmax maximum number of iterations in the multigrid solver
-        INTEGER :: itmax = 1000
-! whichbc 0 if aperiodic
-        INTEGER :: whichbc(3) = 0
-! sets after how many scf cycles the corrective potential should be calculated
-        INTEGER :: n_charge_compensation = 5
-!
-        INTEGER :: ncompx = 1
-        INTEGER :: ncompy = 1
-        INTEGER :: ncompz = 1
-          ! ONLY PWSCF
-!
-        INTEGER :: mr1 = 0
-        INTEGER :: mr2 = 0
-        INTEGER :: mr3 = 0
-
-        REAL(DP) :: cellmin( 3 ) = 0.D0
-          ! ONLY PWSCF
-
-        REAL(DP) :: cellmax( 3 ) = 1.D0
-
-        NAMELIST / ee / comp_thr,    &
-             ncompx,n_charge_compensation,              &
-             ncompy, ncompz,mixing_charge_compensation, &
-             mr1, mr2, mr3, ecutcoarse,                 &
-             errtol, nlev, itmax, whichbc,              &
-             cellmin, cellmax
 
 !=----------------------------------------------------------------------------=!
 !  ELECTRONS Namelist Input Parameters
@@ -1022,9 +889,10 @@ MODULE input_parameters
 
         CHARACTER(len=80) :: ion_dynamics = 'none'
           ! set how ions should be moved
-        CHARACTER(len=80) :: ion_dynamics_allowed(8)
+        CHARACTER(len=80) :: ion_dynamics_allowed(9)
         DATA ion_dynamics_allowed / 'none', 'sd', 'cg', 'langevin', &
-                                    'damp', 'verlet', 'bfgs', 'beeman' /
+                                    'damp', 'verlet', 'bfgs', 'beeman',& 
+                                    'langevin-smc' /
 
         REAL(DP) :: ion_radius(nsx) = 0.5_DP
           ! pseudo-atomic radius of the i-th atomic species
@@ -1427,10 +1295,6 @@ MODULE input_parameters
           ! atomic_positions = 'bohr' | 'angstrong' | 'crystal' | 'alat'
           ! select the units for the atomic positions being read from stdin
 
-        !
-        ! ... variable added for NEB  ( C.S. 17/10/2003 )
-        !
-        !
 !
 !    ION_VELOCITIES
 !
@@ -1481,7 +1345,7 @@ MODULE input_parameters
 !    CELL_PARAMETERS
 !
        REAL(DP) :: rd_ht(3,3) = 0.0_DP
-       CHARACTER(len=80) :: cell_units = 'alat'
+       CHARACTER(len=80) :: cell_units = 'none'
        LOGICAL   :: trd_ht = .false.
 
 !
@@ -1506,14 +1370,6 @@ MODULE input_parameters
         ! logical mask used to specify which kohn sham orbital should be
         ! written to files 'KS.'
 
-!
-!   CLIMBING_IMAGES
-!
-      !
-      ! ... variable added for NEB  ( C.S. 20/11/2003 )
-      !
-      LOGICAL, ALLOCATABLE :: climbing( : )
-
 !
 !   PLOT_WANNIER
 !
@@ -1533,8 +1389,36 @@ MODULE input_parameters
       LOGICAL :: xmloutput = .false.
       ! if .true. PW produce an xml output
 CONTAINS
-
+!
+!----------------------------------------------------------------------------
+SUBROUTINE reset_input_checks()
+  !-----------------------------------------------------------------------------
+  !
+  ! ... This routine sets to .false. flags used to check whether some variables
+  ! ... have been read. If called before reading, allows to read a different
+  ! ... input file without triggering bogus error messages - useful for NEB
+  !
+  IMPLICIT NONE
+  !
+  tapos = .false.
+  tkpoints = .false.
+  taspc = .false.
+  twannier = .false.
+  tconstr = .false.
+  tforces = .false.
+  tocc = .false.
+  tksout = .false.
+  tionvel = .false.
+  tesr = .false.
+  tdipole = .false.
+  tcell = .false.
+  !
+  END SUBROUTINE reset_input_checks
+  !
+  !
+  !-----------------------------------------------------------------------------
   SUBROUTINE allocate_input_ions( ntyp, nat )
+  !-----------------------------------------------------------------------------
     !
     INTEGER, INTENT(in) :: ntyp, nat
     !
@@ -1569,7 +1453,9 @@ CONTAINS
     !
   END SUBROUTINE allocate_input_ions
 
+  !-----------------------------------------------------------------------------
   SUBROUTINE allocate_input_constr()
+  !-----------------------------------------------------------------------------
     !
     IF ( allocated( constr_type_inp ) )   DEALLOCATE( constr_type_inp )
     IF ( allocated( constr_inp ) )        DEALLOCATE( constr_inp )
@@ -1591,7 +1477,9 @@ CONTAINS
     !
   END SUBROUTINE allocate_input_constr
 
+  !-----------------------------------------------------------------------------
   SUBROUTINE allocate_input_iprnks( nksx, nspin )
+  !-----------------------------------------------------------------------------
     !
     INTEGER, INTENT(in) :: nksx, nspin
     !
@@ -1605,8 +1493,9 @@ CONTAINS
     !
   END SUBROUTINE allocate_input_iprnks
 
-
+  !-----------------------------------------------------------------------------
   SUBROUTINE deallocate_input_parameters()
+  !-----------------------------------------------------------------------------
     !
     IF ( allocated( xk ) ) DEALLOCATE( xk )
     IF ( allocated( wk ) ) DEALLOCATE( wk )
