@@ -119,11 +119,13 @@ CONTAINS
       LOGICAL                    :: tend
       INTEGER                    :: i
       !
-      INTEGER :: unit_loc=5
+      ! read_line reads from unit parse_unit
       !
-      !
-      if(present(unit)) unit_loc =  unit
-      parse_unit = unit_loc
+      IF (present(unit)) THEN
+         parse_unit =  unit
+      ELSE
+         parse_unit =  5
+      END IF
       !
       CALL card_default_values( )
       !
@@ -193,8 +195,6 @@ CONTAINS
       ELSEIF ( trim(card) == 'ATOMIC_VELOCITIES' ) THEN
          !
          CALL card_ion_velocities( input_line )
-         IF ( prog == 'CP' .and. ionode ) &
-            WRITE( stdout,'(A)') 'Warning: card '//trim(input_line)//' ignored'
          !
       ELSEIF ( trim(card) == 'KSOUT' ) THEN
          !
@@ -398,8 +398,12 @@ CONTAINS
                         & 'unknown option for ATOMIC_POSITION: '&
                         & // input_line, 1 )
          ENDIF
+         CALL infomsg( 'read_cards ', &
+            & 'DEPRECATED: no units specified in ATOMIC_POSITIONS card' )
          IF ( prog == 'CP' ) atomic_positions = 'bohr'
          IF ( prog == 'PW' ) atomic_positions = 'alat'
+         CALL infomsg( 'read_cards ', &
+            & 'ATOMIC_POSITIONS: units set to '//TRIM(atomic_positions) )
       ENDIF
       !
       reader_loop : DO ia = 1,nat
@@ -526,7 +530,7 @@ CONTAINS
          CALL errore( ' card_atomic_forces ', ' two occurrences ', 2 )
       ENDIF
       !
-      IF( .not. taspc ) THEN
+      IF( .not. tapos ) THEN
          CALL errore( ' card_atomic_forces ', &
                      & ' ATOMIC_SPECIES must be present before ', 2 )
       ENDIF
@@ -626,13 +630,18 @@ CONTAINS
    !
    SUBROUTINE card_kpoints( input_line )
       !
+      USE bz_form, ONLY : transform_label_coord
+      USE input_parameters, ONLY : ibrav, celldm, point_label_type
       IMPLICIT NONE
       !
-      CHARACTER(len=256) :: input_line
-      INTEGER            :: i, j, ijk
+      CHARACTER(len=256) :: input_line, buffer
+      INTEGER            :: i, j
       INTEGER            :: nkaux
       INTEGER, ALLOCATABLE :: wkaux(:)
       REAL(DP), ALLOCATABLE :: xkaux(:,:)
+      INTEGER            :: npk_label, nch
+      CHARACTER(LEN=3), ALLOCATABLE :: letter(:)
+      INTEGER, ALLOCATABLE :: label_list(:)
       REAL(DP) :: delta, wk0
       REAL(DP) :: dkx(3), dky(3)
       LOGICAL, EXTERNAL  :: matches
@@ -698,48 +707,67 @@ CONTAINS
 !
             nkaux=nkstot
             ALLOCATE(xkaux(3,nkstot), wkaux(nkstot))
+            ALLOCATE ( letter(nkstot) )
+            ALLOCATE ( label_list(nkstot) )
+            npk_label=0
             DO i = 1, nkstot
                CALL read_line( input_line, end_of_file = tend, error = terr )
                IF (tend) GOTO 10
                IF (terr) GOTO 20
-               READ(input_line,*, END=10, ERR=20) xkaux(1,i), xkaux(2,i), &
-                                                  xkaux(3,i), wk0
-               wkaux(i) = NINT ( wk0 ) ! beware: wkaux is integer
+               DO j=1,256   ! loop over all characters of input_line
+                  IF ((ICHAR(input_line(j:j)) < 58 .AND. &   ! a digit
+                       ICHAR(input_line(j:j)) > 47) &
+                 .OR. ICHAR(input_line(j:j)) == 43 .OR. &    ! the + sign
+                      ICHAR(input_line(j:j))== 45 .OR. &     ! the - sign
+                      ICHAR(input_line(j:j))== 46 ) THEN     ! a dot .
+!
+!   This is a digit, therefore this line contains the coordinates of the
+!   k point. We read it and exit from the loop on the characters
+!
+                     READ(input_line,*, END=10, ERR=20) xkaux(1,i), &
+                                           xkaux(2,i), xkaux(3,i), wk0
+                     wkaux(i) = NINT ( wk0 ) ! beware: wkaux is integer
+                     EXIT
+                  ELSEIF ((ICHAR(input_line(j:j)) < 123 .AND. &
+                           ICHAR(input_line(j:j)) > 64))  THEN
+!
+!   This is a letter, not a space character. We read the next three 
+!   characters and save them in the letter array, save also which k point
+!   it is
+!
+                     npk_label=npk_label+1
+                     READ(input_line(j:),'(a3)') letter(npk_label)
+                     label_list(npk_label)=i
+!
+!  now we remove the letters from input_line and read the number of points
+!  of the line. The next two line should account for the case in which
+!  there is only one space between the letter and the number of points.
+!
+                     nch=3
+                     IF ( ICHAR(input_line(j+1:j+1))==32 .OR. &
+                          ICHAR(input_line(j+2:j+2))==32 ) nch=2
+                     buffer=input_line(j+nch:)
+                     READ(buffer,*,err=20) wkaux(i)
+                     EXIT
+                  ENDIF
+               ENDDO
             ENDDO
+            IF ( npk_label > 0 ) &
+               CALL transform_label_coord(ibrav, celldm, xkaux, letter, &
+                    label_list, npk_label, nkstot, k_points, point_label_type )
+
+            DEALLOCATE(letter)
+            DEALLOCATE(label_list)
             ! Count k-points first
-            nkstot=0
+            nkstot=SUM(wkaux(1:nkaux-1))+1
             DO i=1,nkaux-1
-               IF ( wkaux(i) > 0 ) THEN
-                  nkstot=nkstot+wkaux(i)
-               ELSEIF ( wkaux(i) == 0 ) THEN
-                  nkstot=nkstot+1
-               ELSE
-                  CALL errore ('card_kpoints', 'wrong number of points',i)
-               ENDIF  
+              IF (wkaux(i)==0) nkstot=nkstot+1
             ENDDO
-            nkstot=nkstot+1
             ALLOCATE ( xk(3,nkstot), wk(nkstot) )
-            ! Now fill the points
-            nkstot=0
-            DO i=1,nkaux-1
-               IF (wkaux(i)>0) THEN
-                  delta=1.0_DP/wkaux(i)
-                  DO j=0,wkaux(i)-1
-                     nkstot=nkstot+1
-                     xk(:,nkstot)=xkaux(:,i)+delta*j*(xkaux(:,i+1)-xkaux(:,i))
-                     wk(nkstot)=1.0_DP
-                  ENDDO
-               ELSEIF (wkaux(i)==0) THEN
-                  nkstot=nkstot+1
-                  xk(:,nkstot)=xkaux(:,i)
-                  wk(nkstot)=1.0_DP
-               ELSE
-                  CALL errore ('card_kpoints', 'wrong number of points',i)
-               ENDIF  
-            ENDDO
-            nkstot=nkstot+1
-            xk(:,nkstot)=xkaux(:,nkaux)
-            wk(nkstot)=1.0_DP
+            !
+            !  generate the points along the lines
+            !
+            CALL generate_k_along_lines(nkaux, xkaux, wkaux, xk, wk, nkstot)
             DEALLOCATE(xkaux)
             DEALLOCATE(wkaux)
          ELSEIF (kband_plane) THEN
@@ -762,17 +790,7 @@ CONTAINS
             ! Count k-points first
             nkstot = wkaux(2) * wkaux(3)
             ALLOCATE ( xk(3,nkstot), wk(nkstot) )
-            dkx(:)=(xkaux(:,2)-xkaux(:,1))/(wkaux(2)-1.0_DP)
-            dky(:)=(xkaux(:,3)-xkaux(:,1))/(wkaux(3)-1.0_DP)
-            wk0=1.0_DP/nkstot
-            ijk=0
-            DO i=1, wkaux(2)
-               DO j = 1, wkaux(3)
-                  ijk=ijk+1
-                  xk(:,ijk) = xkaux(:,1) + dkx(:)*(i-1) + dky(:) * (j-1)
-                  wk(ijk) = wk0
-               ENDDO
-            ENDDO
+            CALL generate_k_in_plane(nkaux, xkaux, wkaux, xk, wk, nkstot)
             DEALLOCATE(xkaux)
             DEALLOCATE(wkaux)
          ELSE
@@ -1004,7 +1022,7 @@ CONTAINS
    !      HT(2,1) HT(2,2) HT(2,3)
    !      HT(3,1) HT(3,2) HT(3,3)
    !
-   !   cell_option == alat      lattice vectors in units of alat (default)
+   !   cell_option == alat      lattice vectors in units of alat
    !   cell_option == bohr      lattice vectors in Bohr
    !   cell_option == angstrom  lattice vectors in Angstrom
    !
@@ -1042,8 +1060,13 @@ CONTAINS
          cell_units = 'bohr'
       ELSEIF ( matches( "ANGSTROM", input_line ) ) THEN
          cell_units = 'angstrom'
-      ELSE
+      ELSEIF ( matches( "ALAT", input_line ) ) THEN
          cell_units = 'alat'
+      ELSE
+         cell_units = 'none'
+         CALL infomsg( 'read_cards ', &
+            & 'DEPRECATED: no units specified in CELL_PARAMETERS card' )
+         ! Cell parameters are set in cell_base_init
       ENDIF
       !
       DO i = 1, 3
@@ -1101,7 +1124,7 @@ CONTAINS
          CALL errore( ' card_ion_velocities ', ' two occurrences', 2 )
       ENDIF
       !
-      IF( .not. taspc ) THEN
+      IF( .not. tapos ) THEN
          CALL errore( ' card_ion_velocities ', &
                      & ' ATOMIC_SPECIES must be present before ', 2 )
       ENDIF
