@@ -139,7 +139,7 @@ CONTAINS
     if (iunps <= 0 .or. iunps >= 100000) &
          call errore('readvan','routine called with wrong 2nd argument', 1)
     !
-    read(iunps, *, err=100 ) &
+    read(iunps, *, err=100, iostat=ios ) &
          (iver(i),i=1,3), (idmy(i),i=1,3)
     write(upf%generated, &
          "('Generated by Vanderbilt code, v. ',i1,'.',i1,'.',i1)") iver
@@ -448,7 +448,11 @@ CONTAINS
     DEALLOCATE (ee, nnlz)
 800 format(4x,'|',i5,i11,5x,f10.2,f12.2,15x,'|')
     if (iver(1) >= 3 .and. nang > 0) then
-       write(fmt,900) 2*nang-1, 40-8*(2*nang-2)
+       IF (nang < 4) THEN
+          write(fmt,900) 2*nang-1, 40-8*(2*nang-2)
+       ELSE
+          write(fmt,900) 2*nang-1, 1
+       ENDIF
 900    format('(4x,"|  rinner =",',i1,'f8.4,',i2,'x,"|")')
        WRITE( stdout,fmt)  (upf%rinner(lp),lp=1,2*nang-1)
     end if
@@ -811,7 +815,6 @@ CONTAINS
     !
     return
 100 call errore('readrrkj','Reading pseudo file',abs(ios))
-    stop
   end subroutine readrrkj
   !
 end module read_uspp_module
