@@ -10,16 +10,17 @@ MODULE read_xml_module
   !
   USE input_parameters
   !
-  USE io_global, ONLY : ionode, ionode_id, xmlinputunit
+  USE io_global, ONLY : ionode, ionode_id, xmlinputunit => qestdin
   USE mp,        ONLY : mp_bcast
+  USE mp_images, ONLY : intra_image_comm
   USE iotk_module, ONLY : iotk_attlenx
   !
   ! ...default and checkin of fields
   !
   USE read_namelists_module, ONLY : control_defaults, system_defaults,&
-       ee_defaults, electrons_defaults, wannier_ac_defaults, ions_defaults, &
+       electrons_defaults, wannier_ac_defaults, ions_defaults, &
        cell_defaults, press_ai_defaults, wannier_defaults, control_bcast, &
-       system_bcast, ee_bcast, electrons_bcast, ions_bcast,cell_bcast, &
+       system_bcast, electrons_bcast, ions_bcast,cell_bcast, &
        press_ai_bcast, wannier_bcast, wannier_ac_bcast, control_checkin, &
        system_checkin, electrons_checkin, ions_checkin, cell_checkin, &
        wannier_checkin, wannier_ac_checkin, fixval
@@ -43,7 +44,7 @@ CONTAINS
   !
   !
   !--------------------------------------------------------!
-  !    This routine organizes reading of the xml file      !
+  !    This routine organizes the reading of the xml file  !
   !    depending on the program                            !
   !--------------------------------------------------------!
   SUBROUTINE read_xml( prog, attr )
@@ -61,15 +62,8 @@ CONTAINS
     CASE ('PW')
        !
        CALL read_xml_common( attr, 'PW' )
-       !
-       !
        CALL read_xml_pw()
        !
-       !
-!    CASE ('NEB')
-!       !
-!       CALL read_xml_common( attr, 'PW' )
-!       !
     CASE ('CP')
        !
        CALL read_xml_common( attr, 'CP' )
@@ -114,7 +108,6 @@ CONTAINS
     CALL electrons_defaults( prog )
     CALL ions_defaults( prog )
     CALL cell_defaults( prog )
-    CALL ee_defaults( prog )
     CALL wannier_defaults( prog )
     CALL wannier_ac_defaults( prog )
     !
@@ -145,9 +138,9 @@ CONTAINS
     !
     !  ... bcast the read attributes
     !
-    CALL mp_bcast( calculation, ionode_id )
-    CALL mp_bcast( prefix, ionode_id )
-    CALL mp_bcast( title, ionode_id )
+    CALL mp_bcast( calculation, ionode_id, intra_image_comm )
+    CALL mp_bcast( prefix, ionode_id, intra_image_comm )
+    CALL mp_bcast( title, ionode_id, intra_image_comm )
     
     !
     ! ... fixing some default values using the calculation variable
@@ -189,7 +182,7 @@ CONTAINS
   !
   !
   !--------------------------------------------------------!
-  ! The rest of the reading for PW program: fields and     !
+  ! The remaining part of the reading for PW: fields and   !
   ! other cards                                            !
   !--------------------------------------------------------!
   SUBROUTINE read_xml_pw( )
@@ -246,7 +239,7 @@ CONTAINS
        ENDIF
     ENDIF
     !
-    CALL mp_bcast( found_al, ionode_id)
+    CALL mp_bcast( found_al, ionode_id, intra_image_comm)
     !
     IF (found_al) THEN
        CALL card_bcast( 'ATOMIC_LIST' )
@@ -276,11 +269,11 @@ CONTAINS
     IF ( calculation == 'nscf' .or. calculation == 'bands'  ) THEN
        !
        IF (startingpot == 'none') startingpot = 'file'
-       IF (startingwfc == 'none') startingwfc = 'atomic'
+       IF (startingwfc == 'none') startingwfc = 'atomic+random'
        !
     ELSE IF ( restart_mode == 'from_scratch' ) THEN
        !
-       IF (startingwfc == 'none') startingwfc = 'atomic'
+       IF (startingwfc == 'none') startingwfc = 'atomic+random'
        IF (startingpot == 'none') startingpot = 'atomic'
        !
     ELSE
@@ -315,7 +308,6 @@ CONTAINS
     CALL ions_bcast( )
     CALL cell_bcast()
     CALL press_ai_bcast()
-    CALL ee_bcast()
     CALL wannier_bcast()
     CALL wannier_ac_bcast()
     !
@@ -349,7 +341,7 @@ CONTAINS
        !
     END IF
     !
-    CALL mp_bcast ( found, ionode_id )
+    CALL mp_bcast ( found, ionode_id, intra_image_comm )
     !
     IF ( found ) CALL card_bcast( 'CONSTRAINTS' )
     !
@@ -409,7 +401,7 @@ CONTAINS
        !
     END IF
     !
-    CALL mp_bcast ( found, ionode_id )
+    CALL mp_bcast ( found, ionode_id, intra_image_comm )
     !
     IF ( found ) CALL card_bcast( 'OCCUPATIONS' )
     !
@@ -440,7 +432,7 @@ CONTAINS
        !
     END IF
     !
-    CALL mp_bcast ( found, ionode_id )
+    CALL mp_bcast ( found, ionode_id, intra_image_comm )
     !
     IF ( found ) CALL card_bcast( 'CLIMBING_IMAGES' )
     !
@@ -527,7 +519,6 @@ CONTAINS
     CALL ions_bcast( )
     CALL cell_bcast()
     CALL press_ai_bcast()
-    CALL ee_bcast()
     CALL wannier_bcast()
     CALL wannier_ac_bcast()
     !
@@ -561,7 +552,7 @@ CONTAINS
        !
     END IF
     !
-    CALL mp_bcast ( found, ionode_id )
+    CALL mp_bcast ( found, ionode_id, intra_image_comm )
     !
     IF ( found ) CALL card_bcast( 'CONSTRAINTS' )
     !
@@ -591,7 +582,7 @@ CONTAINS
        !
     END IF
     !
-    CALL mp_bcast ( found, ionode_id )
+    CALL mp_bcast ( found, ionode_id, intra_image_comm )
     !
     IF ( found ) CALL card_bcast( 'OCCUPATIONS' )
     !
@@ -622,7 +613,7 @@ CONTAINS
        !
     END IF
     !
-    CALL mp_bcast ( found, ionode_id )
+    CALL mp_bcast ( found, ionode_id, intra_image_comm )
     !
     IF ( found ) CALL card_bcast( 'CLIMBING_IMAGES' )
     !
@@ -653,7 +644,7 @@ CONTAINS
        !
     END IF
     !
-    CALL mp_bcast ( found, ionode_id )
+    CALL mp_bcast ( found, ionode_id, intra_image_comm )
     !
     IF ( found ) CALL card_bcast( 'PLOT_WANNIER' )
     !
