@@ -41,8 +41,8 @@ PROGRAM average
   USE klist,                ONLY : nks
   USE parameters,           ONLY : ntypx
   USE constants,            ONLY : pi
-  USE run_info,        ONLY : title
-  USE io_global,            ONLY : stdout
+  USE run_info,             ONLY : title
+  USE io_global,            ONLY : stdout, ionode
   USE cell_base,            ONLY : ibrav, alat, omega, celldm, tpiba, &
                                    tpiba2, at, bg
   USE gvect,                ONLY : gcutm
@@ -56,7 +56,7 @@ PROGRAM average
   USE wavefunctions_module, ONLY : psic
   USE io_files,             ONLY : iunpun
   USE scf,                  ONLY : rho
-  USE mp_global,            ONLY : mpime, root, mp_startup
+  USE mp_global,            ONLY : mp_startup
   USE environment,          ONLY : environment_start
   !
   IMPLICIT NONE
@@ -114,7 +114,7 @@ PROGRAM average
   !
   ! Works for parallel machines but only for one processor !!!
   !
-  IF ( mpime == root ) THEN
+  IF ( ionode ) THEN
      !
      inunit = 5
      READ (inunit, *, err = 1100, iostat = ios) nfile
@@ -132,7 +132,8 @@ PROGRAM average
 
 1100 CALL errore ('average', 'readin input', abs (ios) )
 
-     CALL read_io_header(filename (1), title, dfftp%nr1x, dfftp%nr2x, dfftp%nr3x, dfftp%nr1, dfftp%nr2, dfftp%nr3, &
+     CALL read_io_header(filename (1), title, dfftp%nr1x, dfftp%nr2x, &
+                         dfftp%nr3x, dfftp%nr1, dfftp%nr2, dfftp%nr3, &
           nat, ntyp, ibrav, celldm, at, gcutm, dual, ecutwfc, plot_num)
      nspin = 1
      CALL latgen (ibrav, celldm, at(1,1), at(1,2), at(1,3), omega )
