@@ -155,7 +155,7 @@ SUBROUTINE local_dos (iflag, lsign, kpoint, kband, spin_component, &
      IF (ik == kpoint_pool .and.i_am_the_pool.or. iflag /= 0) THEN
         IF (lsda) current_spin = isk (ik)
         CALL gk_sort (xk (1, ik), ngm, g, ecutwfc / tpiba2, npw, igk, g2kin)
-        CALL davcio (evc, nwordwfc, iunwfc, ik, - 1)
+        CALL davcio (evc, 2*nwordwfc, iunwfc, ik, - 1)
         CALL init_us_2 (npw, igk, xk (1, ik), vkb)
 
         IF (gamma_only) THEN
