@@ -64,8 +64,10 @@
 !               process (called from write_wfng)
 ! write_rhog  - generates real/complex charge density in G-space
 !               (units of the number of electronic states per unit cell)
-! write_vxcg  - generates real/complex exchange-correlation potential in G-space
-!               (units of Rydberg) [only local part of Vxc]
+! calc_rhog   - computes charge density by summing over a subset of occupied
+!               bands (called from write_rhog), destroys charge density
+! write_vxcg  - generates real/complex exchange-correlation potential in
+!               G-space (units of Rydberg) [only local part of Vxc]
 ! write_vxc0  - prints real/complex exchange-correlation potential at G=0
 !               (units of eV) [only local part of Vxc]
 ! write_vxc_r - calculates matrix elements of exchange-correlation potential
@@ -77,6 +79,7 @@
 ! write_vkbg  - generates complex Kleinman-Bylander projectors in G-space
 !               (units of Rydberg)
 ! check_inversion - checks whether real/complex version is appropriate
+!               (called from everywhere)
 !
 ! Quantum ESPRESSO stores the wavefunctions in is-ik-ib-ig order
 ! BerkeleyGW stores the wavefunctions in ik-ib-is-ig order
@@ -101,6 +104,7 @@ PROGRAM pw2bgw
   USE kinds, ONLY : DP
   USE lsda_mod, ONLY : nspin
   USE mp, ONLY : mp_bcast
+  USE mp_world, ONLY : world_comm
   USE mp_global, ONLY : mp_startup
   USE paw_variables, ONLY : okpaw
   USE scf, ONLY : rho_core, rhog_core
@@ -126,6 +130,8 @@ PROGRAM pw2bgw
   integer :: wfng_nvmax
   logical :: rhog_flag
   character ( len = 256 ) :: rhog_file
+  integer :: rhog_nvmin
+  integer :: rhog_nvmax
   logical :: vxcg_flag
   character ( len = 256 ) :: vxcg_file
   logical :: vxc0_flag
@@ -147,8 +153,8 @@ PROGRAM pw2bgw
     real_or_complex, symm_type, wfng_flag, wfng_file, wfng_kgrid, &
     wfng_nk1, wfng_nk2, wfng_nk3, wfng_dk1, wfng_dk2, wfng_dk3, &
     wfng_occupation, wfng_nvmin, wfng_nvmax, rhog_flag, rhog_file, &
-    vxcg_flag, vxcg_file, vxc0_flag, vxc0_file, vxc_flag, &
-    vxc_file, vxc_integral, vxc_diag_nmin, vxc_diag_nmax, &
+    rhog_nvmin, rhog_nvmax, vxcg_flag, vxcg_file, vxc0_flag, vxc0_file, &
+    vxc_flag, vxc_file, vxc_integral, vxc_diag_nmin, vxc_diag_nmax, &
     vxc_offdiag_nmin, vxc_offdiag_nmax, vxc_zero_rho_core, &
     vscg_flag, vscg_file, vkbg_flag, vkbg_file
 
@@ -159,7 +165,7 @@ PROGRAM pw2bgw
   character (len=1), external :: lowercase
 
 #ifdef __PARA
-  CALL mp_startup ( )
+  CALL mp_startup ( world_comm )
 #endif
   CALL environment_start ( codename )
 
@@ -182,6 +188,8 @@ PROGRAM pw2bgw
   wfng_nvmax = 0
   rhog_flag = .FALSE.
   rhog_file = 'RHO'
+  rhog_nvmin = 0
+  rhog_nvmax = 0
   vxcg_flag = .FALSE.
   vxcg_file = 'VXC'
   vxc0_flag = .FALSE.
@@ -193,7 +201,7 @@ PROGRAM pw2bgw
   vxc_diag_nmax = 0
   vxc_offdiag_nmin = 0
   vxc_offdiag_nmax = 0
-  vxc_zero_rho_core = .FALSE.
+  vxc_zero_rho_core = .TRUE.
   vscg_flag = .FALSE.
   vscg_file = 'VSC'
   vkbg_flag = .FALSE.
@@ -220,41 +228,43 @@ PROGRAM pw2bgw
   ENDIF
 
   tmp_dir = trimcheck ( outdir )
-  CALL mp_bcast ( outdir, ionode_id )
-  CALL mp_bcast ( tmp_dir, ionode_id )
-  CALL mp_bcast ( prefix, ionode_id )
-  CALL mp_bcast ( real_or_complex, ionode_id )
-  CALL mp_bcast ( symm_type, ionode_id )
-  CALL mp_bcast ( wfng_flag, ionode_id )
-  CALL mp_bcast ( wfng_file, ionode_id )
-  CALL mp_bcast ( wfng_kgrid, ionode_id )
-  CALL mp_bcast ( wfng_nk1, ionode_id )
-  CALL mp_bcast ( wfng_nk2, ionode_id )
-  CALL mp_bcast ( wfng_nk3, ionode_id )
-  CALL mp_bcast ( wfng_dk1, ionode_id )
-  CALL mp_bcast ( wfng_dk2, ionode_id )
-  CALL mp_bcast ( wfng_dk3, ionode_id )
-  CALL mp_bcast ( wfng_occupation, ionode_id )
-  CALL mp_bcast ( wfng_nvmin, ionode_id )
-  CALL mp_bcast ( wfng_nvmax, ionode_id )
-  CALL mp_bcast ( rhog_flag, ionode_id )
-  CALL mp_bcast ( rhog_file, ionode_id )
-  CALL mp_bcast ( vxcg_flag, ionode_id )
-  CALL mp_bcast ( vxcg_file, ionode_id )
-  CALL mp_bcast ( vxc0_flag, ionode_id )
-  CALL mp_bcast ( vxc0_file, ionode_id )
-  CALL mp_bcast ( vxc_flag, ionode_id )
-  CALL mp_bcast ( vxc_integral, ionode_id )
-  CALL mp_bcast ( vxc_file, ionode_id )
-  CALL mp_bcast ( vxc_diag_nmin, ionode_id )
-  CALL mp_bcast ( vxc_diag_nmax, ionode_id )
-  CALL mp_bcast ( vxc_offdiag_nmin, ionode_id )
-  CALL mp_bcast ( vxc_offdiag_nmax, ionode_id )
-  CALL mp_bcast ( vxc_zero_rho_core, ionode_id )
-  CALL mp_bcast ( vscg_flag, ionode_id )
-  CALL mp_bcast ( vscg_file, ionode_id )
-  CALL mp_bcast ( vkbg_flag, ionode_id )
-  CALL mp_bcast ( vkbg_file, ionode_id )
+  CALL mp_bcast ( outdir, ionode_id, world_comm )
+  CALL mp_bcast ( tmp_dir, ionode_id, world_comm )
+  CALL mp_bcast ( prefix, ionode_id, world_comm )
+  CALL mp_bcast ( real_or_complex, ionode_id, world_comm )
+  CALL mp_bcast ( symm_type, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_flag, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_file, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_kgrid, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_nk1, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_nk2, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_nk3, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_dk1, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_dk2, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_dk3, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_occupation, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_nvmin, ionode_id, world_comm )
+  CALL mp_bcast ( wfng_nvmax, ionode_id, world_comm )
+  CALL mp_bcast ( rhog_flag, ionode_id, world_comm )
+  CALL mp_bcast ( rhog_file, ionode_id, world_comm )
+  CALL mp_bcast ( rhog_nvmin, ionode_id, world_comm )
+  CALL mp_bcast ( rhog_nvmax, ionode_id, world_comm )
+  CALL mp_bcast ( vxcg_flag, ionode_id, world_comm )
+  CALL mp_bcast ( vxcg_file, ionode_id, world_comm )
+  CALL mp_bcast ( vxc0_flag, ionode_id, world_comm )
+  CALL mp_bcast ( vxc0_file, ionode_id, world_comm )
+  CALL mp_bcast ( vxc_flag, ionode_id, world_comm )
+  CALL mp_bcast ( vxc_integral, ionode_id, world_comm )
+  CALL mp_bcast ( vxc_file, ionode_id, world_comm )
+  CALL mp_bcast ( vxc_diag_nmin, ionode_id, world_comm )
+  CALL mp_bcast ( vxc_diag_nmax, ionode_id, world_comm )
+  CALL mp_bcast ( vxc_offdiag_nmin, ionode_id, world_comm )
+  CALL mp_bcast ( vxc_offdiag_nmax, ionode_id, world_comm )
+  CALL mp_bcast ( vxc_zero_rho_core, ionode_id, world_comm )
+  CALL mp_bcast ( vscg_flag, ionode_id, world_comm )
+  CALL mp_bcast ( vscg_file, ionode_id, world_comm )
+  CALL mp_bcast ( vkbg_flag, ionode_id, world_comm )
+  CALL mp_bcast ( vkbg_file, ionode_id, world_comm )
 
   CALL read_file ( )
 
@@ -286,15 +296,6 @@ PROGRAM pw2bgw
     IF ( ionode ) WRITE ( 6, '(5x,"done write_wfng",/)' )
   ENDIF
 
-  IF ( rhog_flag ) THEN
-    output_file_name = TRIM ( outdir ) // '/' // TRIM ( rhog_file )
-    IF ( ionode ) WRITE ( 6, '(5x,"call write_rhog")' )
-    CALL start_clock ( 'write_rhog' )
-    CALL write_rhog ( output_file_name, real_or_complex, symm_type )
-    CALL stop_clock ( 'write_rhog' )
-    IF ( ionode ) WRITE ( 6, '(5x,"done write_rhog",/)' )
-  ENDIF
-
   IF ( vxcg_flag ) THEN
     output_file_name = TRIM ( outdir ) // '/' // TRIM ( vxcg_file )
     IF ( ionode ) WRITE ( 6, '(5x,"call write_vxcg")' )
@@ -357,6 +358,19 @@ PROGRAM pw2bgw
     IF ( ionode ) WRITE ( 6, '(5x,"done write_vkbg",/)' )
   ENDIF
 
+  ! since calc_rhog (called from write_rhog) destroys charge density,
+  ! it must be called after v_xc (called from write_vxcg, write_vxc0,
+  ! write_vxc_r, write_vxc_g)
+  IF ( rhog_flag ) THEN
+    output_file_name = TRIM ( outdir ) // '/' // TRIM ( rhog_file )
+    IF ( ionode ) WRITE ( 6, '(5x,"call write_rhog")' )
+    CALL start_clock ( 'write_rhog' )
+    CALL write_rhog ( output_file_name, real_or_complex, symm_type, &
+      rhog_nvmin, rhog_nvmax )
+    CALL stop_clock ( 'write_rhog' )
+    IF ( ionode ) WRITE ( 6, '(5x,"done write_rhog",/)' )
+  ENDIF
+
   IF ( ionode ) WRITE ( 6, * )
   IF ( wfng_flag ) CALL print_clock ( 'write_wfng' )
   IF ( rhog_flag ) CALL print_clock ( 'write_rhog' )
@@ -378,7 +392,7 @@ PROGRAM pw2bgw
 
   STOP
 
-END PROGRAM pw2bgw
+CONTAINS
 
 !-------------------------------------------------------------------------------
 
@@ -397,9 +411,10 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
   USE klist, ONLY : xk, wk, ngk, nks, nkstot
   USE lsda_mod, ONLY : nspin, isk
   USE mp, ONLY : mp_sum, mp_max, mp_get, mp_bcast, mp_barrier
-  USE mp_global, ONLY : mpime, nproc, world_comm, kunit, me_pool, &
+  USE mp_pools, ONLY : kunit, me_pool, &
     root_pool, my_pool_id, npool, nproc_pool, intra_pool_comm
   USE mp_wave, ONLY : mergewf
+  USE mp_world, ONLY : mpime, nproc, world_comm
   USE start_k, ONLY : nk1, nk2, nk3, k1, k2, k3
   USE symm_base, ONLY : s, ftau, nsym
   USE wavefunctions_module, ONLY : evc
@@ -454,8 +469,6 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
 
   INTEGER, EXTERNAL :: atomic_number
 
-  CALL check_inversion ( real_or_complex, nsym, s, nspin, .true., .true. )
-
   IF ( real_or_complex .EQ. 1 .OR. nspin .GT. 1 ) THEN
     proc_wf = .TRUE.
   ELSE
@@ -518,7 +531,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
     IF ( ik .GE. iks .AND. ik .LE. ike .AND. is .NE. isk ( ik ) ) &
       ierr = ierr + 1
   ENDDO
-  CALL mp_max ( ierr )
+  CALL mp_max ( ierr, world_comm )
   IF ( ierr .GT. 0 ) &
     CALL errore ( 'write_wfng', 'smap', ierr )
 
@@ -600,6 +613,8 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
     ENDDO
   ENDDO
 
+  CALL check_inversion ( real_or_complex, nsym, s, nspin, .true., .true., translation )
+
   ALLOCATE ( et_g ( nb, nk_g ) )
 
   DO ik = 1, nk_l
@@ -609,7 +624,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
   ENDDO
 #ifdef __PARA
   CALL poolrecover ( et_g, nb, nk_g, nk_l )
-  CALL mp_bcast ( et_g, ionode_id )
+  CALL mp_bcast ( et_g, ionode_id, world_comm )
 #endif
 
   ALLOCATE ( wg_g ( nb, nk_g ) )
@@ -706,10 +721,10 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
   DO ik = 1, nk_l
     ngk_g ( ik + iks - 1 ) = ngk ( ik )
   ENDDO
-  CALL mp_sum ( ngk_g )
+  CALL mp_sum ( ngk_g, world_comm )
 
   npw_g = MAXVAL ( igk_l2g ( :, : ) )
-  CALL mp_max ( npw_g )
+  CALL mp_max ( npw_g, world_comm )
 
   npwx_g = MAXVAL ( ngk_g ( : ) )
 
@@ -791,7 +806,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
         itmp ( igk_l2g ( ig, ik - iks + 1 ) ) = igk_l2g ( ig, ik - iks + 1 )
       ENDDO
     ENDIF
-    CALL mp_sum ( itmp )
+    CALL mp_sum ( itmp, world_comm )
     ngg = 0
     DO ig = 1, npw_g
       IF ( itmp ( ig ) .EQ. ig ) THEN
@@ -812,7 +827,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
 
     local_pw = 0
     IF ( ik .GE. iks .AND. ik .LE. ike ) THEN
-      CALL davcio ( evc, nwordwfc, iunwfc, ik - iks + 1, - 1 )
+      CALL davcio ( evc, 2*nwordwfc, iunwfc, ik - iks + 1, - 1 )
       local_pw = ngk ( ik - iks + 1 )
     ENDIF
 
@@ -840,7 +855,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
       IF ( ( ik .GE. iks ) .AND. ( ik .LE. ike ) ) THEN
         IF ( me_pool .EQ. root_pool ) ipmask ( mpime + 1 ) = 1
       ENDIF
-      CALL mp_sum ( ipmask )
+      CALL mp_sum ( ipmask, world_comm )
       DO j = 1, nproc
         IF ( ipmask ( j ) .EQ. 1 ) ipsour = j - 1
       ENDDO
@@ -853,11 +868,11 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
       igwx = MAXVAL ( igwf_l2g ( 1 : local_pw ) )
     CALL mp_max ( igwx, intra_pool_comm )
     IF ( ipsour .NE. ionode_id ) &
-      CALL mp_get ( igwx, igwx, mpime, ionode_id, ipsour, 1 )
+      CALL mp_get ( igwx, igwx, mpime, ionode_id, ipsour, 1, world_comm )
     ierr = 0
     IF ( ik .GE. iks .AND. ik .LE. ike .AND. igwx .NE. ngk_g ( ik ) ) &
       ierr = 1
-    CALL mp_max ( ierr )
+    CALL mp_max ( ierr, world_comm )
     IF ( ierr .GT. 0 ) &
       CALL errore ( 'write_wfng', 'igwx ngk_g', ierr )
 
@@ -890,7 +905,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
           wfng_buf ( ig, is ) = ( 0.0D0, 0.0D0 )
         ENDDO
 #ifdef __PARA
-        CALL mp_barrier ( )
+        CALL mp_barrier ( world_comm )
         CALL MPI_Scatter ( wfng_buf ( :, is ), ngkdist_l, MPI_DOUBLE_COMPLEX, &
         wfng_dist ( :, ib, is ), ngkdist_l, MPI_DOUBLE_COMPLEX, &
         ionode_id, world_comm, ierr )
@@ -923,7 +938,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
       DO ib = 1, nb
         DO is = 1, ns
 #ifdef __PARA
-          CALL mp_barrier ( )
+          CALL mp_barrier ( world_comm )
           CALL MPI_Gather ( wfng_dist ( :, ib, is ), ngkdist_l, &
           MPI_DOUBLE_COMPLEX, wfng_buf ( :, is ), ngkdist_l, &
           MPI_DOUBLE_COMPLEX, ionode_id, world_comm, ierr )
@@ -971,7 +986,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
   DEALLOCATE ( smap )
   DEALLOCATE ( kmap )
 
-  CALL mp_barrier ( )
+  CALL mp_barrier ( world_comm )
 
   RETURN
 
@@ -987,12 +1002,13 @@ SUBROUTINE real_wfng ( ik, ngkdist_l, nb, ns, energy, wfng_dist )
   USE kinds, ONLY : DP
   USE io_global, ONLY : ionode
   USE mp, ONLY : mp_sum
+  USE mp_world, ONLY : world_comm
 
   IMPLICIT NONE
 
   integer, intent (in) :: ik, ngkdist_l, nb, ns
-  real (DP), intent (in) :: energy ( nb, ns )
-  complex (DP), intent (inout) :: wfng_dist ( ngkdist_l, nb, ns )
+  real (DP), intent (in) :: energy ( :, : ) ! ( nb, ns )
+  complex (DP), intent (inout) :: wfng_dist ( :, :, : ) ! ( ngkdist_l, nb, ns )
 
   real (DP), PARAMETER :: eps2 = 1.0D-2
   real (DP), PARAMETER :: eps5 = 1.0D-5
@@ -1063,7 +1079,7 @@ SUBROUTINE real_wfng ( ik, ngkdist_l, nb, ns, energy, wfng_dist )
         DO ig = 1, ngkdist_l
           x = x + dble ( wfc ( ig ) ) **2
         ENDDO
-        CALL mp_sum ( x )
+        CALL mp_sum ( x, world_comm )
         IF ( x .LT. eps2 ) null_map ( inull ( i ), i ) = 0
         IF ( x .GT. eps2 ) null_map ( inull ( i ), i ) = 1
         inull ( i ) = inull ( i ) + 1
@@ -1071,7 +1087,7 @@ SUBROUTINE real_wfng ( ik, ngkdist_l, nb, ns, energy, wfng_dist )
         DO ig = 1, ngkdist_l
           x = x + aimag ( wfc ( ig ) ) **2
         ENDDO
-        CALL mp_sum ( x )
+        CALL mp_sum ( x, world_comm )
         IF ( x .LT. eps2 ) null_map ( inull ( i ), i ) = 0
         IF ( x .GT. eps2 ) null_map ( inull ( i ), i ) = 1
         inull ( i ) = inull ( i ) + 1
@@ -1115,7 +1131,7 @@ SUBROUTINE real_wfng ( ik, ngkdist_l, nb, ns, energy, wfng_dist )
         DO ig = 1, ngkdist_l
           x = x + phi ( ig, j ) **2
         ENDDO
-        CALL mp_sum ( x )
+        CALL mp_sum ( x, world_comm )
         x = sqrt ( x )
         DO ig = 1, ngkdist_l
           phi ( ig, j ) = phi ( ig, j ) / x
@@ -1137,7 +1153,7 @@ SUBROUTINE real_wfng ( ik, ngkdist_l, nb, ns, energy, wfng_dist )
           DO ig = 1, ngkdist_l
             x = x + phi ( ig, j + 1 ) * psi ( ig, k )
           ENDDO
-          CALL mp_sum ( x )
+          CALL mp_sum ( x, world_comm )
           DO ig = 1, ngkdist_l
             vec ( ig ) = vec ( ig ) - psi ( ig, k ) * x
           ENDDO
@@ -1146,7 +1162,7 @@ SUBROUTINE real_wfng ( ik, ngkdist_l, nb, ns, energy, wfng_dist )
         DO ig = 1, ngkdist_l
           x = x + vec ( ig ) **2
         ENDDO
-        CALL mp_sum ( x )
+        CALL mp_sum ( x, world_comm )
         x = sqrt ( x )
         IF ( x .GT. eps6 ) THEN
           reduced_span = reduced_span + 1
@@ -1191,7 +1207,8 @@ END SUBROUTINE real_wfng
 
 !-------------------------------------------------------------------------------
 
-SUBROUTINE write_rhog ( output_file_name, real_or_complex, symm_type )
+SUBROUTINE write_rhog ( output_file_name, real_or_complex, symm_type, &
+  rhog_nvmin, rhog_nvmax )
 
   USE cell_base, ONLY : omega, alat, tpiba, tpiba2, at, bg, ibrav
   USE constants, ONLY : pi, tpi, eps6
@@ -1202,7 +1219,8 @@ SUBROUTINE write_rhog ( output_file_name, real_or_complex, symm_type )
   USE kinds, ONLY : DP
   USE lsda_mod, ONLY : nspin
   USE mp, ONLY : mp_sum
-  USE mp_global, ONLY : intra_pool_comm
+  USE mp_world, ONLY : world_comm
+  USE mp_pools, ONLY : intra_pool_comm
   USE scf, ONLY : rho
   USE symm_base, ONLY : s, ftau, nsym
 
@@ -1211,6 +1229,8 @@ SUBROUTINE write_rhog ( output_file_name, real_or_complex, symm_type )
   character ( len = 256 ), intent (in) :: output_file_name
   integer, intent (in) :: real_or_complex
   character ( len = 9 ), intent (in) :: symm_type
+  integer, intent (in) :: rhog_nvmin
+  integer, intent (in) :: rhog_nvmax
 
   character :: cdate*9, ctime*9, sdate*32, stime*32, stitle*32
   integer :: unit, id, is, ig, i, j, k, ierr
@@ -1224,8 +1244,6 @@ SUBROUTINE write_rhog ( output_file_name, real_or_complex, symm_type )
 
   INTEGER, EXTERNAL :: atomic_number
 
-  CALL check_inversion ( real_or_complex, nsym, s, nspin, .true., .true. )
-
   CALL date_and_tim ( cdate, ctime )
   WRITE ( sdate, '(A2,"-",A3,"-",A4,21X)' ) cdate(1:2), cdate(3:5), cdate(6:9)
   WRITE ( stime, '(A8,24X)' ) ctime(1:8)
@@ -1290,6 +1308,8 @@ SUBROUTINE write_rhog ( output_file_name, real_or_complex, symm_type )
     ENDDO
   ENDDO
 
+  CALL check_inversion ( real_or_complex, nsym, s, nspin, .true., .true., translation )
+
   alat2 = alat ** 2
   recvol = 8.0D0 * pi**3 / omega
 
@@ -1321,6 +1341,9 @@ SUBROUTINE write_rhog ( output_file_name, real_or_complex, symm_type )
     ENDDO
   ENDDO
 
+  IF ( rhog_nvmin .NE. 0 .AND. rhog_nvmax .NE. 0 ) &
+    CALL calc_rhog ( rhog_nvmin, rhog_nvmax )
+
   ALLOCATE ( g_g ( nd, ng_g ) )
   ALLOCATE ( rhog_g ( ng_g, ns ) )
 
@@ -1392,6 +1415,84 @@ END SUBROUTINE write_rhog
 
 !-------------------------------------------------------------------------------
 
+SUBROUTINE calc_rhog (rhog_nvmin, rhog_nvmax)
+
+! calc_rhog    Originally By Brad D. Malone    Last Modified (night before his thesis defense)
+! Computes charge density by summing over a subset of occupied bands
+
+  USE cell_base, ONLY : omega, tpiba2
+  USE fft_base, ONLY : dfftp
+  USE fft_interfaces, ONLY : fwfft, invfft
+  USE gvect, ONLY : ngm, g, nl
+  USE io_files, ONLY : nwordwfc, iunwfc
+  USE klist, ONLY : xk, nkstot
+  USE lsda_mod, ONLY : nspin, isk
+  USE mp, ONLY : mp_sum
+  USE mp_world, ONLY : world_comm
+  USE mp_pools, ONLY : kunit, my_pool_id, inter_pool_comm, npool
+  USE noncollin_module, ONLY : nspin_mag
+  USE scf, ONLY : rho
+  USE symme, ONLY : sym_rho, sym_rho_init
+  USE wavefunctions_module, ONLY : evc, psic
+  USE wvfct, ONLY : npw, igk, wg, g2kin, ecutwfc
+
+  IMPLICIT NONE
+
+  integer, intent (in) :: rhog_nvmin
+  integer, intent (in) :: rhog_nvmax
+
+  integer :: ik, is, ib, ig, ir, nkbl, nkl, nkr, iks, ike
+
+  nkbl = nkstot / kunit
+  nkl = kunit * (nkbl / npool)
+  nkr = (nkstot - nkl * npool) / kunit
+  IF (my_pool_id .LT. nkr) nkl = nkl + kunit
+  iks = nkl * my_pool_id + 1
+  IF (my_pool_id .GE. nkr) iks = iks + nkr * kunit
+  ike = iks + nkl - 1
+
+  CALL weights ()
+
+  rho%of_r (:, :) = 0.0D0
+
+  ! take psi to R-space, compute rho in R-space
+  DO ik = iks, ike
+    is = isk (ik)
+    CALL gk_sort (xk (1, ik - iks + 1), ngm, g, ecutwfc &
+      / tpiba2, npw, igk, g2kin)
+    CALL davcio (evc, 2*nwordwfc, iunwfc, ik - iks + 1, -1)
+    DO ib = rhog_nvmin, rhog_nvmax
+      psic (:) = (0.0D0, 0.0D0)
+      DO ig = 1, npw
+        psic (nl (igk (ig))) = evc (ig, ib)
+      ENDDO
+      CALL invfft ('Dense', psic, dfftp)
+      DO ir = 1, dfftp%nnr
+        rho%of_r (ir, is) = rho%of_r (ir, is) + wg (ib, ik) / omega &
+          * (dble (psic (ir)) **2 + aimag (psic (ir)) **2)
+      ENDDO
+    ENDDO
+  ENDDO
+  CALL mp_sum (rho%of_r, inter_pool_comm)
+
+  ! take rho to G-space
+  DO is = 1, nspin
+    psic (:) = (0.0D0, 0.0D0)
+    psic (:) = rho%of_r (:, is)
+    CALL fwfft ('Dense', psic, dfftp)
+    rho%of_g (:, is) = psic (nl (:))
+  ENDDO
+
+  ! symmetrize rho (didn`t make a difference)
+  CALL sym_rho_init (.False.)
+  CALL sym_rho (nspin_mag, rho%of_g)
+
+  RETURN
+
+END SUBROUTINE calc_rhog
+
+!-------------------------------------------------------------------------------
+
 SUBROUTINE write_vxcg ( output_file_name, real_or_complex, symm_type, &
   vxc_zero_rho_core )
 
@@ -1406,7 +1507,7 @@ SUBROUTINE write_vxcg ( output_file_name, real_or_complex, symm_type, &
   USE kinds, ONLY : DP
   USE lsda_mod, ONLY : nspin
   USE mp, ONLY : mp_sum
-  USE mp_global, ONLY : intra_pool_comm
+  USE mp_pools, ONLY : intra_pool_comm
   USE scf, ONLY : rho, rho_core, rhog_core
   USE symm_base, ONLY : s, ftau, nsym
   USE wavefunctions_module, ONLY : psic
@@ -1431,8 +1532,6 @@ SUBROUTINE write_vxcg ( output_file_name, real_or_complex, symm_type, &
 
   INTEGER, EXTERNAL :: atomic_number
 
-  CALL check_inversion ( real_or_complex, nsym, s, nspin, .true., .true. )
-
   CALL date_and_tim ( cdate, ctime )
   WRITE ( sdate, '(A2,"-",A3,"-",A4,21X)' ) cdate(1:2), cdate(3:5), cdate(6:9)
   WRITE ( stime, '(A8,24X)' ) ctime(1:8)
@@ -1498,6 +1597,8 @@ SUBROUTINE write_vxcg ( output_file_name, real_or_complex, symm_type, &
     ENDDO
   ENDDO
 
+  CALL check_inversion ( real_or_complex, nsym, s, nspin, .true., .true., translation )
+
   alat2 = alat ** 2
   recvol = 8.0D0 * pi**3 / omega
 
@@ -1617,7 +1718,7 @@ SUBROUTINE write_vxc0 ( output_file_name, vxc_zero_rho_core )
   USE kinds, ONLY : DP
   USE lsda_mod, ONLY : nspin
   USE mp, ONLY : mp_sum
-  USE mp_global, ONLY : intra_pool_comm
+  USE mp_pools, ONLY : intra_pool_comm
   USE scf, ONLY : rho, rho_core, rhog_core
   USE wavefunctions_module, ONLY : psic
 
@@ -1710,7 +1811,7 @@ SUBROUTINE write_vxc_r (output_file_name, diag_nmin, diag_nmax, &
   USE klist, ONLY : xk, nkstot
   USE lsda_mod, ONLY : nspin, isk
   USE mp, ONLY : mp_sum
-  USE mp_global, ONLY : kunit, my_pool_id, intra_pool_comm, &
+  USE mp_pools, ONLY : kunit, my_pool_id, intra_pool_comm, &
     inter_pool_comm, npool
   USE scf, ONLY : rho, rho_core, rhog_core
   USE wavefunctions_module, ONLY : evc, psic
@@ -1788,7 +1889,7 @@ SUBROUTINE write_vxc_r (output_file_name, diag_nmin, diag_nmax, &
   DO ik = iks, ike
     CALL gk_sort (xk (1, ik - iks + 1), ngm, g, ecutwfc &
       / tpiba2, npw, igk, g2kin)
-    CALL davcio (evc, nwordwfc, iunwfc, ik - iks + 1, -1)
+    CALL davcio (evc, 2*nwordwfc, iunwfc, ik - iks + 1, -1)
     IF (ndiag .GT. 0) THEN
       DO ib = diag_nmin, diag_nmax
         psic (:) = (0.0D0, 0.0D0)
@@ -1902,7 +2003,7 @@ SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
   USE klist, ONLY : xk, nkstot
   USE lsda_mod, ONLY : nspin, isk
   USE mp, ONLY : mp_sum
-  USE mp_global, ONLY : kunit, my_pool_id, intra_pool_comm, &
+  USE mp_pools, ONLY : kunit, my_pool_id, intra_pool_comm, &
     inter_pool_comm, npool
   USE scf, ONLY : rho, rho_core, rhog_core
   USE wavefunctions_module, ONLY : evc, psic
@@ -1981,7 +2082,7 @@ SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
   DO ik = iks, ike
     CALL gk_sort (xk (1, ik - iks + 1), ngm, g, ecutwfc &
       / tpiba2, npw, igk, g2kin)
-    CALL davcio (evc, nwordwfc, iunwfc, ik - iks + 1, -1)
+    CALL davcio (evc, 2*nwordwfc, iunwfc, ik - iks + 1, -1)
     IF (ndiag .GT. 0) THEN
       DO ib = diag_nmin, diag_nmax
         psic (:) = (0.0D0, 0.0D0)
@@ -2106,7 +2207,6 @@ SUBROUTINE write_vscg ( output_file_name, real_or_complex, symm_type )
 
   USE cell_base, ONLY : omega, alat, tpiba, tpiba2, at, bg, ibrav
   USE constants, ONLY : pi, tpi, eps6
-  USE ener, ONLY : etxc, vtxc
   USE fft_base, ONLY : dfftp
   USE fft_interfaces, ONLY : fwfft
   USE gvect, ONLY : ngm, ngm_g, ig_l2g, nl, mill, ecutrho
@@ -2115,7 +2215,7 @@ SUBROUTINE write_vscg ( output_file_name, real_or_complex, symm_type )
   USE kinds, ONLY : DP
   USE lsda_mod, ONLY : nspin
   USE mp, ONLY : mp_sum
-  USE mp_global, ONLY : intra_pool_comm
+  USE mp_pools, ONLY : intra_pool_comm
   USE scf, ONLY : vltot, v
   USE symm_base, ONLY : s, ftau, nsym
   USE wavefunctions_module, ONLY : psic
@@ -2139,8 +2239,6 @@ SUBROUTINE write_vscg ( output_file_name, real_or_complex, symm_type )
 
   INTEGER, EXTERNAL :: atomic_number
 
-  CALL check_inversion ( real_or_complex, nsym, s, nspin, .true., .true. )
-
   CALL date_and_tim ( cdate, ctime )
   WRITE ( sdate, '(A2,"-",A3,"-",A4,21X)' ) cdate(1:2), cdate(3:5), cdate(6:9)
   WRITE ( stime, '(A8,24X)' ) ctime(1:8)
@@ -2208,6 +2306,8 @@ SUBROUTINE write_vscg ( output_file_name, real_or_complex, symm_type )
     ENDDO
   ENDDO
 
+  CALL check_inversion ( real_or_complex, nsym, s, nspin, .true., .true., translation )
+
   alat2 = alat ** 2
   recvol = 8.0D0 * pi**3 / omega
 
@@ -2324,8 +2424,9 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
   USE klist, ONLY : xk, wk, ngk, nks, nkstot
   USE lsda_mod, ONLY : nspin, isk
   USE mp, ONLY : mp_sum, mp_max, mp_get, mp_barrier
-  USE mp_global, ONLY : mpime, nproc, world_comm, kunit, me_pool, &
-    root_pool, my_pool_id, npool, nproc_pool, intra_pool_comm
+  USE mp_world, ONLY : mpime, nproc, world_comm
+  USE mp_pools, ONLY : kunit, me_pool, root_pool, my_pool_id, npool, &
+                       nproc_pool, intra_pool_comm
   USE mp_wave, ONLY : mergewf
   USE start_k, ONLY : nk1, nk2, nk3, k1, k2, k3
   USE symm_base, ONLY : s, ftau, nsym
@@ -2480,7 +2581,7 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
     IF ( ik .GE. iks .AND. ik .LE. ike .AND. is .NE. isk ( ik ) ) &
       ierr = ierr + 1
   ENDDO
-  CALL mp_max ( ierr )
+  CALL mp_max ( ierr, world_comm )
   IF ( ierr .GT. 0 ) &
     CALL errore ( 'write_vkbg', 'smap', ierr )
 
@@ -2512,9 +2613,9 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
   DO ik = 1, nks
     ngk_g ( ik + iks - 1 ) = ngk ( ik )
   ENDDO
-  CALL mp_sum ( ngk_g )
+  CALL mp_sum ( ngk_g, world_comm )
   npw_g = MAXVAL ( igk_l2g ( :, : ) )
-  CALL mp_max ( npw_g )
+  CALL mp_max ( npw_g, world_comm )
   npwx_g = MAXVAL ( ngk_g ( : ) )
 
   CALL cryst_to_cart (nkstot, xk, at, -1)
@@ -2569,7 +2670,7 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
         itmp ( igk_l2g ( ig, ik - iks + 1 ) ) = igk_l2g ( ig, ik - iks + 1 )
       ENDDO
     ENDIF
-    CALL mp_sum ( itmp )
+    CALL mp_sum ( itmp, world_comm )
     ngg = 0
     DO ig = 1, npw_g
       IF ( itmp ( ig ) .EQ. ig ) THEN
@@ -2618,7 +2719,7 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
       IF ( ( ik .GE. iks ) .AND. ( ik .LE. ike ) ) THEN
         IF ( me_pool .EQ. root_pool ) ipmask ( mpime + 1 ) = 1
       ENDIF
-      CALL mp_sum ( ipmask )
+      CALL mp_sum ( ipmask, world_comm )
       DO j = 1, nproc
         IF ( ipmask ( j ) .EQ. 1 ) ipsour = j - 1
       ENDDO
@@ -2631,11 +2732,11 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
       igwx = MAXVAL ( igwf_l2g ( 1 : local_pw ) )
     CALL mp_max ( igwx, intra_pool_comm )
     IF ( ipsour .NE. ionode_id ) &
-      CALL mp_get ( igwx, igwx, mpime, ionode_id, ipsour, 1 )
+      CALL mp_get ( igwx, igwx, mpime, ionode_id, ipsour, 1, world_comm )
     ierr = 0
     IF ( ik .GE. iks .AND. ik .LE. ike .AND. igwx .NE. ngk_g ( ik ) ) &
       ierr = 1
-    CALL mp_max ( ierr )
+    CALL mp_max ( ierr, world_comm )
     IF ( ierr .GT. 0 ) &
       CALL errore ( 'write_vkbg', 'igwx ngk_g', ierr )
 
@@ -2688,26 +2789,33 @@ END SUBROUTINE write_vkbg
 
 !-------------------------------------------------------------------------------
 
-subroutine check_inversion(real_or_complex, ntran, mtrx, nspin, warn, real_need_inv)
+subroutine check_inversion(real_or_complex, ntran, mtrx, nspin, warn, real_need_inv, tnp)
 
-! check_inversion    Originally By D. Strubbe    Last Modified 10/14/2010
+! check_inversion    Originally By David A. Strubbe    Last Modified 11/18/2013
 ! Check whether our choice of real/complex version is appropriate given the
 ! presence or absence of inversion symmetry.
 
+  USE constants, ONLY : eps6
   USE io_global, ONLY : ionode
+  USE kinds, ONLY : DP
 
   implicit none
 
   integer, intent(in) :: real_or_complex
   integer, intent(in) :: ntran
-  integer, intent(in) :: mtrx(3, 3, 48)
+  integer, intent(in) :: mtrx(3, 3, 48) !< symmetry operations matrices
   integer, intent(in) :: nspin
-  logical, intent(in) :: warn ! set to false to suppress warnings, for converters
-  logical, intent(in) :: real_need_inv ! use for generating routines to block real without inversion
+  logical, intent(in) :: warn !< set to false to suppress warnings, for converters
+  logical, intent(in) :: real_need_inv !< use for generating routines to block real without inversion
+     !! this is not always true so that it is possible to run real without using symmetries
+  real(DP), intent(in) :: tnp(3, 48) !< fractional translations.
+     !! optional only to avoid changing external interface for library.
 
   integer :: invflag, isym, ii, jj, itest
+  logical :: origin_inv
 
   invflag = 0
+  origin_inv = .false.
   do isym = 1, ntran
     itest = 0
     do ii = 1, 3
@@ -2719,27 +2827,43 @@ subroutine check_inversion(real_or_complex, ntran, mtrx, nspin, warn, real_need_
         endif
       enddo
     enddo
-    if(itest .eq. 0) invflag = invflag + 1
-    if(invflag .gt. 1) call errore('check_inversion', 'More than one inversion symmetry operation is present.', invflag)
+    if(itest .eq. 0) then
+      invflag = invflag + 1
+      !if(present(tnp)) then
+        if(sum(abs(tnp(1:3, isym))) < eps6) origin_inv = .true.
+      !else
+      !  origin_inv = .true.
+      !endif
+    endif
   enddo
+  if(invflag > 0 .and. .not. origin_inv) then
+    write(0, '(a)') "WARNING: Inversion symmetry is present only with a fractional translation."
+    write(0, '(a)') "Apply the translation so inversion is about the origin, to be able to use the real version."
+  endif
+  if(invflag .gt. 1) write(0, '(a)') "WARNING: More than one inversion symmetry operation is present."
+
+!  if(invflag > 0 .and. .not. present(tnp)) then
+!    write(0, '(a)') "WARNING: check_inversion did not receive fractional translations."
+!    write(0, '(a)') "Cannot confirm that inversion symmetry is about the origin for use of real version."
+!  endif
 
   if(real_or_complex .eq. 2) then
-    if(invflag .ne. 0 .and. warn .and. nspin == 1) then
-      if(ionode) write(6, '(a)') 'WARNING: Inversion symmetry is present. The real version would be faster.'
+    if(origin_inv .and. warn .and. nspin == 1) then
+      if(ionode) &
+        write(0, '(a)') "WARNING: Inversion symmetry about the origin is present. The real version would be faster."
     endif
   else
-    if(invflag .eq. 0) then
+    if(.not. origin_inv) then
       if(real_need_inv) then
-        call errore('check_inversion', 'The real version cannot be used without inversion symmetry.', -1)
+        call errore("check_inversion", "The real version cannot be used without inversion symmetry about the origin.", -1)
       endif
       if(ionode) then
-        write(6, '(a)') 'WARNING: Inversion symmetry is absent in symmetries used to reduce k-grid.'
-        write(6, '(a)') 'Be sure inversion is still a spatial symmetry, or you must use complex version instead.'
+        write(0, '(a)') "WARNING: Inversion symmetry about the origin is absent in symmetries used to reduce k-grid."
+        write(0, '(a)') "Be sure inversion about the origin is still a spatial symmetry, or you must use complex version instead."
       endif
     endif
-    if(nspin .eq. 2) then
-      call errore('check_inversion', &
-        'Real version may only be used for spin-unpolarized calculations.', nspin)
+    if(nspin > 1) then
+      call errore("check_inversion", "Real version may only be used for spin-unpolarized calculations.", nspin)
     endif
   endif
 
@@ -2749,3 +2873,5 @@ end subroutine check_inversion
 
 !-------------------------------------------------------------------------------
 
+END PROGRAM pw2bgw
+
