@@ -38,13 +38,14 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
    USE klist,                ONLY : nelec, degauss, nks, xk, wk
    USE wvfct,                ONLY : npwx, npw, nbnd, ecutwfc
    USE noncollin_module,     ONLY : noncolin, npol
-   USE wavefunctions_module, ONLY : evc
    USE bp,                   ONLY : nppstr_3d, mapgm_global, nx_el
    USE fixed_occ
    USE gvect,   ONLY : ig_l2g
    USE mp,                   ONLY : mp_sum
-   USE mp_global,            ONLY : intra_bgrp_comm
-   USE becmod,               ONLY : calbec
+   USE mp_bands,             ONLY : intra_bgrp_comm
+   USE mp_world,             ONLY : world_comm
+   USE becmod,    ONLY : calbec,bec_type,allocate_bec_type,deallocate_bec_type
+   USE spin_orb, ONLY: lspinorb
 !  --- Avoid implicit definitions ---
    IMPLICIT NONE
 
@@ -113,14 +114,12 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
    REAL(dp), ALLOCATABLE :: wstring(:)
    REAL(dp) :: ylm_dk(lmaxq*lmaxq)
    REAL(dp) :: zeta_mod
-   COMPLEX(dp), ALLOCATABLE :: aux(:)
-   COMPLEX(dp), ALLOCATABLE :: aux0(:)
+   COMPLEX(dp), ALLOCATABLE :: aux(:,:)
+   COMPLEX(dp), ALLOCATABLE :: aux0(:,:)
    ! For noncollinear calculations
-   COMPLEX(dp), ALLOCATABLE :: aux_2(:)
-   COMPLEX(dp), ALLOCATABLE :: aux0_2(:)
-   COMPLEX(dp) :: becp0(nkb,nbnd)
-   COMPLEX(dp) :: becp_bp(nkb,nbnd)
-   COMPLEX(dp) , ALLOCATABLE :: cphik(:)
+   COMPLEX(dp), ALLOCATABLE :: aux_2(:,:)
+   COMPLEX(dp), ALLOCATABLE :: aux0_2(:,:)
+    COMPLEX(dp) , ALLOCATABLE :: cphik(:)
    COMPLEX(dp) :: det
    COMPLEX(dp) :: mat(nbnd,nbnd)
    COMPLEX(dp) :: pref
@@ -143,20 +142,24 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
    COMPLEX(kind=DP) :: sca
    COMPLEX(kind=DP), ALLOCATABLE :: aux_g(:)
    COMPLEX(kind=DP), ALLOCATABLE :: aux_g_2(:) ! noncollinear case
-
+   TYPE(bec_type) :: becp0, becp_bp
+   COMPLEX(DP), ALLOCATABLE :: q_dk_so(:,:,:,:)
 
 !  -------------------------------------------------------------------------   !
 !                               INITIALIZATIONS
 !  -------------------------------------------------------------------------   !
+
+   call start_clock('c_phase_field')
+
    ALLOCATE (psi1(npol*npwx,nbnd))
    ALLOCATE (psi(npol*npwx,nbnd))
-   ALLOCATE (aux(ngm))
-   ALLOCATE (aux0(ngm))
+   ALLOCATE (aux(ngm,nbnd))
+   ALLOCATE (aux0(ngm,nbnd))
    nspinnc=nspin
    IF (noncolin) THEN
       nspinnc=1
-      ALLOCATE (aux_2(ngm))
-      ALLOCATE (aux0_2(ngm))
+      ALLOCATE (aux_2(ngm,nbnd))
+      ALLOCATE (aux0_2(ngm,nbnd))
    END IF
    ALLOCATE (map_g(npwx))
    ALLOCATE (l_cal(nbnd))
@@ -167,6 +170,12 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
    endif
 
 
+   if(okvan) then
+      call allocate_bec_type(nkb,nbnd,becp0)
+      call allocate_bec_type(nkb,nbnd,becp_bp)
+      IF (lspinorb) ALLOCATE(q_dk_so(nhm,nhm,4,ntyp))
+   endif
+   
    pola=0.d0 !set to 0 electronic polarization   
    zeta_tot=(1.d0,0.d0)
 
@@ -300,6 +309,7 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
             ENDDO
          endif
       ENDDO
+      IF (lspinorb) CALL transform_qq_so(q_dk,q_dk_so)
    endif
    
 !  -------------------------------------------------------------------------   !
@@ -317,7 +327,11 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
          IF ( nspin == 2 .AND. tfixed_occ) THEN
             l_cal(nb) = ( f_inp(nb,is) /= 0.0_dp )
          ELSE
-            l_cal(nb) = ( nb <= NINT ( nelec/2.0_dp ) )
+             IF (noncolin) THEN
+                l_cal(nb) = ( nb <= NINT ( nelec) )
+             ELSE
+                l_cal(nb) = ( nb <= NINT ( nelec/2.0_dp ) )
+             ENDIF
          ENDIF
       END DO
        
@@ -417,21 +431,21 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
                ENDIF
 
                mat=(0.d0,0.d0)
+               aux=(0.d0,0.d0)
+               if(noncolin) aux_2=(0.d0,0.d0)
                DO mb=1,nbnd
                   IF ( .NOT. l_cal(mb) ) THEN
                      mat(mb,mb)=(1.d0, 0.d0)
                   ELSE
-                     aux=(0.d0,0.d0)
-                     IF (noncolin) aux_2=(0.d0,0.d0)
                      IF (kpar /= (nppstr_3d(pdir)+1)) THEN
                         DO ig=1,npw1
-                           aux(igk1(ig))=psi1(ig,mb)
-                           IF (noncolin) aux_2(igk1(ig))=psi1(ig+npwx,mb)
+                           aux(igk1(ig),mb)=psi1(ig,mb)
+                           IF (noncolin) aux_2(igk1(ig),mb)=psi1(ig+npwx,mb)
                         ENDDO
                      ELSE IF( .not. l_para) THEN
                         DO ig=1,npw1
-                           aux(map_g(ig))=psi1(ig,mb)
-                           IF (noncolin) aux_2(map_g(ig))=psi1(ig+npwx,mb)
+                           aux(map_g(ig),mb)=psi1(ig,mb)
+                           IF (noncolin) aux_2(map_g(ig),mb)=psi1(ig+npwx,mb)
                         ENDDO
                      ELSE
 ! allocate global array
@@ -444,30 +458,43 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
                            aux_g(mapgm_global(ig_l2g(igk1(ig)),pdir))=psi1(ig,mb)
                            IF(noncolin) aux_g_2(mapgm_global(ig_l2g(igk1(ig)),pdir))=psi1(ig+npwx,mb)
                         ENDDO
-                        CALL mp_sum(aux_g(:))
-                        IF (noncolin) CALL mp_sum(aux_g_2(:)) !non-collinear
+                        CALL mp_sum(aux_g(:),world_comm)
+                        IF (noncolin) CALL mp_sum(aux_g_2(:),world_comm) !non-collinear
                         DO ig=1,ngm
-                           aux(ig) = aux_g(ig_l2g(ig))
-                           IF (noncolin) aux_2(ig) = aux_g_2(ig_l2g(ig))
+                           aux(ig,mb) = aux_g(ig_l2g(ig))
+                           IF (noncolin) aux_2(ig,mb) = aux_g_2(ig_l2g(ig))
                         ENDDO
                         DEALLOCATE (aux_g)
                         IF(noncolin) DEALLOCATE (aux_g_2)
                      END IF
-                     DO nb=1,nbnd
-                        IF ( l_cal(nb) ) THEN
-                           aux0=(0.d0,0.d0)
-                           IF(noncolin) aux0_2=(0.d0,0.d0)
-                           DO ig=1,npw0
-                              aux0(igk0(ig))=psi(ig,nb)
-                              IF(noncolin) aux0_2(igk0(ig))=psi(ig+npwx,nb)
-                           END DO
-! do scalar product
-                           mat(nb,mb) = zdotc(ngm,aux0,1,aux,1)
-                           IF (noncolin) mat(nb,mb) = mat(nb,mb)+zdotc(ngm,aux0_2,1,aux_2,1)
-                        END IF                           
-                     ENDDO
                   END IF
+               END DO
+               aux0=(0.d0,0.d0)
+               if(noncolin) aux0_2=(0.d0,0.d0)
+               DO nb=1,nbnd
+                  DO ig=1,npw0
+                     aux0(igk0(ig),nb)=psi(ig,nb)
+                     IF(noncolin) aux0_2(igk0(ig),nb)=psi(ig+npwx,nb)
+                  END DO
                ENDDO
+               call ZGEMM('C','N',nbnd,nbnd,ngm,(1.d0,0.d0),aux0,ngm,aux,ngm,(0.d0,0.d0),mat,nbnd)
+               if(noncolin) call ZGEMM('C','N',nbnd,nbnd,ngm,(1.d0,0.d0),aux0_2,ngm,aux_2,ngm,(1.d0,0.d0),mat,nbnd)
+
+           !    DO nb=1,nbnd
+           !       IF ( l_cal(nb) ) THEN
+           !                aux0=(0.d0,0.d0)
+           !                IF(noncolin) aux0_2=(0.d0,0.d0)
+           !                DO ig=1,npw0
+           !                   aux0(igk0(ig))=psi(ig,nb)
+           !                   IF(noncolin) aux0_2(igk0(ig))=psi(ig+npwx,nb)
+           !                END DO
+! do scalar product
+           !                mat(nb,mb) = zdotc(ngm,aux0,1,aux,1)
+           !                IF (noncolin) mat(nb,mb) = mat(nb,mb)+zdotc(ngm,aux0_2,1,aux_2,1)
+           !             END IF                           
+           !          ENDDO
+           !       END IF
+           !    ENDDO
 !
                CALL  mp_sum( mat, intra_bgrp_comm )
 
@@ -487,8 +514,21 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
                               nhjkbm = nh(np)
                               jkb1 = jkb - nhjkb
                               DO j = 1,nhjkbm
-                                 pref = pref+CONJG(becp0(jkb,nb))*becp_bp(jkb1+j,mb) &
-                                      *q_dk(nhjkb,j,np)*struc(na)
+                                 if(lspinorb) then
+                                    pref = pref+CONJG(becp0%nc(jkb,1,nb))*becp_bp%nc(jkb1+j,1,mb) &
+                                         *q_dk_so(nhjkb,j,1,np)*struc(na)
+                                    pref = pref+CONJG(becp0%nc(jkb,1,nb))*becp_bp%nc(jkb1+j,2,mb) &
+                                         *q_dk_so(nhjkb,j,2,np)*struc(na)
+                                    pref = pref+CONJG(becp0%nc(jkb,2,nb))*becp_bp%nc(jkb1+j,1,mb) &
+                                         *q_dk_so(nhjkb,j,3,np)*struc(na)
+                                    pref = pref+CONJG(becp0%nc(jkb,2,nb))*becp_bp%nc(jkb1+j,2,mb) &
+                                         *q_dk_so(nhjkb,j,4,np)*struc(na)
+                                    
+                                 else
+
+                                    pref = pref+CONJG(becp0%k(jkb,nb))*becp_bp%k(jkb1+j,mb) &
+                                         *q_dk(nhjkb,j,np)*struc(na)
+                                 endif
                               ENDDO
                            ENDDO
                       
@@ -586,6 +626,14 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
    DEALLOCATE(psi1)
    IF (ALLOCATED(aux_2)) DEALLOCATE(aux_2)
    IF (ALLOCATED(aux0_2)) DEALLOCATE(aux0_2)
+
+   if(okvan) then
+      call deallocate_bec_type(becp0)
+      call deallocate_bec_type(becp_bp)
+      IF (lspinorb) DEALLOCATE(q_dk_so)
+   endif
+   call stop_clock('c_phase_field')
+
 !------------------------------------------------------------------------------!
 
 END SUBROUTINE c_phase_field
