@@ -42,12 +42,12 @@ INTEGER :: &
           elem(8,12),  &   ! Which elements in the smat list for each class
           which_irr(12)    ! See above 
 
-REAL(DP) :: smat(3,3,nrot), cmat(3,3), ax(3), ars
+REAL(DP) :: smat(3,3,nrot), cmat(3,3), ax(3), bx(3), ars
 
-INTEGER :: done(48), irot, jrot, krot, iclass, i
+INTEGER :: done(48), irot, jrot, krot, iclass, i, other, other1
 INTEGER :: tipo_sym, ipol, axis, axis1, axis2, ts
 REAL(DP), PARAMETER :: eps = 1.d-7
-REAL(DP) :: angle_rot, angle_rot_s, ax_save(3,2:4)
+REAL(DP) :: angle_rot, angle_rot_s, angle_vectors, ax_save(3,2:4)
 LOGICAL :: compare_mat, is_axis, is_parallel, first, first1, done_ax(6)
 !
 ! Divide the group in classes.
@@ -584,11 +584,23 @@ ELSEIF (code_group==23) THEN
          IF (nelem(iclass)==1) THEN
             which_irr(iclass)=4
          ELSE
-            IF (first) THEN
+            IF (first.AND.first1) THEN
                which_irr(iclass)=5
+               other=6
+               CALL versor(smat(1,1,elem(1,iclass)),ax)
+               first=.FALSE.
+            ELSEIF (first) THEN
+               CALL versor(smat(1,1,elem(1,iclass)),bx)
+               IF (MOD(INT(angle_vectors(ax,bx)),60)==0) THEN
+                  which_irr(iclass)=5
+                  other=6
+               ELSE
+                  which_irr(iclass)=6
+                  other=5
+               ENDIF
                first=.FALSE.
             ELSE
-               which_irr(iclass)=6
+               which_irr(iclass)=other
             END IF 
          END IF
       ELSE IF (ts==2) THEN
@@ -597,12 +609,24 @@ ELSEIF (code_group==23) THEN
           IF (nelem(iclass)==1) THEN
              which_irr(iclass)=10
           ELSE 
-             IF (first1) THEN
+             IF (first.AND.first1) THEN
                 which_irr(iclass)=11
+                other1=12
+                CALL mirror_axis(smat(1,1,elem(1,iclass)),ax)
+                first1=.FALSE.
+             ELSEIF (first1) THEN
+                CALL mirror_axis(smat(1,1,elem(1,iclass)),bx)
+                IF (MOD(INT(angle_vectors(ax,bx)),60)==0) THEN
+                   which_irr(iclass)=11
+                   other1=12
+                ELSE
+                   which_irr(iclass)=12
+                   other1=11
+                ENDIF
                 first1=.FALSE.
              ELSE
-                which_irr(iclass)=12
-             ENDIF
+                which_irr(iclass)=other1
+             END IF 
           END IF
       ELSE IF (ts==6) THEN
          ars=angle_rot_s(smat(1,1,elem(1,iclass)))
@@ -720,7 +744,7 @@ ELSE IF (code_group==29) THEN
 !
 !  T_h
 !
-   IF (nclass /= 4) CALL errore('divide_class','Wrong classes for T_h',1)
+   IF (nclass /= 8) CALL errore('divide_class','Wrong classes for T_h',1)
    DO iclass=2,nclass
       ts=tipo_sym(smat(1,1,elem(1,iclass)))
       IF (ts==3) THEN
@@ -925,7 +949,7 @@ IF (ts==4) THEN
 !   then the general case
 !
    DO ipol=1,3
-      ax(ipol)=sqrt((smat(ipol,ipol)+1.d0)/2.d0)
+      ax(ipol)=sqrt(ABS(smat(ipol,ipol)+1.d0)/2.d0)
    END DO
    
    DO ipol=1,3
@@ -2673,3 +2697,25 @@ is_parallel=(ABS(cross)< 1.d-6)
 RETURN
 END FUNCTION is_parallel
 
+FUNCTION angle_vectors(ax,bx)
+!
+!  This function returns the angle, in degrees between two vectors
+!
+USE kinds, ONLY : DP
+USE constants, ONLY : pi
+IMPLICIT none
+REAL(DP) :: angle_vectors
+REAL(DP) :: ax(3), bx(3)
+REAL(DP) :: cosangle, moda, modb
+
+moda=sqrt(ax(1)**2+ax(2)**2+ax(3)**2)
+modb=sqrt(bx(1)**2+bx(2)**2+bx(3)**2)
+
+IF (moda<1.d-12.OR.modb<1.d-12) &
+   CALL errore('angle vectors','zero module vector',1)
+
+cosangle = (ax(1)*bx(1)+ax(2)*bx(2)+ax(3)*bx(3))/moda/modb
+angle_vectors = acos(cosangle) * 180.d0 / pi
+
+RETURN
+END FUNCTION angle_vectors
