@@ -17,6 +17,7 @@ subroutine h_epsi_her_set(pdir, e_field)
   ! spin polarized systems supported only with fixed occupations
 
   USE noncollin_module,     ONLY : noncolin, npol
+  USE spin_orb, ONLY: lspinorb
   USE kinds,    ONLY : DP
   USE us
   USE wvfct,    ONLY : igk, g2kin, npwx, npw, nbnd, ecutwfc
@@ -29,17 +30,16 @@ subroutine h_epsi_her_set(pdir, e_field)
   USE uspp_param, ONLY: upf, nh, nhm, nbetam, lmaxq
   USE bp,         ONLY : nppstr_3d, fact_hepsi, evcel, evcp=>evcelp, &
                          evcm=>evcelm, mapgp_global, mapgm_global, nx_el
-  USE basis
   USE klist
   USE cell_base, ONLY: at, alat, tpiba, omega, tpiba2,bg
   USE ions_base, ONLY: ityp, tau, nat,ntyp => nsp
   USE io_files,  ONLY: iunwfc, nwordwfc, iunefieldm, iunefieldp
-  USE buffers,   ONLY: get_buffer
+  USE buffers,   ONLY: get_buffer, save_buffer
   USE constants, ONLY : e2, pi, tpi, fpi
   USE fixed_occ
   USE mp,        ONLY : mp_sum
-  USE mp_global, ONLY : intra_bgrp_comm
-  USE becmod,    ONLY : calbec
+  USE mp_bands,  ONLY : intra_bgrp_comm
+  USE becmod,    ONLY : bec_type, becp, calbec,allocate_bec_type, deallocate_bec_type
   !
   implicit none
   !
@@ -49,7 +49,7 @@ subroutine h_epsi_her_set(pdir, e_field)
   !  --- Internal definitions ---
 
    COMPLEX(DP), ALLOCATABLE  :: evct(:,:)!for temporary wavefunctios
-   INTEGER :: i
+   INTEGER :: i,ipol
    INTEGER :: igk1(npwx)
    INTEGER :: igk0(npwx)
    INTEGER :: ig
@@ -100,12 +100,11 @@ subroutine h_epsi_her_set(pdir, e_field)
    COMPLEX(dp), ALLOCATABLE  :: aux0(:)
    ! Also for noncollinear calculation
    COMPLEX(DP), ALLOCATABLE :: aux_2(:)
-   COMPLEX(DP), ALLOCATABLE :: aux0_2(:)
-   COMPLEX(dp) :: becp0(nkb,nbnd)
-   COMPLEX(dp) :: becp_bp(nkb,nbnd)
+   COMPLEX(DP), ALLOCATABLE :: aux0_2(:),aux0vec(:,:),aux1vec(:,:)
+  
    COMPLEX(dp) :: cdet(2)
    COMPLEX(dp) :: cdwork(nbnd)
-   COMPLEX(dp) :: mat(nbnd,nbnd)
+   COMPLEX(dp), ALLOCATABLE :: mat(:,:)
    COMPLEX(dp) :: pref
    COMPLEX(dp) :: q_dk(nhm,nhm,ntyp)
    COMPLEX(dp) :: q_dkp(nhm,nhm,ntyp)!to store the terms T^dagger e^(iGx) T
@@ -114,7 +113,7 @@ subroutine h_epsi_her_set(pdir, e_field)
 
    
    COMPLEX(dp) :: sca,sca1
-   COMPLEX(dp) :: ps(nkb,nbnd)
+   COMPLEX(dp) :: ps(nkb,nbnd*npol)
    COMPLEX(dp) :: matbig(nks,nbnd,nbnd)
    INTEGER :: mdone(nks)
    INTEGER :: ijkb0,  ibnd,jh, ih, ikb, ik
@@ -122,15 +121,35 @@ subroutine h_epsi_her_set(pdir, e_field)
 
    LOGICAL, ALLOCATABLE :: l_cal(:) ! flag for empty/occupied states
    INTEGER, ALLOCATABLE  :: map_g(:)
+   TYPE(bec_type) :: becp_bp,becp0
+     
 
    REAL(dp) :: dkfact
    LOGICAL  :: l_para! if true new parallel treatment
    COMPLEX(kind=DP), ALLOCATABLE :: aux_g(:)
    COMPLEX(kind=DP), ALLOCATABLE :: aux_g_2(:) ! non-collinear case
+
+   COMPLEX(DP), ALLOCATABLE :: q_dk_so(:,:,:,:),q_dkp_so(:,:,:,:)
+
+
+
    !  --- Define a small number ---
    eps=0.000001d0
 
+   allocate(mat(nbnd,nbnd))
+
    if(ABS(e_field)<eps) return
+   call start_clock('h_epsi_set')
+
+
+
+  if(okvan) then
+      CALL allocate_bec_type (nkb,nbnd,becp0)
+     CALL allocate_bec_type (nkb,nbnd,becp_bp)
+     IF (lspinorb) ALLOCATE(q_dk_so(nhm,nhm,4,ntyp))
+     IF (lspinorb) ALLOCATE(q_dkp_so(nhm,nhm,4,ntyp))
+  endif
+
 
 !  -------------------------------------------------------------------------   !
 !                               INITIALIZATIONS
@@ -207,12 +226,16 @@ subroutine h_epsi_her_set(pdir, e_field)
          IF ( nspin == 2 .AND. tfixed_occ) THEN
             l_cal(nb) = ( f_inp(nb,is) /= 0.0_dp )
          ELSE
-            l_cal(nb) = ( nb <= NINT ( nelec/2.0_dp ) )
+            IF (noncolin) THEN
+                l_cal(nb) = ( nb <= NINT ( nelec) )
+             ELSE
+                l_cal(nb) = ( nb <= NINT ( nelec/2.0_dp ) )
+             END IF
          ENDIF
       END DO
 
       ik_stringa=mod(ik-1,nppstr_3d(pdir))+1
-      !nstring=nks/nppstr_3d(pdir)
+     
  
       if(okvan) then
 !  --- Initialize arrays ---
@@ -279,7 +302,7 @@ subroutine h_epsi_her_set(pdir, e_field)
                ENDDO
             endif
          ENDDO
-         
+         IF (lspinorb) CALL transform_qq_so(q_dk,q_dk_so)
 !  --- Calculate the q-space real spherical harmonics at -dk [Y_LM] --- 
          dkmod=dkm(1)**2+dkm(2)**2+dkm(3)**2
          CALL ylmr2(lmaxq*lmaxq, 1, dkm, dkmod, ylm_dk)
@@ -298,6 +321,7 @@ subroutine h_epsi_her_set(pdir, e_field)
                ENDDO
             endif
          ENDDO
+         IF (lspinorb) CALL transform_qq_so(q_dkp,q_dkp_so)
       
       endif
 
@@ -315,7 +339,7 @@ subroutine h_epsi_her_set(pdir, e_field)
 
 !       
       if(ik_stringa /= 1) then
-         
+
          CALL gk_sort(xk(1,nx_el(ik-1,pdir)),ngm,g,ecutwfc/tpiba2, &
               &    npw0,igk0,g2kin_bp) 
          CALL get_buffer (evct,nwordwfc,iunwfc,nx_el(ik-1,pdir))
@@ -371,7 +395,7 @@ subroutine h_epsi_her_set(pdir, e_field)
 
                   DO ig=1,npw0
                      aux(igk0(ig))=evct(ig,mb)
-                     IF (noncolin) aux(igk0(ig))=evct(ig+npwx,mb)
+                     IF (noncolin) aux_2(igk0(ig))=evct(ig+npwx,mb)
                   END DO
                    
                   mat(nb,mb) = zdotc(ngm,aux0,1,aux,1)
@@ -397,10 +421,26 @@ subroutine h_epsi_her_set(pdir, e_field)
                         np = ityp(na)
                         nhjkbm = nh(np)
                         jkb1 = jkb - nhjkb
-                        DO j = 1,nhjkbm
-                           pref = pref+CONJG(becp_bp(jkb,nb))*becp0(jkb1+j,mb) &
-                     &        *q_dkp(nhjkb,j,np)*CONJG(struc(na))
-                        ENDDO
+                        if(lspinorb) THEN
+                           DO j = 1,nhjkbm
+                              pref = pref+CONJG(becp_bp%nc(jkb,1,nb))* becp0%nc(jkb1+j,1,mb)  &
+                                   *q_dkp_so(nhjkb,j,1,np)*CONJG(struc(na))   
+                              pref= pref+CONJG(becp_bp%nc(jkb,1,nb))* becp0%nc(jkb1+j,2,mb)  &
+                                   *q_dkp_so(nhjkb,j,2,np)*CONJG(struc(na)) 
+                              pref= pref +CONJG(becp_bp%nc(jkb,2,nb))* becp0%nc(jkb1+j,1,mb)  &
+                                   *q_dkp_so(nhjkb,j,3,np)*CONJG(struc(na)) 
+                              pref= pref +CONJG(becp_bp%nc(jkb,2,nb))* becp0%nc(jkb1+j,2,mb)   &
+                                   *q_dkp_so(nhjkb,j,4,np)*CONJG(struc(na))
+
+                           ENDDO
+
+                        ELSE
+                           DO j = 1,nhjkbm
+                              pref = pref+CONJG(becp_bp%k(jkb,nb))*becp0%k(jkb1+j,mb) &
+                                   &        *q_dkp(nhjkb,j,np)*CONJG(struc(na))
+                           ENDDO
+
+                        END IF
                      ENDDO
                      mat(nb,mb) = mat(nb,mb) + pref
                   endif
@@ -459,8 +499,20 @@ subroutine h_epsi_her_set(pdir, e_field)
                            jkb = ijkb0 + jh
                            do ih = 1, nh (nt)
                               ikb = ijkb0 + ih
-                              ps (ikb, ibnd) = ps (ikb, ibnd) + &
-                                q_dkp(ih,jh,ityp(na))*CONJG(struc(na))* becp0(jkb,ibnd)
+                              if(lspinorb) then
+                                   ps (ikb, (ibnd-1)*npol+1) = ps (ikb,(ibnd-1)*npol+1 ) + &
+                                      q_dkp_so(ih,jh,1,ityp(na))*CONJG(struc(na))* becp0%nc(jkb,1,ibnd)
+                                   ps (ikb, (ibnd-1)*npol+1) = ps (ikb,(ibnd-1)*npol+1 ) + &
+                                        q_dkp_so(ih,jh,2,ityp(na))*CONJG(struc(na))* becp0%nc(jkb,2,ibnd)
+                                   ps (ikb, (ibnd-1)*npol+2) = ps (ikb,(ibnd-1)*npol+2 ) + &
+                                        q_dkp_so(ih,jh,3,ityp(na))*CONJG(struc(na))* becp0%nc(jkb,1,ibnd)
+                                   ps (ikb, (ibnd-1)*npol+2) = ps (ikb,(ibnd-1)*npol+2 ) + &
+                                        q_dkp_so(ih,jh,4,ityp(na))*CONJG(struc(na))* becp0%nc(jkb,2,ibnd)
+
+                              else
+                                 ps (ikb, ibnd) = ps (ikb, ibnd) + &
+                                      q_dkp(ih,jh,ityp(na))*CONJG(struc(na))* becp0%k(jkb,ibnd)
+                              endif
                            enddo
                         enddo
                      enddo
@@ -469,22 +521,26 @@ subroutine h_epsi_her_set(pdir, e_field)
                enddo
             enddo
 
-            call ZGEMM ('N', 'N', npw1, nbnd , nkb, (1.d0, 0.d0) , vkb, &!vkb is relative to the last ik read
+            call ZGEMM ('N', 'N', npw1, nbnd*npol , nkb, (1.d0, 0.d0) , vkb, &!vkb is relative to the last ik read
                  npwx, ps, nkb, (1.d0, 0.d0) , evct, npwx)
             do m=1,nbnd
                do nb=1,nbnd
                   do ig=1,npw1
                      evcm(ig,m,pdir)=evcm(ig,m,pdir) + mat(nb,m)*evct(ig,nb)
                   enddo
+                  if(noncolin) then
+                     do ig=1,npw1
+                        evcm(ig+npwx,m,pdir)=evcm(ig+npwx,m,pdir) + mat(nb,m)*evct(ig+npwx,nb)
+                     enddo
+                  endif
                enddo
             enddo
          endif
-    
+
  
 !           --- End of dot products between wavefunctions and betas ---
       ELSE !(ik_stringa == 1)
-         
-
+      
          CALL gk_sort(xk(1,nx_el(ik+nppstr_3d(pdir)-1,pdir)),ngm,g,ecutwfc/tpiba2, &
            &   npw0,igk0,g2kin_bp) 
          CALL get_buffer (evct,nwordwfc,iunwfc,nx_el(ik+nppstr_3d(pdir)-1,pdir))
@@ -585,8 +641,8 @@ subroutine h_epsi_her_set(pdir, e_field)
                   aux_g(mapgp_global(ig_l2g(igk0(ig)),pdir))=evct(ig,mb)
                   IF (noncolin) aux_g_2(mapgp_global(ig_l2g(igk0(ig)),pdir))=evct(ig+npwx,mb)
                 enddo
-                call mp_sum(aux_g(:))
-                IF (noncolin) call mp_sum(aux_g_2(:))
+                call mp_sum(aux_g(:),intra_bgrp_comm)
+                IF (noncolin) call mp_sum(aux_g_2(:),intra_bgrp_comm)
               END IF
 
          DO nb=1,nbnd
@@ -646,9 +702,21 @@ subroutine h_epsi_her_set(pdir, e_field)
                         nhjkbm = nh(np)
                         jkb1 = jkb - nhjkb
                         DO j = 1,nhjkbm
-                           pref = pref+CONJG(becp_bp(jkb,nb))*becp0(jkb1+j,mb) &
-                             *q_dkp(nhjkb,j,np)*CONJG(struc(na))
-                        ENDDO
+                           if(lspinorb) then
+                                pref = pref+CONJG(becp_bp%nc(jkb,1,nb))*becp0%nc(jkb1+j,1,mb) &
+                                     *q_dkp_so(nhjkb,j,1,np)*CONJG(struc(na))
+                                pref = pref+CONJG(becp_bp%nc(jkb,1,nb))*becp0%nc(jkb1+j,2,mb) &
+                                     *q_dkp_so(nhjkb,j,2,np)*CONJG(struc(na))
+                                pref = pref+CONJG(becp_bp%nc(jkb,2,nb))*becp0%nc(jkb1+j,1,mb) &
+                                     *q_dkp_so(nhjkb,j,3,np)*CONJG(struc(na))
+                                pref = pref+CONJG(becp_bp%nc(jkb,2,nb))*becp0%nc(jkb1+j,2,mb) &
+                                     *q_dkp_so(nhjkb,j,4,np)*CONJG(struc(na))
+
+                           else
+                              pref = pref+CONJG(becp_bp%k(jkb,nb))*becp0%k(jkb1+j,mb) &
+                                   *q_dkp(nhjkb,j,np)*CONJG(struc(na))
+                           endif
+                           ENDDO
                      ENDDO
                      mat(nb,mb) = mat(nb,mb) + pref
                   endif
@@ -708,8 +776,8 @@ subroutine h_epsi_her_set(pdir, e_field)
                   IF (noncolin) aux_g_2(mapgp_global(ig_l2g(igk0(ig)),pdir))=evct(ig+npwx,nb)
                enddo
 !put evct on global  array
-               call mp_sum(aux_g(:))
-               IF (noncolin) call mp_sum(aux_g_2(:))
+               call mp_sum(aux_g(:),intra_bgrp_comm)
+               IF (noncolin) call mp_sum(aux_g_2(:),intra_bgrp_comm)
                do m=1,nbnd
                   do ig=1,npw1
                      evcm(ig,m,pdir)=evcm(ig,m,pdir)+mat(nb,m)*aux_g(ig_l2g(igk1(ig)))
@@ -733,8 +801,19 @@ subroutine h_epsi_her_set(pdir, e_field)
                            jkb = ijkb0 + jh
                            do ih = 1, nh (nt)
                               ikb = ijkb0 + ih
-                              ps (ikb, ibnd) = ps (ikb, ibnd) + &
-                                q_dkp(ih,jh,ityp(na))*CONJG(struc(na))* becp0(jkb,ibnd)
+                              if(lspinorb) then
+                                 ps (ikb, (ibnd-1)*npol+1) = ps (ikb,(ibnd-1)*npol+1 ) + &
+                                      q_dkp_so(ih,jh,1,ityp(na))*CONJG(struc(na))* becp0%nc(jkb,1,ibnd)
+                                 ps (ikb, (ibnd-1)*npol+1) = ps (ikb,(ibnd-1)*npol+1 ) + &
+                                      q_dkp_so(ih,jh,2,ityp(na))*CONJG(struc(na))* becp0%nc(jkb,2,ibnd)
+                                 ps (ikb, (ibnd-1)*npol+2) = ps (ikb,(ibnd-1)*npol+2 ) + &
+                                      q_dkp_so(ih,jh,3,ityp(na))*CONJG(struc(na))* becp0%nc(jkb,1,ibnd)
+                                 ps (ikb, (ibnd-1)*npol+2) = ps (ikb,(ibnd-1)*npol+2 ) + &
+                                        q_dkp_so(ih,jh,4,ityp(na))*CONJG(struc(na))* becp0%nc(jkb,2,ibnd)
+                              else
+                                 ps (ikb, ibnd) = ps (ikb, ibnd) + &
+                                      q_dkp(ih,jh,ityp(na))*CONJG(struc(na))* becp0%k(jkb,ibnd)
+                              endif
                            enddo
                         enddo
                      enddo
@@ -743,16 +822,20 @@ subroutine h_epsi_her_set(pdir, e_field)
                enddo
             enddo
             
-            call ZGEMM ('N', 'N', npw1, nbnd , nkb, (1.d0, 0.d0) , vkb, &!vkb is relative to the last ik read
+            call ZGEMM ('N', 'N', npw1, nbnd*npol , nkb, (1.d0, 0.d0) , vkb, &!vkb is relative to the last ik read
                  npwx, ps, nkb, (1.d0, 0.d0) , evct, npwx)
             do m=1,nbnd
                do nb=1,nbnd
                   do ig=1,npw1
                      evcm(ig,m,pdir)=evcm(ig,m,pdir) + mat(nb,m)*evct(ig,nb)
+                     if(noncolin) then
+                        evcm(ig+npwx,m,pdir)=evcm(ig+npwx,m,pdir) + mat(nb,m)*evct(ig+npwx,nb)
+                     endif
                   enddo
                enddo
             enddo
          endif
+
       ENDIF
 
 ! calculate  S-1(k,k+1)
@@ -760,7 +843,6 @@ subroutine h_epsi_her_set(pdir, e_field)
     
 !       
       if(ik_stringa /= nppstr_3d(pdir)) then
-         
          CALL gk_sort(xk(1,nx_el(ik+1,pdir)),ngm,g,ecutwfc/tpiba2, &
            &    npw0,igk0,g2kin_bp) 
          CALL get_buffer (evct,nwordwfc,iunwfc,nx_el(ik+1,pdir))
@@ -800,35 +882,42 @@ subroutine h_epsi_her_set(pdir, e_field)
 
         IF(mdone(nx_el(ik+1,pdir))==0) THEN
          mat=(0.d0,0.d0)
-         DO nb=1,nbnd
-            DO mb=1,nbnd
-               IF ( .NOT. l_cal(nb) .OR. .NOT. l_cal(mb) ) THEN
-                  IF ( nb == mb )  mat(nb,mb)=1.d0
-               ELSE
-                  aux=(0.d0,0.d0)
-                  aux0=(0.d0,0.d0)
-                  IF(noncolin) aux_2=(0.d0,0.d0)
-                  IF(noncolin) aux0_2=(0.d0,0.d0)
-                  DO ig=1,npw1
-                     aux0(igk1(ig))=evcel(ig,nb)
-                     if (noncolin) aux0_2(igk1(ig))=evcel(ig+npwx,nb)
-                  END DO
-                  DO ig=1,npw0
-                     aux(igk0(ig))=evct(ig,mb)
-                     if (noncolin) aux_2(igk0(ig))=evct(ig+npwx,mb)
-                  END DO
-                  mat(nb,mb) = zdotc(ngm,aux0,1,aux,1)
-                  if (noncolin) mat(nb,mb)=mat(nb,mb)+zdotc(ngm,aux0_2,1,aux_2,1)
-!                    --- Calculate the augmented part: ij=KB projectors, ---
-!                    --- R=atom index: SUM_{ijR} q(ijR) <u_nk|beta_iR>   ---
-!                    --- <beta_jR|u_mk'> e^i(k-k')*R =                   ---
-!                    --- also <u_nk|beta_iR>=<psi_nk|beta_iR> = becp^*   ---
-            endif
-         END DO
-      END DO
+         allocate(aux0vec(ngm,nbnd),aux1vec(ngm,nbnd))
+         aux0vec=(0.d0,0.d0)
+         aux1vec=(0.d0,0.d0)
+         do nb=1,nbnd
+            DO ig=1,npw1
+               aux0vec(igk1(ig),nb)=evcel(ig,nb)
+            END DO
+         end do
+         do nb=1,nbnd
+            DO ig=1,npw0
+               aux1vec(igk0(ig),nb)=evct(ig,nb)
+            END DO            
+         enddo
+         call ZGEMM('C','N',nbnd,nbnd,ngm,(1.d0,0.d0),aux0vec,ngm,aux1vec,ngm,(0.d0,0.d0),mat,nbnd)
+         if(noncolin) then
+             aux0vec=(0.d0,0.d0)
+             aux1vec=(0.d0,0.d0)
+             do nb=1,nbnd
+               DO ig=1,npw1
+                  aux0vec(igk1(ig),nb)=evcel(ig+npwx,nb)
+               END DO
+            end do
+            do nb=1,nbnd
+               DO ig=1,npw0
+                  aux1vec(igk0(ig),nb)=evct(ig+npwx,nb)
+               END DO
+            enddo
+            call ZGEMM('C','N',nbnd,nbnd,ngm,(1.d0,0.d0),aux0vec,ngm,aux1vec,ngm,(1.d0,0.d0),mat,nbnd)
+         endif
+         deallocate(aux0vec,aux1vec)
+
+
       !
       call mp_sum( mat, intra_bgrp_comm )
       !
+
       DO nb=1,nbnd
          DO mb=1,nbnd
             IF ( l_cal(nb) .AND. l_cal(mb) ) THEN
@@ -841,8 +930,21 @@ subroutine h_epsi_her_set(pdir, e_field)
                      nhjkbm = nh(np)
                      jkb1 = jkb - nhjkb
                      DO j = 1,nhjkbm
-                        pref = pref+CONJG(becp_bp(jkb,nb))*becp0(jkb1+j,mb) &
-                             *q_dk(nhjkb,j,np)*struc(na)
+                        if(lspinorb) then
+                            pref = pref+CONJG(becp_bp%nc(jkb,1,nb))*becp0%nc(jkb1+j,1,mb) &
+                                 *q_dk_so(nhjkb,j,1,np)*struc(na)
+                            pref = pref+CONJG(becp_bp%nc(jkb,1,nb))*becp0%nc(jkb1+j,2,mb) &
+                                 *q_dk_so(nhjkb,j,2,np)*struc(na)
+                            pref = pref+CONJG(becp_bp%nc(jkb,2,nb))*becp0%nc(jkb1+j,1,mb) &
+                                 *q_dk_so(nhjkb,j,3,np)*struc(na)
+                            pref = pref+CONJG(becp_bp%nc(jkb,2,nb))*becp0%nc(jkb1+j,2,mb) &
+                                 *q_dk_so(nhjkb,j,4,np)*struc(na)
+
+
+                        else
+                           pref = pref+CONJG(becp_bp%k(jkb,nb))*becp0%k(jkb1+j,mb) &
+                                *q_dk(nhjkb,j,np)*struc(na)
+                        endif
                      ENDDO
                   ENDDO
                   mat(nb,mb) = mat(nb,mb) + pref
@@ -851,15 +953,16 @@ subroutine h_epsi_her_set(pdir, e_field)
          ENDDO
       ENDDO
 
+
 !              --- Calculate matrix inverse ---
       CALL zgefa(mat,nbnd,nbnd,ivpt,info)
       CALL errore('h_epsi_her_set','error in zgefa',abs(info))
       CALL zgedi(mat,nbnd,nbnd,ivpt,cdet,cdwork,1)
       matbig(nx_el(ik+1,pdir),:,:)=TRANSPOSE(CONJG(mat))
       mdone(nx_el(ik+1,pdir))=1
-     ELSE
+   ELSE
       mat=TRANSPOSE(CONJG(matbig(nx_el(ik+1,pdir),:,:)))
-     END IF
+   END IF
 !    mat=S^-1(k,k-1)
       do ig=1,npw0
          gtr(1)=g(1,igk0(ig))
@@ -898,8 +1001,20 @@ subroutine h_epsi_her_set(pdir, e_field)
                         jkb = ijkb0 + jh
                         do ih = 1, nh (nt)
                            ikb = ijkb0 + ih
-                           ps (ikb, ibnd) = ps (ikb, ibnd) + &
-                                q_dk(ih,jh,ityp(na))*struc(na)* becp0(jkb,ibnd)
+                           if(noncolin) then
+                              ps (ikb, (ibnd-1)*npol+1) = ps (ikb,(ibnd-1)*npol+1 ) + &
+                                   q_dk_so(ih,jh,1,ityp(na))*struc(na)* becp0%nc(jkb,1,ibnd)
+                              ps (ikb, (ibnd-1)*npol+1) = ps (ikb,(ibnd-1)*npol+1 ) + &
+                                   q_dk_so(ih,jh,2,ityp(na))*struc(na)* becp0%nc(jkb,2,ibnd)
+                              ps (ikb, (ibnd-1)*npol+2) = ps (ikb,(ibnd-1)*npol+2 ) + &
+                                   q_dk_so(ih,jh,3,ityp(na))*struc(na)* becp0%nc(jkb,1,ibnd)
+                              ps (ikb, (ibnd-1)*npol+2) = ps (ikb,(ibnd-1)*npol+2 ) + &
+                                   q_dk_so(ih,jh,4,ityp(na))*struc(na)* becp0%nc(jkb,2,ibnd)
+
+                           else
+                              ps (ikb, ibnd) = ps (ikb, ibnd) + &
+                                   q_dk(ih,jh,ityp(na))*struc(na)* becp0%k(jkb,ibnd)
+                           endif
                         enddo
                      enddo
                   enddo
@@ -908,20 +1023,30 @@ subroutine h_epsi_her_set(pdir, e_field)
             enddo
          enddo
 
-         call ZGEMM ('N', 'N', npw1, nbnd , nkb, (1.d0, 0.d0) , vkb, &!vkb is relative to the last ik read
+         call ZGEMM ('N', 'N', npw1, nbnd*npol , nkb, (1.d0, 0.d0) , vkb, &!vkb is relative to the last ik read
                  npwx, ps, nkb, (1.d0, 0.d0) , evct, npwx)
-         do m=1,nbnd
-            do nb=1,nbnd
-               do ig=1,npw1
-                  evcp(ig,m,pdir)=evcp(ig,m,pdir) + mat(nb,m)*evct(ig,nb)
-               enddo
-            enddo
-         enddo
+                 
+         evct(npw1+1:npwx,1:nbnd)=(0.d0,0.d0)
+         if(noncolin) evct(npwx+npw1+1:2*npwx,1:nbnd)=(0.d0,0.d0)
+         evcp(npw1+1:npwx,1:nbnd,pdir)=(0.d0,0.d0)
+         if(noncolin) evcp(npwx+npw1+1:2*npwx,1:nbnd,pdir)=(0.d0,0.d0)
+         if(.not.noncolin) then
+            call ZGEMM('N','N',npw1,nbnd,nbnd,(1.d0,0.d0),evct,npwx,mat,nbnd,&
+                 &(1.d0,0.d0),evcp(1,1,pdir),npwx)
+         else
+            call ZGEMM('N','N',npwx*npol,nbnd,nbnd,(1.d0,0.d0),evct,npwx*npol,mat,nbnd,&
+              &(1.d0,0.d0),evcp(1,1,pdir),npwx*npol)
+         end if
+
+
+
+
       endif
          
 !           --- End of dot products between wavefunctions and betas ---
+     
    else
-       
+     
       CALL gk_sort(xk(1,nx_el(ik-nppstr_3d(pdir)+1,pdir)),ngm,g,ecutwfc/tpiba2, &
            &    npw0,igk0,g2kin_bp) 
       CALL get_buffer (evct,nwordwfc,iunwfc,nx_el(ik-nppstr_3d(pdir)+1,pdir))
@@ -1019,8 +1144,8 @@ subroutine h_epsi_her_set(pdir, e_field)
                      aux_g(mapgm_global(ig_l2g(igk0(ig)),pdir))=evct(ig,mb)
                      IF (noncolin) aux_g_2(mapgm_global(ig_l2g(igk0(ig)),pdir))=evct(ig+npwx,mb)
                   enddo
-                  call mp_sum(aux_g(:))
-                  IF (noncolin) call mp_sum(aux_g_2(:))
+                  call mp_sum(aux_g(:),intra_bgrp_comm)
+                  IF (noncolin) call mp_sum(aux_g_2(:),intra_bgrp_comm)
                end if
 
       DO nb=1,nbnd
@@ -1076,8 +1201,19 @@ subroutine h_epsi_her_set(pdir, e_field)
                      nhjkbm = nh(np)
                      jkb1 = jkb - nhjkb
                      DO j = 1,nhjkbm
-                        pref = pref+CONJG(becp_bp(jkb,nb))*becp0(jkb1+j,mb) &
-                             *q_dk(nhjkb,j,np)*struc(na)
+                        if(lspinorb) then
+                           pref = pref+CONJG(becp_bp%nc(jkb,1,nb))*becp0%nc(jkb1+j,1,mb) &
+                                *q_dk_so(nhjkb,j,1,np)*struc(na)
+                           pref = pref+CONJG(becp_bp%nc(jkb,1,nb))*becp0%nc(jkb1+j,2,mb) &
+                                *q_dk_so(nhjkb,j,2,np)*struc(na)
+                           pref = pref+CONJG(becp_bp%nc(jkb,2,nb))*becp0%nc(jkb1+j,1,mb) &
+                                *q_dk_so(nhjkb,j,3,np)*struc(na)
+                           pref = pref+CONJG(becp_bp%nc(jkb,2,nb))*becp0%nc(jkb1+j,2,mb) &
+                                *q_dk_so(nhjkb,j,4,np)*struc(na)
+                        else
+                           pref = pref+CONJG(becp_bp%k(jkb,nb))*becp0%k(jkb1+j,mb) &
+                                *q_dk(nhjkb,j,np)*struc(na)
+                        endif
                      ENDDO
                   ENDDO
                   mat(nb,mb) = mat(nb,mb) + pref
@@ -1094,7 +1230,7 @@ subroutine h_epsi_her_set(pdir, e_field)
       mdone(nx_el(ik-nppstr_3d(pdir)+1,pdir))=1
      ELSE
       mat=TRANSPOSE(CONJG(matbig(nx_el(ik-nppstr_3d(pdir)+1,pdir),:,:)))
-     END IF
+   END IF
        
 !    mat=S^-1(k,k-1)
       if(.not.l_para) then
@@ -1137,8 +1273,8 @@ subroutine h_epsi_her_set(pdir, e_field)
                IF (noncolin) aux_g_2(mapgm_global(ig_l2g(igk0(ig)),pdir))=evct(ig+npwx,nb)
             enddo
 !put evct on global  array
-            call mp_sum(aux_g(:))
-            IF (noncolin) call mp_sum(aux_g_2(:))
+            call mp_sum(aux_g(:),intra_bgrp_comm)
+            IF (noncolin) call mp_sum(aux_g_2(:),intra_bgrp_comm)
             do m=1,nbnd
                do ig=1,npw1
                   evcp(ig,m,pdir)=evcp(ig,m,pdir)+mat(nb,m)*aux_g(ig_l2g(igk1(ig)))
@@ -1149,8 +1285,7 @@ subroutine h_epsi_her_set(pdir, e_field)
          enddo
           deallocate(aux_g)
           IF (noncolin) deallocate(aux_g_2)
-
-      endif
+       endif
       if(okvan) then
          evct(:,:) =  (0.d0, 0.d0)
          ps (:,:) = (0.d0, 0.d0)
@@ -1163,8 +1298,20 @@ subroutine h_epsi_her_set(pdir, e_field)
                         jkb = ijkb0 + jh
                         do ih = 1, nh (nt)
                            ikb = ijkb0 + ih
-                           ps (ikb, ibnd) = ps (ikb, ibnd) + &
-                                q_dk(ih,jh,ityp(na))*struc(na)* becp0(jkb,ibnd)
+                           if(lspinorb) then
+                              ps (ikb, (ibnd-1)*npol+1) = ps (ikb,(ibnd-1)*npol+1 ) + &
+                                   q_dk_so(ih,jh,1,ityp(na))*struc(na)* becp0%nc(jkb,1,ibnd)
+                              ps (ikb, (ibnd-1)*npol+1) = ps (ikb,(ibnd-1)*npol+1 ) + &
+                                   q_dk_so(ih,jh,2,ityp(na))*struc(na)* becp0%nc(jkb,2,ibnd)
+                              ps (ikb, (ibnd-1)*npol+2) = ps (ikb,(ibnd-1)*npol+2 ) + &
+                                   q_dk_so(ih,jh,3,ityp(na))*struc(na)* becp0%nc(jkb,1,ibnd)
+                              ps (ikb, (ibnd-1)*npol+2) = ps (ikb,(ibnd-1)*npol+2 ) + &
+                                   q_dk_so(ih,jh,4,ityp(na))*struc(na)* becp0%nc(jkb,2,ibnd)
+
+                           else
+                              ps (ikb, ibnd) = ps (ikb, ibnd) + &
+                                   q_dk(ih,jh,ityp(na))*struc(na)* becp0%k(jkb,ibnd)
+                           endif
                         enddo
                      enddo
                   enddo
@@ -1173,24 +1320,27 @@ subroutine h_epsi_her_set(pdir, e_field)
             enddo
          enddo
 
-         call ZGEMM ('N', 'N', npw1, nbnd , nkb, (1.d0, 0.d0) , vkb, &!vkb is relative to the ik read
+         call ZGEMM ('N', 'N', npw1, nbnd*npol , nkb, (1.d0, 0.d0) , vkb, &!vkb is relative to the ik read
                  npwx, ps, nkb, (1.d0, 0.d0) , evct, npwx)
          do m=1,nbnd
             do nb=1,nbnd
                do ig=1,npw1
                   evcp(ig,m,pdir)=evcp(ig,m,pdir) + mat(nb,m)*evct(ig,nb)
+                  if(noncolin) then
+                     evcp(ig+npwx,m,pdir)=evcp(ig+npwx,m,pdir) + mat(nb,m)*evct(ig+npwx,nb)
+                  endif
                enddo
             enddo
          enddo
       endif
 
    ENDIF
-
+   
 
 
 !writes projectors to disk 
-   call davcio(evcm(:,:,pdir), 2*nwordwfc,iunefieldm,nx_el(ik,pdir)+(pdir-1)*nks,1)
-   call davcio(evcp(:,:,pdir), 2*nwordwfc,iunefieldp,nx_el(ik,pdir)+(pdir-1)*nks,1)
+   call save_buffer (evcm(:,:,pdir), nwordwfc, iunefieldm, nx_el(ik,pdir)+(pdir-1)*nks)
+   call save_buffer (evcp(:,:,pdir), nwordwfc, iunefieldp, nx_el(ik,pdir)+(pdir-1)*nks)
 
   END  DO !on ik
 
@@ -1202,8 +1352,13 @@ subroutine h_epsi_her_set(pdir, e_field)
   DEALLOCATE(aux,aux0)
   IF (ALLOCATED(aux_2)) DEALLOCATE(aux_2)
   IF (ALLOCATED(aux0_2)) DEALLOCATE(aux0_2)
+  if(okvan)CALL deallocate_bec_type (becp0)
+  if(okvan)CALL deallocate_bec_type (becp_bp)
+  if(okvan.and.lspinorb) deallocate(q_dk_so)
+  if(okvan.and.lspinorb) deallocate(q_dkp_so)
+  deallocate(mat)
 
-  
+  call stop_clock('h_epsi_set')
 !  --
 !------------------------------------------------------------------------------!
    return
