@@ -23,10 +23,10 @@ SUBROUTINE hinit0()
   USE vlocal,       ONLY : strf
   USE wvfct,        ONLY : npw, g2kin, igk, ecutwfc
   USE io_files,     ONLY : iunigk
-  USE realus,       ONLY : qpointlist,betapointlist,init_realspace_vars,real_space
+  USE realus,       ONLY : generate_qpointlist,betapointlist,init_realspace_vars,real_space
   use ldaU,         ONLY : lda_plus_U, U_projection
-  USE control_flags, ONLY : tqr 
-  USE io_global,  ONLY : stdout
+  USE control_flags,ONLY : tqr 
+  USE io_global,    ONLY : stdout
   !
   IMPLICIT NONE
   !
@@ -93,9 +93,8 @@ SUBROUTINE hinit0()
   !
   CALL set_rhoc()
   !
-  IF ( tqr ) CALL qpointlist() 
+  IF ( tqr ) CALL generate_qpointlist()
   IF (real_space ) then
-   !call qpointlist()
    call betapointlist()
    call init_realspace_vars()
    write(stdout,'(5X,"Real space initialisation completed")')    
