@@ -35,16 +35,17 @@ SUBROUTINE move_ions()
   USE ener,                   ONLY : etot
   USE force_mod,              ONLY : force, sigma
   USE control_flags,          ONLY : istep, nstep, upscale, lbfgs, ldamped, &
-                                     lconstrain, conv_ions, &
+                                     lconstrain, conv_ions, use_SMC, &
                                      lmd, llang, history, tr2
   USE relax,                  ONLY : epse, epsf, epsp, starting_scf_threshold
   USE lsda_mod,               ONLY : lsda, absmag
-  USE mp_global,              ONLY : intra_image_comm
+  USE mp_images,              ONLY : intra_image_comm
   USE io_global,              ONLY : ionode_id, ionode
   USE mp,                     ONLY : mp_bcast
   USE bfgs_module,            ONLY : bfgs, terminate_bfgs
   USE basic_algebra_routines, ONLY : norm
   USE dynamics_module,        ONLY : verlet, langevin_md, proj_verlet
+  USE dynamics_module,        ONLY : smart_MC
   USE dfunct,                 only : newd
   !
   IMPLICIT NONE
@@ -61,6 +62,8 @@ SUBROUTINE move_ions()
   INTEGER,  ALLOCATABLE :: fixion(:)
   real(dp) :: tr
   !
+  IF (use_SMC) CALL smart_MC()  ! for smart monte carlo method
+  !
   ! ... only one node does the calculation in the parallel case
   !
   IF ( ionode ) THEN
