@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2007 Quantum ESPRESSO group
+! Copyright (C) 2001-2013 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -19,7 +19,7 @@ MODULE scf
   USE lsda_mod,     ONLY : nspin
   USE ldaU,         ONLY : lda_plus_u, Hubbard_lmax
   USE ions_base,    ONLY : nat
-  USE io_files,     ONLY : diropn
+  USE buffers,      ONLY : open_buffer, close_buffer, get_buffer, save_buffer
   USE funct,        ONLY : dft_is_meta
   USE fft_base,     ONLY : dfftp
   USE fft_interfaces,ONLY: invfft
@@ -28,6 +28,7 @@ MODULE scf
   USE paw_variables,ONLY : okpaw
   USE uspp_param,   ONLY : nhm
   USE extfield,     ONLY : dipfield, emaxpos, eopreg, edir
+  USE control_flags,ONLY : lxdm
   !
   SAVE
   !
@@ -100,7 +101,9 @@ MODULE scf
                        rlen_dip=0, &
                        start_rho=0, start_kin=0, start_ldaU=0, start_bec=0, &
                        start_dipole=0
-  REAL(DP), PRIVATE, ALLOCATABLE:: io_buffer(:)
+  ! DFT+U, colinear and noncolinear cases
+  LOGICAL, PRIVATE :: lda_plus_u_co, lda_plus_u_nc
+  COMPLEX(DP), PRIVATE, ALLOCATABLE:: io_buffer(:)
 CONTAINS
 
  SUBROUTINE create_scf_type ( rho, do_not_allocate_becsum )
@@ -113,7 +116,7 @@ CONTAINS
 #ifdef __STD_F95
  nullify (rho%kin_r, rho%kin_g, rho%ns, rho%ns_nc, rho%bec)
 #endif
-   if (dft_is_meta()) then
+   if (dft_is_meta() .or. lxdm) then
       allocate ( rho%kin_r( dfftp%nnr, nspin) )
       allocate ( rho%kin_g( ngm, nspin ) )
    else
@@ -121,10 +124,10 @@ CONTAINS
       allocate ( rho%kin_g(1,1) )
    endif
 
-   if (lda_plus_u) then
-     allocate (rho%ns(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat))
-     allocate (rho%ns_nc(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat))
-   endif 
+   lda_plus_u_co = lda_plus_u .and. .not. (nspin == 4 )
+   lda_plus_u_nc = lda_plus_u .and.       (nspin == 4 )
+   if (lda_plus_u_co) allocate (rho%ns(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat))
+   if (lda_plus_u_nc) allocate (rho%ns_nc(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat))
 
    if (okpaw) then ! See the top of the file for clarification
       if(present(do_not_allocate_becsum)) then
@@ -169,21 +172,26 @@ CONTAINS
 #ifdef __STD_F95
    nullify (rho%kin_g, rho%ns, rho%ns_nc, rho%bec)
 #endif
-   if (dft_is_meta()) allocate (rho%kin_g( ngms, nspin ) )
-   if (lda_plus_u) then
-    allocate (rho%ns(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat))
-    allocate (rho%ns_nc(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat))
-   endif
-   if (okpaw)         allocate (rho%bec(nhm*(nhm+1)/2,nat,nspin))
-
    rho%of_g = 0._dp
-   if (dft_is_meta()) rho%kin_g = 0._dp
-   if (lda_plus_u) then
-     rho%ns       = 0._dp
-     rho%ns_nc    = 0._dp
+   if (dft_is_meta() .or. lxdm) then
+      allocate (rho%kin_g( ngms, nspin ) )
+      rho%kin_g = 0._dp
+   end if
+   lda_plus_u_co = lda_plus_u .and. .not. (nspin == 4 )
+   lda_plus_u_nc = lda_plus_u .and.       (nspin == 4 )
+   if (lda_plus_u_nc) then
+      allocate (rho%ns_nc(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat))
+      rho%ns_nc    = 0._dp
    endif
-   if (okpaw)         rho%bec   = 0._dp
-   if (dipfield)      rho%el_dipole =  0._dp
+   if (lda_plus_u_co) then
+      allocate (rho%ns(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat))
+      rho%ns       = 0._dp
+   endif
+   if (okpaw) then
+      allocate (rho%bec(nhm*(nhm+1)/2,nat,nspin))
+      rho%bec   = 0._dp
+   end if
+   rho%el_dipole =  0._dp
    
  return
  END SUBROUTINE create_mix_type
@@ -215,11 +223,9 @@ CONTAINS
       
    rho_m%of_g(1:ngms,:) = rho_s%of_g(1:ngms,:)
    
-   if (dft_is_meta()) rho_m%kin_g(1:ngms,:) = rho_s%kin_g(1:ngms,:)
-   if (lda_plus_u) then
-     rho_m%ns     = rho_s%ns
-     rho_m%ns_nc  = rho_s%ns_nc
-   endif
+   if (dft_is_meta() .or. lxdm) rho_m%kin_g(1:ngms,:) = rho_s%kin_g(1:ngms,:)
+   if (lda_plus_u_nc) rho_m%ns_nc  = rho_s%ns_nc
+   if (lda_plus_u_co) rho_m%ns     = rho_s%ns
    if (okpaw)         rho_m%bec = rho_s%bec
    
    if (dipfield) then
@@ -250,7 +256,7 @@ CONTAINS
       rho_s%of_r(:,is) = psic(:)
    END DO
 
-   if (dft_is_meta()) then
+   if (dft_is_meta() .or. lxdm) then
       rho_s%kin_g(1:ngms,:) = rho_m%kin_g(:,:)
       ! define rho_s%kin_r 
       DO is = 1, nspin
@@ -262,10 +268,8 @@ CONTAINS
       END DO
    end if
 
-   if (lda_plus_u) then
-     rho_s%ns(:,:,:,:)    = rho_m%ns(:,:,:,:)
-     rho_s%ns_nc(:,:,:,:) = rho_m%ns_nc(:,:,:,:)
-   endif 
+   if (lda_plus_u_nc) rho_s%ns_nc(:,:,:,:) = rho_m%ns_nc(:,:,:,:)
+   if (lda_plus_u_co) rho_s%ns(:,:,:,:)    = rho_m%ns(:,:,:,:)
    if (okpaw)      rho_s%bec(:,:,:)  = rho_m%bec(:,:,:)
        
    return
@@ -281,14 +285,12 @@ CONTAINS
   TYPE(scf_type), INTENT(INOUT) :: Y
   Y%of_r  = X%of_r
   Y%of_g  = X%of_g
-  if (dft_is_meta()) then
+  if (dft_is_meta() .or. lxdm) then
      Y%kin_r = X%kin_r
      Y%kin_g = X%kin_g
   end if
-  if (lda_plus_u) then
-    Y%ns    = X%ns
-    Y%ns_nc = X%ns_nc
-  endif
+  if (lda_plus_u_nc) Y%ns_nc = X%ns_nc
+  if (lda_plus_u_co) Y%ns    = X%ns
   if (okpaw)      Y%bec = X%bec
   !
   RETURN
@@ -305,11 +307,9 @@ CONTAINS
   TYPE(mix_type), INTENT(IN)    :: X
   TYPE(mix_type), INTENT(INOUT) :: Y
   Y%of_g  = Y%of_g  + A * X%of_g
-  if (dft_is_meta()) Y%kin_g = Y%kin_g + A * X%kin_g
-  if (lda_plus_u) then
-    Y%ns    = Y%ns    + A * X%ns
-    Y%ns_nc = Y%ns_nc + A * X%ns_nc
-  endif
+  if (dft_is_meta() .or. lxdm) Y%kin_g = Y%kin_g + A * X%kin_g
+  if (lda_plus_u_nc) Y%ns_nc = Y%ns_nc + A * X%ns_nc
+  if (lda_plus_u_co) Y%ns = Y%ns + A * X%ns
   if (okpaw)     Y%bec = Y%bec + A * X%bec
   if (dipfield)  Y%el_dipole =  Y%el_dipole + A * X%el_dipole
   !
@@ -325,11 +325,9 @@ CONTAINS
   TYPE(mix_type), INTENT(IN)    :: X
   TYPE(mix_type), INTENT(INOUT) :: Y
   Y%of_g  = X%of_g
-  if (dft_is_meta()) Y%kin_g = X%kin_g
-  if (lda_plus_u) then
-    Y%ns     = X%ns
-    Y%ns_nc  = X%ns_nc
-  endif
+  if (dft_is_meta() .or. lxdm) Y%kin_g = X%kin_g
+  if (lda_plus_u_nc) Y%ns_nc  = X%ns_nc
+  if (lda_plus_u_co) Y%ns  = X%ns
   if (okpaw)      Y%bec = X%bec
   if (dipfield)   Y%el_dipole =  X%el_dipole
   !
@@ -346,11 +344,9 @@ CONTAINS
   REAL(DP),       INTENT(IN)    :: A
   TYPE(mix_type), INTENT(INOUT) :: X
   X%of_g(:,:)  = A * X%of_g(:,:)
-  if (dft_is_meta()) X%kin_g = A * X%kin_g
-  if (lda_plus_u) then
-    X%ns    = A * X%ns
-    X%ns_nc = A * X%ns_nc
-  endif
+  if (dft_is_meta() .or. lxdm) X%kin_g = A * X%kin_g
+  if (lda_plus_u_nc) X%ns_nc = A * X%ns_nc
+  if (lda_plus_u_co) X%ns    = A * X%ns
   if (okpaw)      X%bec= A * X%bec
   if (dipfield)   X%el_dipole =  A * X%el_dipole
   !
@@ -378,7 +374,7 @@ CONTAINS
          rhoin%of_r(:,is) = psic(:)
       END DO
       !
-      if (dft_is_meta()) then
+      if (dft_is_meta() .or. lxdm) then
          rhoin%kin_g = rhoin%kin_g + alphamix * ( input_rhout%kin_g-rhoin%kin_g)
          rhoin%kin_g(1:ngms,1:nspin) = (0.d0,0.d0)
          ! define rho_s%of_r 
@@ -393,50 +389,57 @@ CONTAINS
    else
       rhoin%of_g(:,:)= (0.d0,0.d0)
       rhoin%of_r(:,:)= 0.d0
-      if (dft_is_meta()) then
+      if (dft_is_meta() .or. lxdm) then
          rhoin%kin_g(:,:)= (0.d0,0.d0)
          rhoin%kin_r(:,:)= 0.d0
       endif
    endif
-   if (lda_plus_u) then
-     rhoin%ns(:,:,:,:)    = 0.d0
-     rhoin%ns_nc(:,:,:,:) = 0.d0
-   endif
+   if (lda_plus_u_nc) rhoin%ns_nc(:,:,:,:) = 0.d0
+   if (lda_plus_u_co) rhoin%ns(:,:,:,:)    = 0.d0
+
    return
  end subroutine high_frequency_mixing 
 
 
- subroutine diropn_mix_file( iunit, extension, exst )
+ subroutine open_mix_file( iunit, extension, exst )
+   USE control_flags,        ONLY : io_level
    implicit none
    character(len=*), intent(in) :: extension
    integer, intent(in) :: iunit
    logical :: exst
-   ! define lengths of different record chunks
+   ! define lengths (in real numbers) of different record chunks
    rlen_rho = 2 * ngms * nspin
-   if (dft_is_meta() ) rlen_kin =  2 * ngms * nspin
-   if (lda_plus_u)     rlen_ldaU = (2*Hubbard_lmax+1)**2 *nspin*nat
-   if (okpaw)          rlen_bec =  (nhm*(nhm+1)/2) * nat * nspin
+   if (dft_is_meta() .or. lxdm) rlen_kin =  2 * ngms * nspin
+   if (lda_plus_u_co)  rlen_ldaU = (2*Hubbard_lmax+1)**2 *nspin*nat
+   if (lda_plus_u_nc)  rlen_ldaU = 2 * (2*Hubbard_lmax+1)**2 *nspin*nat
+   if (okpaw)          rlen_bec = (nhm*(nhm+1)/2) * nat * nspin
    if (dipfield)       rlen_dip = 1
-   ! define total record length
-   record_length = rlen_rho + rlen_kin + rlen_ldaU + rlen_bec + rlen_dip
-   ! and the starting point of different chunks
+   ! define the starting point of the different chunks. Beware: each starting point
+   ! is the index of a COMPLEX array. When real arrays with odd dimension are copied
+   ! to/from the complex array io_buffer, the last complex number will be half-filled
+   ! but must still be counted as one!
    start_rho = 1
-   start_kin = start_rho + rlen_rho
-   start_ldaU = start_kin + rlen_kin
-   start_bec = start_ldaU + rlen_ldaU
-   start_dipole = start_bec + rlen_bec
+   start_kin = start_rho + rlen_rho / 2
+   start_ldaU = start_kin + rlen_kin / 2
+   start_bec = start_ldaU + ( rlen_ldaU + 1 ) / 2 
+   start_dipole = start_bec + ( rlen_bec + 1 ) / 2
+   ! define total record length, in complex numbers
+   record_length = start_dipole + rlen_dip - 1
    ! open file and allocate io_buffer
-   call diropn ( iunit, extension, record_length, exst)
-   allocate (io_buffer(record_length+1))
+   call open_buffer ( iunit, extension, record_length, io_level, exst)
+   allocate (io_buffer(record_length))
+   ! setting to zero -prevents trouble with "holes" due to odd dimensions of real arrays
+   io_buffer (:) = (0.0_dp, 0.0_dp)
    !
  return
- end subroutine diropn_mix_file
+ end subroutine open_mix_file
  !
- subroutine close_mix_file( iunit )
+ subroutine close_mix_file( iunit, stat )
    implicit none
    integer, intent(in) :: iunit
+   character(len=*), intent(in) :: stat
    deallocate (io_buffer)
-   close(iunit,status='keep')
+   call close_buffer ( iunit, trim(stat) ) 
    return
  end subroutine close_mix_file
 
@@ -444,26 +447,23 @@ CONTAINS
    implicit none
    type (mix_type) :: rho
    integer, intent(in) :: iunit, record, iflag
-   if (iflag > 0) then
-      
+
+   if (iflag > 0) then      
       call DCOPY(rlen_rho,rho%of_g,1,io_buffer(start_rho),1)
-      if (dft_is_meta()) call DCOPY(rlen_kin, rho%kin_g,1,io_buffer(start_kin),1)
-      if (lda_plus_u)    call DCOPY(rlen_ldaU,rho%ns,   1,io_buffer(start_ldaU),1)
-      if (lda_plus_u)    call DCOPY(rlen_ldaU,rho%ns_nc, 1,io_buffer(start_ldaU),1)
-      if (okpaw)         call DCOPY(rlen_bec, rho%bec,  1,io_buffer(start_bec),1)
-      if (dipfield)      call DCOPY(1, rho%el_dipole,  1,io_buffer(start_dipole),1)
-      
-   end if
-   CALL davcio( io_buffer, record_length, iunit, record, iflag )
-   if (iflag < 0) then
-
+      if (dft_is_meta() .or. lxdm)  call DCOPY(rlen_kin, rho%kin_g,1,io_buffer(start_kin),1)
+      if (lda_plus_u_nc)  call DCOPY(rlen_ldaU,rho%ns_nc, 1,io_buffer(start_ldaU),1)
+      if (lda_plus_u_co)  call DCOPY(rlen_ldaU,rho%ns,   1,io_buffer(start_ldaU),1)
+      if (okpaw)          call DCOPY(rlen_bec, rho%bec,  1,io_buffer(start_bec),1)
+      if (dipfield)       io_buffer(start_dipole) = CMPLX ( rho%el_dipole, 0.0_dp )
+      CALL save_buffer( io_buffer, record_length, iunit, record )   
+   else if (iflag < 0 ) then 
+      CALL get_buffer( io_buffer, record_length, iunit, record )
       call DCOPY(rlen_rho,io_buffer(start_rho),1,rho%of_g,1)
-      if (dft_is_meta()) call DCOPY(start_kin,io_buffer(start_kin), 1,rho%kin_g,1)
-      if (lda_plus_u)    call DCOPY(rlen_ldaU,io_buffer(start_ldaU),1,rho%ns,1)
-      if (lda_plus_u)    call DCOPY(rlen_ldaU,io_buffer(start_ldaU),1,rho%ns_nc,1)
+      if (dft_is_meta() .or. lxdm) call DCOPY(rlen_kin,io_buffer(start_kin), 1,rho%kin_g,1)
+      if (lda_plus_u_co) call DCOPY(rlen_ldaU,io_buffer(start_ldaU),1,rho%ns,1)
+      if (lda_plus_u_nc) call DCOPY(rlen_ldaU,io_buffer(start_ldaU),1,rho%ns_nc,1)
       if (okpaw)         call DCOPY(rlen_bec, io_buffer(start_bec), 1,rho%bec,1)
-      if (dipfield)      call DCOPY(1, io_buffer(start_dipole), 1, rho%el_dipole, 1)
-
+      if (dipfield)      rho%el_dipole = REAL ( io_buffer(start_dipole) )
    end if
  end subroutine davcio_mix_type
  !
@@ -481,7 +481,7 @@ CONTAINS
   USE spin_orb,      ONLY : domag
   USE control_flags, ONLY : gamma_only
   USE paw_onecenter, ONLY : paw_ddot
-  USE mp_global,     ONLY : intra_bgrp_comm
+  USE mp_bands,      ONLY : intra_bgrp_comm
   USE mp,            ONLY : mp_sum
   !
   IMPLICIT NONE
@@ -599,8 +599,8 @@ CONTAINS
   ! 
   ! Beware: paw_ddot has a hidden parallelization on all processors
   !         it must be called on all processors or else it will hang
-  !
-  IF (okpaw)         rho_ddot = rho_ddot + paw_ddot(rho1%bec, rho2%bec)
+  ! Beware: commented out because it yields too often negative values
+  ! IF (okpaw)         rho_ddot = rho_ddot + paw_ddot(rho1%bec, rho2%bec)
   IF (dipfield)      rho_ddot = rho_ddot + (e2/2.0_DP)* &
                                     (rho1%el_dipole * rho2%el_dipole)*omega/fpi
 
@@ -620,7 +620,7 @@ FUNCTION tauk_ddot( rho1, rho2, gf )
   USE cell_base,     ONLY : omega, tpiba2
   USE gvect,         ONLY : gg, gstart
   USE control_flags, ONLY : gamma_only
-  USE mp_global,     ONLY : intra_bgrp_comm
+  USE mp_bands,      ONLY : intra_bgrp_comm
   USE mp,            ONLY : mp_sum
   !
   IMPLICIT NONE
@@ -767,7 +767,7 @@ END FUNCTION ns_ddot
   USE cell_base,     ONLY : omega, tpiba2
   USE gvect,         ONLY : gg, gstart
   USE control_flags, ONLY : gamma_only
-  USE mp_global,     ONLY : intra_bgrp_comm
+  USE mp_bands,      ONLY : intra_bgrp_comm
   USE mp,            ONLY : mp_sum
   !
   IMPLICIT NONE
@@ -811,16 +811,13 @@ END FUNCTION ns_ddot
   !
   CALL mp_bcast ( rho%of_g, root, comm )
   CALL mp_bcast ( rho%of_r, root, comm )
-  IF ( dft_is_meta() ) THEN
+  IF ( dft_is_meta() .or. lxdm) THEN
      CALL mp_bcast ( rho%kin_g, root, comm )
      CALL mp_bcast ( rho%kin_r, root, comm )
   END IF
-  IF ( lda_plus_u ) THEN
-     CALL mp_bcast ( rho%ns,    root, comm )
-     CALL mp_bcast ( rho%ns_nc, root, comm )
-  END IF
-  IF ( okpaw ) &
-     CALL mp_bcast ( rho%bec,   root, comm )
+  IF ( lda_plus_u_co) CALL mp_bcast ( rho%ns,    root, comm )
+  IF ( lda_plus_u_nc) CALL mp_bcast ( rho%ns_nc, root, comm )
+  IF ( okpaw )        CALL mp_bcast ( rho%bec,   root, comm )
   !
   END SUBROUTINE
  !
