@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2002-2009 Quantum ESPRESSO group
+! Copyright (C) 2002-2013 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -17,20 +17,19 @@ SUBROUTINE stres_hub ( sigmah )
    USE kinds,     ONLY : DP
    USE ions_base, ONLY : nat, ityp
    USE cell_base, ONLY : omega, at, bg
-   USE ldaU,      ONLY : hubbard_lmax, hubbard_l, hubbard_u, &
-                         hubbard_alpha, lda_plus_u_kind, U_projection
+   USE ldaU,      ONLY : hubbard_lmax, hubbard_l, is_hubbard, &
+                         lda_plus_u_kind, U_projection
    USE scf,       ONLY : v
    USE lsda_mod,  ONLY : nspin
    USE symme,     ONLY : symmatrix
-   USE io_files,  ONLY : prefix, iunocc
+   USE io_files,  ONLY : prefix
    USE io_global, ONLY : stdout, ionode
    !
    IMPLICIT NONE
    !
-   REAL (DP) :: sigmah(3,3)        ! output: the Hubbard stresses
-
-   INTEGER :: ipol, jpol, na, nt, is, m1,m2
-   INTEGER :: ldim
+   REAL (DP), INTENT(OUT) :: sigmah(3,3) ! the Hubbard contribution to stresses
+   !
+   INTEGER :: ipol, jpol, na, nt, is, m1,m2, ldim
    REAL (DP), ALLOCATABLE :: dns(:,:,:,:)
    !       dns(ldim,ldim,nspin,nat), ! the derivative of the atomic occupations
    !
@@ -45,13 +44,12 @@ SUBROUTINE stres_hub ( sigmah )
 
    ldim = 2 * Hubbard_lmax + 1
    ALLOCATE (dns(ldim,ldim,nspin,nat))
-   dns(:,:,:,:) = 0.d0
-
+   !
 #ifdef DEBUG
    DO na=1,nat
       DO is=1,nspin
          nt = ityp(na)
-         IF (Hubbard_U(nt).NE.0.d0.OR.Hubbard_alpha(nt).NE.0.d0) THEN
+         IF ( is_hubbard(nt) ) THEN
             WRITE( stdout,'(a,2i3)') 'NS(NA,IS) ', na,is
             DO m1=1,ldim
                WRITE( stdout,'(7f10.4)') (v%ns(m1,m2,is,na),m2=1,ldim)
@@ -69,10 +67,10 @@ SUBROUTINE stres_hub ( sigmah )
 !
    DO ipol = 1,3
       DO jpol = 1,3
-         CALL dndepsilon(dns,ldim,ipol,jpol)
+         CALL dndepsilon(ipol,jpol,ldim,dns)
          DO na = 1,nat                 
             nt = ityp(na)
-            IF ( Hubbard_U(nt) /= 0.d0 .OR. Hubbard_alpha(nt) /= 0.d0 ) THEN
+            IF ( is_hubbard(nt) ) THEN
                DO is = 1,nspin
 #ifdef DEBUG
                   WRITE( stdout,'(a,4i3)') 'DNS(IPOL,JPOL,NA,IS) ', ipol,jpol,na,is
@@ -116,7 +114,7 @@ SUBROUTINE stres_hub ( sigmah )
 END  SUBROUTINE stres_hub
 !
 !-----------------------------------------------------------------------
-SUBROUTINE dndepsilon ( dns,ldim,ipol,jpol )
+SUBROUTINE dndepsilon ( ipol,jpol,ldim,dns )
    !-----------------------------------------------------------------------
    ! This routine computes the derivative of the ns atomic occupations with
    ! respect to the strain epsilon(ipol,jpol) used to obtain the hubbard
@@ -125,50 +123,58 @@ SUBROUTINE dndepsilon ( dns,ldim,ipol,jpol )
    USE kinds,                ONLY : DP
    USE wavefunctions_module, ONLY : evc
    USE ions_base,            ONLY : nat, ityp
-   USE basis,                ONLY : natomwfc
    USE control_flags,        ONLY : gamma_only   
    USE klist,                ONLY : nks, xk, ngk
-   USE ldaU,                 ONLY : swfcatom, Hubbard_l, &
-                                    Hubbard_U, Hubbard_alpha, oatwfc
+   USE ldaU,                 ONLY : wfcU, nwfcU, offsetU, Hubbard_l, &
+                                    is_hubbard, copy_U_wfc
+   USE basis,                ONLY : natomwfc
    USE lsda_mod,             ONLY : lsda, nspin, current_spin, isk
    USE wvfct,                ONLY : nbnd, npwx, npw, igk, wg
    USE uspp,                 ONLY : nkb, vkb
    USE becmod,               ONLY : bec_type, becp, calbec, &
                                     allocate_bec_type, deallocate_bec_type
    USE io_files,             ONLY : iunigk, nwordwfc, iunwfc, &
-                                    iunat, iunsat, nwordatwfc
+                                    iunhub, nwordwfcU, nwordatwfc
    USE buffers,              ONLY : get_buffer
-   USE mp_global,            ONLY : inter_pool_comm
+   USE mp_pools,             ONLY : inter_pool_comm, intra_pool_comm, &
+                                    me_pool, nproc_pool
    USE mp,                   ONLY : mp_sum
 
    IMPLICIT NONE
    !
    ! I/O variables first
    !
-   INTEGER :: ipol, jpol, ldim
-   REAL (DP) :: dns(ldim,ldim,nspin,nat)
+   INTEGER, INTENT(IN) :: ipol, jpol, ldim
+   REAL(DP), INTENT(OUT) :: dns(ldim,ldim,nspin,nat)
    !
    ! local variable
    !
    INTEGER :: ik,    & ! counter on k points
               ibnd,  & !    "    "  bands
               is,    & !    "    "  spins
-              na, nt, m1, m2
-
-   COMPLEX (DP), ALLOCATABLE :: spsi(:,:)
+              na, nt, m1, m2, nb_s, nb_e, mykey
+   REAL(DP), ALLOCATABLE :: dns_(:,:,:,:) ! partial contribution 
+   COMPLEX (DP), ALLOCATABLE :: spsi(:,:), wfcatom(:,:)
    type (bec_type) :: proj, dproj
-!   COMPLEX (DP), ALLOCATABLE :: dproj(:,:)
-!   REAL (DP), ALLOCATABLE :: drproj(:,:)
    !
+   ! poor-man parallelization over bands
+   ! - if nproc_pool=1   : nb_s=1, nb_e=nbnd, mykey=0
+   ! - if nproc_pool<=nbnd:each processor calculates band nb_s to nb_e; mykey=0
+   ! - if nproc_pool>nbnd :each processor takes care of band na_s=nb_e;
+   !   mykey labels how many times each band appears (mykey=0 first time etc.)
    !
+   CALL block_distribute( nbnd, me_pool, nproc_pool, nb_s, nb_e, mykey )
+   !
+   ALLOCATE ( wfcatom (npwx,natomwfc) )
    ALLOCATE ( spsi(npwx,nbnd) )
-   call allocate_bec_type( natomwfc,nbnd, proj)
-   call allocate_bec_type ( natomwfc,nbnd, dproj )
+   call allocate_bec_type ( nwfcU,nbnd, proj)
+   call allocate_bec_type ( nwfcU,nbnd, dproj )
    call allocate_bec_type ( nkb,nbnd, becp )
+   ALLOCATE ( dns_(ldim,ldim,nspin,nat) )
    !
    ! D_Sl for l=1 and l=2 are already initialized, for l=0 D_S0 is 1
    !
-   ! Offset of atomic wavefunctions initialized in setup and stored in oatwfc
+   ! Offset of atomic wavefunctions initialized in setup and stored in offsetU
   
    dns(:,:,:,:) = 0.d0
    !
@@ -178,59 +184,75 @@ SUBROUTINE dndepsilon ( dns,ldim,ipol,jpol )
 
    DO ik = 1, nks
       IF (lsda) current_spin = isk(ik)
-      IF (nks > 1) READ (iunigk) igk
       npw = ngk(ik)
       !
+      IF (nks > 1) THEN
+         READ (iunigk) igk
+         CALL get_buffer (evc, nwordwfc, iunwfc, ik)
+      END IF
+      CALL init_us_2 (npw,igk,xk(1,ik),vkb)
+      CALL calbec( npw, vkb, evc, becp )
+      CALL s_psi  (npwx, npw, nbnd, evc, spsi )
+      
+      ! re-calculate atomic wfc - wfcatom is used here as work space
+
+      CALL atomic_wfc (ik, wfcatom)
+      call copy_U_wfc (wfcatom)
+
+      ! wfcU contains Hubbard-U atomic wavefunctions
+      ! proj=<wfcU|S|evc> - no need to read S*wfcU from buffer
+      !
+      CALL calbec ( npw, wfcU, spsi, proj)
+      !
       ! now we need the first derivative of proj with respect to
       ! epsilon(ipol,jpol)
       !
-      CALL get_buffer (evc, nwordwfc, iunwfc, ik)
-      CALL init_us_2 (npw,igk,xk(1,ik),vkb)
-      CALL calbec( npw, vkb, evc, becp )
-      CALL s_psi  (npwx, npw, nbnd, evc, spsi )
-! read atomic wfc - swfcatom is used as work space
-      CALL davcio(swfcatom,nwordatwfc,iunat,ik,-1)
       IF ( gamma_only ) THEN
-         CALL dprojdepsilon_gamma (swfcatom, spsi, ipol, jpol, dproj%r)
+         CALL dprojdepsilon_gamma (spsi, ipol, jpol, nb_s, nb_e, mykey, dproj%r)
       ELSE
-         CALL dprojdepsilon_k (swfcatom, spsi, ik, ipol, jpol, dproj%k)
+         CALL dprojdepsilon_k (spsi, ik, ipol, jpol, nb_s, nb_e, mykey, dproj%k)
       END IF
-      CALL davcio(swfcatom,nwordatwfc,iunsat,ik,-1)
-      CALL calbec ( npw, swfcatom, evc, proj)
       !
       ! compute the derivative of the occupation numbers (quantities dn(m1,m2))
       ! of the atomic orbitals. They are real quantities as well as n(m1,m2)
       !
+      ! band parallelization. If each band appears more than once
+      ! compute its contribution only once (i.e. when mykey=0)
+      dns_(:,:,:,:) = 0.0_dp
+      IF ( mykey /= 0 ) GO TO 10
       DO na = 1,nat
          nt = ityp(na)
-         IF ( Hubbard_U(nt) /= 0.d0 .OR. Hubbard_alpha(nt) /= 0.d0) THEN        
+         IF ( is_hubbard(nt) ) THEN        
             DO m1 = 1, 2 * Hubbard_l(nt) + 1
                DO m2 = m1, 2 * Hubbard_l(nt) + 1
                   IF ( gamma_only ) THEN
-                     DO ibnd = 1,nbnd
-                        dns(m1,m2,current_spin,na) = &
-                           dns(m1,m2,current_spin,na) + wg(ibnd,ik) *&
-                                   (proj%r(oatwfc(na)+m1,ibnd) *      &
-                                    dproj%r(oatwfc(na)+m2,ibnd) +      &
-                                    dproj%r(oatwfc(na)+m1,ibnd) *      &
-                                    proj%r(oatwfc(na)+m2,ibnd))
+                     DO ibnd = nb_s, nb_e
+                        dns_(m1,m2,current_spin,na) = &
+                           dns_(m1,m2,current_spin,na) + wg(ibnd,ik) *&
+                                   ( proj%r(offsetU(na)+m1,ibnd) *      &
+                                    dproj%r(offsetU(na)+m2,ibnd) +      &
+                                    dproj%r(offsetU(na)+m1,ibnd) *      &
+                                     proj%r(offsetU(na)+m2,ibnd))
                      END DO
                   ELSE
-                     DO ibnd = 1,nbnd
-                        dns(m1,m2,current_spin,na) = &
-                           dns(m1,m2,current_spin,na) + wg(ibnd,ik) *&
-                               DBLE(proj%k(oatwfc(na)+m1,ibnd) *      &
-                              CONJG(dproj%k(oatwfc(na)+m2,ibnd) ) +    &
-                                    dproj%k(oatwfc(na)+m1,ibnd)*       &
-                              CONJG(proj%k(oatwfc(na)+m2,ibnd) ) )
+                     DO ibnd = nb_s, nb_e
+                        dns_(m1,m2,current_spin,na) = &
+                           dns_(m1,m2,current_spin,na) + wg(ibnd,ik) *&
+                               DBLE(proj%k(offsetU(na)+m1,ibnd) *      &
+                              CONJG(dproj%k(offsetU(na)+m2,ibnd) ) +    &
+                                    dproj%k(offsetU(na)+m1,ibnd)*       &
+                              CONJG(proj%k(offsetU(na)+m2,ibnd) ) )
                      END DO
                   END IF
                END DO
             END DO
          END IF
       END DO
+10    CALL mp_sum(dns_, intra_pool_comm)
+      dns(:,:,:,:) = dns(:,:,:,:) + dns_(:,:,:,:)
    END DO                 ! on k-points
    !
+   DEALLOCATE ( dns_ )
    CALL mp_sum( dns, inter_pool_comm )
    !
    ! In nspin.eq.1 k-point weight wg is normalized to 2 el/band 
@@ -238,7 +260,7 @@ SUBROUTINE dndepsilon ( dns,ldim,ipol,jpol )
    !
    IF (nspin.EQ.1) dns = 0.5d0 * dns
    !
-   ! impose hermeticity of dn_{m1,m2}
+   ! impose hermiticity of dn_{m1,m2}
    !
    DO na = 1,nat
       nt = ityp(na)
@@ -251,15 +273,17 @@ SUBROUTINE dndepsilon ( dns,ldim,ipol,jpol )
       END DO
    END DO
 
-   DEALLOCATE ( spsi )
    call deallocate_bec_type (proj)
    call deallocate_bec_type (dproj)
    call deallocate_bec_type (becp)
+   DEALLOCATE ( spsi )
+   DEALLOCATE ( wfcatom  )
+
    RETURN
 END SUBROUTINE dndepsilon
 !
 !-----------------------------------------------------------------------
-SUBROUTINE dprojdepsilon_k ( wfcatom, spsi, ik, ipol, jpol, dproj )
+SUBROUTINE dprojdepsilon_k ( spsi, ik, ipol, jpol, nb_s, nb_e, mykey, dproj )
    !-----------------------------------------------------------------------
    !
    ! This routine computes the first derivative of the projection
@@ -268,49 +292,45 @@ SUBROUTINE dprojdepsilon_k ( wfcatom, spsi, ik, ipol, jpol, dproj )
    ! f_{kv} <\fi^{at}_{I,m1}|S|\psi_{k,v,s}><\psi_{k,v,s}|S|\fi^{at}_{I,m2}>)
    !
    USE kinds,                ONLY : DP
-   USE basis,                ONLY : natomwfc
    USE cell_base,            ONLY : tpiba
    USE ions_base,            ONLY : nat, ntyp => nsp, ityp
    USE gvect,                ONLY : g
    USE klist,                ONLY : nks, xk
-   USE ldaU,                 ONLY : Hubbard_l, Hubbard_U, Hubbard_alpha
+   USE ldaU,                 ONLY : hubbard_l, is_hubbard, nwfcU, wfcU
    USE lsda_mod,             ONLY : lsda, nspin, current_spin, isk
    USE wvfct,                ONLY : nbnd, npwx, npw, igk, wg
    USE uspp,                 ONLY : nkb, vkb, qq
    USE uspp_param,           ONLY : upf, nhm, nh
    USE wavefunctions_module, ONLY : evc
    USE becmod,               ONLY : bec_type, becp, calbec
-   USE mp_global,            ONLY : intra_bgrp_comm
+   USE mp_bands,             ONLY : intra_bgrp_comm
    USE mp,                   ONLY : mp_sum
 
    IMPLICIT NONE
    !
    ! I/O variables first
    !
-   INTEGER, INTENT(IN) :: ik, ipol, jpol
+   INTEGER, INTENT(IN) :: ik, ipol, jpol, nb_s, nb_e, mykey
    COMPLEX (DP), INTENT(IN)  :: &
-           wfcatom(npwx,natomwfc), &! the atomic wfc
            spsi(npwx,nbnd)          ! S|evc>
    COMPLEX (DP), INTENT(OUT) :: &
-           dproj(natomwfc,nbnd)     ! the derivative of the projection
+           dproj(nwfcU,nbnd)     ! the derivative of the projection
    !
-   INTEGER :: i, ig, jkb2, lmax_wfc, na, ibnd, iwf, nt, ih,jh, &
-              nworddw, nworddb
+   INTEGER :: i, ig, ijkb0, na, ibnd, iwf, nt, ih,jh
    REAL (DP) :: xyz(3,3), q, a1, a2
    REAL (DP), PARAMETER :: eps=1.0d-8
-   COMPLEX (DP), EXTERNAL :: zdotc
    COMPLEX (DP), ALLOCATABLE :: &
-           dwfc(:,:), aux(:,:), dbeta(:,:), aux1(:,:), &
+           dwfc(:,:), aux(:,:), dbeta(:,:), aux0(:,:), aux1(:,:), &
            betapsi(:,:), dbetapsi(:,:), wfatbeta(:,:), wfatdbeta(:,:)
 
-   !       dwfc(npwx,natomwfc),   ! the derivative of the atomic d wfc
-   !       aux(npwx,natomwfc),    ! auxiliary array
-   !       dbeta(npwx,nkb),       ! the derivative of the beta function
-   !       aux1(npwx,nkb),        ! auxiliary array
-   !       betapsi(nhm,nbnd),     ! <beta|evc>
-   !       dbetapsi(nhm,nbnd),    ! <dbeta|evc>
-   !       wfatbeta(natomwfc,nhm),! <wfc|beta>
-   !       wfatdbeta(natomwfc,nhm)! <wfc|dbeta>
+   !       dwfc(npwx,nwfcU),   ! the derivative of the atomic d wfc
+   !       aux(npwx,nwfcU),    ! auxiliary array
+   !       dbeta(npwx,nkb),    ! the derivative of the beta function
+   !       aux0,aux1(npwx,nkb),! auxiliary arrays
+   !       betapsi(nhm,nbnd),  ! <beta|evc>
+   !       dbetapsi(nhm,nbnd), ! <dbeta|evc>
+   !       wfatbeta(nwfcU,nhm),! <wfc|beta>
+   !       wfatdbeta(nwfcU,nhm)! <wfc|dbeta>
 
    REAL (DP), ALLOCATABLE :: gk(:,:), qm1(:)
    !       gk(3,npwx),
@@ -328,20 +348,12 @@ SUBROUTINE dprojdepsilon_k ( wfcatom, spsi, ik, ipol, jpol, dproj )
    ! <d\fi^{at}_{I,m1}/d\epsilon(ipol,jpol)|S|\psi_{k,v,s}>
    !
    ALLOCATE ( qm1(npwx), gk(3,npwx) )
-   ALLOCATE ( dwfc(npwx,natomwfc), aux(npwx,natomwfc) )
-
-   nworddw = 2*npwx*natomwfc
-   nworddb = 2*npwx*nkb
-
-   lmax_wfc = 0
-   DO nt=1, ntyp
-      lmax_wfc=MAX(lmax_wfc,MAXVAL(upf(nt)%lchi(1:upf(nt)%nwfc)))
-   END DO
+   ALLOCATE ( dwfc(npwx,nwfcU), aux(npwx,nwfcU) )
 
    ! here the derivative of the Bessel function
-   CALL gen_at_dj (ik,natomwfc,lmax_wfc,dwfc)
+   CALL gen_at_dj (ik,nwfcU,is_hubbard,hubbard_l,dwfc)
    ! and here the derivative of the spherical harmonic
-   CALL gen_at_dy (ik,natomwfc,lmax_wfc,xyz(1,ipol),aux)
+   CALL gen_at_dy (ik,nwfcU,is_hubbard,hubbard_l,xyz(1,ipol),aux)
 
    DO ig = 1,npw
       gk(1,ig) = (xk(1,ik)+g(1,igk(ig)))*tpiba
@@ -355,11 +367,11 @@ SUBROUTINE dprojdepsilon_k ( wfcatom, spsi, ik, ipol, jpol, dproj )
       END IF
       a1 = -gk(jpol,ig)
       a2 = -gk(ipol,ig)*gk(jpol,ig)*qm1(ig)
-      DO iwf = 1,natomwfc
+      DO iwf = 1,nwfcU
          dwfc(ig,iwf) = aux(ig,iwf)*a1 + dwfc(ig,iwf)*a2
       END DO
    END DO
-   IF (ipol.EQ.jpol) dwfc(1:npw,:) = dwfc(1:npw,:) - wfcatom(1:npw,:)*0.5d0
+   IF (ipol.EQ.jpol) dwfc(1:npw,:) = dwfc(1:npw,:) - wfcU(1:npw,:)*0.5d0
 
    CALL calbec ( npw, dwfc, spsi, dproj )
 
@@ -368,67 +380,90 @@ SUBROUTINE dprojdepsilon_k ( wfcatom, spsi, ik, ipol, jpol, dproj )
    ! Now the derivatives of the beta functions: we compute the term
    ! <\fi^{at}_{I,m1}|dS/d\epsilon(ipol,jpol)|\psi_{k,v,s}>
    !
-   ALLOCATE (dbeta(npwx,nkb), aux1(npwx,nkb), &
-             dbetapsi(nhm,nbnd), betapsi(nhm,nbnd), wfatbeta(natomwfc,nhm), &
-             wfatdbeta(natomwfc,nhm) )
+   ALLOCATE (aux0(npwx,nkb), aux1(npwx,nkb) )
 
    ! here the derivative of the Bessel function
-   CALL gen_us_dj (ik,dbeta)
+   CALL gen_us_dj (ik, aux0)
    ! and here the derivative of the spherical harmonic
-   CALL gen_us_dy (ik,xyz(1,ipol),aux1)
+   CALL gen_us_dy (ik, xyz(1,ipol), aux1)
 
-   jkb2 = 0
+   ijkb0 = 0
    DO nt=1,ntyp
+      ALLOCATE (dbeta(npwx,nh(nt)), dbetapsi(nh(nt),nbnd), betapsi(nh(nt),nbnd), &
+                wfatbeta(nwfcU,nh(nt)), wfatdbeta(nwfcU,nh(nt)) )
       DO na=1,nat
          IF ( ityp(na) .EQ. nt ) THEN
             DO ih=1,nh(nt)
-               jkb2 = jkb2 + 1
                ! now we compute the true dbeta function
                DO ig = 1,npw
-                  dbeta(ig,jkb2) = - aux1(ig,jkb2)*gk(jpol,ig) - &
-                        dbeta(ig,jkb2) * gk(ipol,ig) * gk(jpol,ig) * qm1(ig)
+                  dbeta(ig,ih) = - aux1(ig,ijkb0+ih)*gk(jpol,ig) - &
+                       aux0(ig,ijkb0+ih) * gk(ipol,ig) * gk(jpol,ig) * qm1(ig)
                   IF (ipol.EQ.jpol) &
-                     dbeta(ig,jkb2) = dbeta(ig,jkb2) - vkb(ig,jkb2)*0.5d0
-               END DO
-               DO ibnd = 1,nbnd
-                  betapsi(ih,ibnd)= becp%k(jkb2,ibnd)
-                  dbetapsi(ih,ibnd)= zdotc(npw,dbeta(1,jkb2),1,evc(1,ibnd),1)
-               END DO
-               DO iwf=1,natomwfc
-                  wfatbeta(iwf,ih) = zdotc(npw,wfcatom(1,iwf),1,vkb(1,jkb2),1)
-                  wfatdbeta(iwf,ih)= zdotc(npw,wfcatom(1,iwf),1,dbeta(1,jkb2),1)
+                       dbeta(ig,ih) = dbeta(ig,ih) - vkb(ig,ijkb0+ih)*0.5d0
                END DO
             END DO
+            CALL calbec(npw, dbeta, evc, dbetapsi )
+            CALL calbec(npw, wfcU, dbeta,wfatdbeta )
             !
-            CALL mp_sum( dbetapsi, intra_bgrp_comm )
-            CALL mp_sum( wfatbeta, intra_bgrp_comm )
-            CALL mp_sum( wfatdbeta, intra_bgrp_comm )
+            ! dbeta is now used as work space to store vkb
+            DO ih=1,nh(nt)
+               DO ig = 1,npw
+                  dbeta(ig,ih) = vkb(ig,ijkb0+ih)
+               END DO
+            END DO
+            CALL calbec(npw, wfcU, dbeta, wfatbeta )
             !
-            DO ibnd = 1,nbnd
-               DO ih=1,nh(nt)
+            ! here starts band parallelization
+            ! beta is here used as work space to calculate dbetapsi
+            betapsi(:,:) = (0.0_dp, 0.0_dp)
+            DO ih=1,nh(nt)
+               DO ibnd = nb_s,nb_e
                   DO jh = 1,nh(nt)
-                     DO iwf=1,natomwfc
-                        dproj(iwf,ibnd) = dproj(iwf,ibnd) +               &
-                                          qq(ih,jh,nt) *                  &
-                               ( wfatdbeta(iwf,ih)*betapsi(jh,ibnd) +     &
-                                 wfatbeta(iwf,ih)*dbetapsi(jh,ibnd) )
-                     END DO
+                     betapsi(ih,ibnd) = betapsi(ih,ibnd) + &
+                          qq(ih,jh,nt)  * dbetapsi(jh,ibnd)
                   END DO
                END DO
             END DO
+            dbetapsi (:,:) =  betapsi(:,:)
+            !
+            DO ih=1,nh(nt)
+               DO ibnd = nb_s,nb_e
+                  betapsi(ih,ibnd)= (0.0_dp, 0.0_dp)
+                  DO jh = 1,nh(nt)
+                     betapsi(ih,ibnd) = betapsi(ih,ibnd) + &
+                          qq(ih,jh,nt) * becp%k(ijkb0+jh,ibnd)
+                  END DO
+               END DO
+            END DO
+            !
+            ijkb0 = ijkb0 + nh(nt)
+            !
+            ! dproj(iwf,ibnd) = \sum_ih wfatdbeta(iwf,ih)*betapsi(ih,ibnd) +
+            !                           wfatbeta(iwf,ih)*dbetapsi(ih,ibnd) 
+            !
+            IF ( mykey == 0 ) THEN
+               CALL ZGEMM('N','N',nwfcU, nb_e-nb_s+1, nh(nt), (1.0_dp,0.0_dp), &
+                    wfatdbeta, nwfcU, betapsi(1,nb_s), nh(nt),(1.0_dp,0.0_dp), &
+                    dproj(1,nb_s), nwfcU)
+               CALL ZGEMM('N','N',nwfcU,nb_e-nb_s+1, nh(nt), (1.0_dp,0.0_dp),  &
+                    wfatbeta, nwfcU, dbetapsi(1,nb_s), nh(nt),(1.0_dp,0.0_dp), &
+                    dproj(1,nb_s), nwfcU)
+            END IF
+            ! end band parallelization - only dproj(1,nb_s:nb_e) are calculated
          END IF
       END DO
+      DEALLOCATE (dbeta, dbetapsi, betapsi, wfatbeta, wfatdbeta )
    END DO
 
 
-   DEALLOCATE (dbeta, aux1, dbetapsi, betapsi, wfatbeta, wfatdbeta )
+   DEALLOCATE ( aux0, aux1 )
    DEALLOCATE ( qm1, gk )
 
    RETURN
 END SUBROUTINE dprojdepsilon_k
 !
 !-----------------------------------------------------------------------
-SUBROUTINE dprojdepsilon_gamma ( wfcatom, spsi, ipol, jpol, dproj )
+SUBROUTINE dprojdepsilon_gamma ( spsi, ipol, jpol, nb_s, nb_e, mykey, dproj )
    !-----------------------------------------------------------------------
    !
    ! This routine computes the first derivative of the projection
@@ -437,49 +472,45 @@ SUBROUTINE dprojdepsilon_gamma ( wfcatom, spsi, ipol, jpol, dproj )
    ! f_{kv} <\fi^{at}_{I,m1}|S|\psi_{k,v,s}><\psi_{k,v,s}|S|\fi^{at}_{I,m2}>)
    !
    USE kinds,                ONLY : DP
-   USE basis,                ONLY : natomwfc
    USE cell_base,            ONLY : tpiba
    USE ions_base,            ONLY : nat, ntyp => nsp, ityp
    USE gvect,                ONLY : g, gstart
    USE klist,                ONLY : nks, xk
-   USE ldaU,                 ONLY : Hubbard_l, Hubbard_U, Hubbard_alpha
+   USE ldaU,                 ONLY : is_hubbard, hubbard_l, nwfcU, wfcU
    USE lsda_mod,             ONLY : lsda, nspin, current_spin, isk
    USE wvfct,                ONLY : nbnd, npwx, npw, igk, wg
    USE uspp,                 ONLY : nkb, vkb, qq
    USE uspp_param,           ONLY : upf, nhm, nh
    USE wavefunctions_module, ONLY : evc
    USE becmod,               ONLY : bec_type, becp, calbec
-   USE mp_global,            ONLY : intra_bgrp_comm
+   USE mp_bands,             ONLY : intra_bgrp_comm
    USE mp,                   ONLY : mp_sum
 
    IMPLICIT NONE
    !
    ! I/O variables first
    !
-   INTEGER, INTENT(IN) :: ipol, jpol
+   INTEGER, INTENT(IN) :: ipol, jpol, nb_s, nb_e, mykey
    COMPLEX (DP), INTENT(IN)  :: &
-           wfcatom(npwx,natomwfc), &! the atomic wfc
            spsi(npwx,nbnd)          ! S|evc>
    REAL (DP), INTENT(OUT) :: &
-           dproj(natomwfc,nbnd)     ! the derivative of the projection
+           dproj(nwfcU,nbnd)     ! the derivative of the projection
    !
-   INTEGER :: ik=1, i, ig, jkb2, lmax_wfc, na, ibnd, iwf, nt, ih,jh, &
-              nworddw, nworddb
+   INTEGER :: ik=1, i, ig, ijkb0, na, ibnd, iwf, nt, ih,jh
    REAL (DP) :: xyz(3,3), q, a1, a2
    REAL (DP), PARAMETER :: eps=1.0d-8
-   REAL (DP), EXTERNAL :: ddot
    COMPLEX (DP), ALLOCATABLE :: &
-           dwfc(:,:), aux(:,:), dbeta(:,:), aux1(:,:)
-   !       dwfc(npwx,natomwfc),   ! the derivative of the atomic d wfc
-   !       aux(npwx,natomwfc),    ! auxiliary array
-   !       dbeta(npwx,nkb),       ! the derivative of the beta function
-   !       aux1(npwx,nkb),        ! auxiliary array
+           dwfc(:,:), aux(:,:), dbeta(:,:), aux0(:,:), aux1(:,:)
+   !       dwfc(npwx,nwfcU),   ! the derivative of the atomic d wfc
+   !       aux(npwx,nwfcU),    ! auxiliary array
+   !       dbeta(npwx,nkb),    ! the derivative of the beta function
+   !       aux0,aux1(npwx,nkb) ! auxiliary arrays
    REAL (DP), ALLOCATABLE :: &
            betapsi(:,:), dbetapsi(:,:), wfatbeta(:,:), wfatdbeta(:,:)
    !       betapsi(nhm,nbnd),     ! <beta|evc>
    !       dbetapsi(nhm,nbnd),    ! <dbeta|evc>
-   !       wfatbeta(natomwfc,nhm),! <wfc|beta>
-   !       wfatdbeta(natomwfc,nhm)! <wfc|dbeta>
+   !       wfatbeta(nwfcU,nhm),! <wfc|beta>
+   !       wfatdbeta(nwfcU,nhm)! <wfc|dbeta>
 
    REAL (DP), ALLOCATABLE :: gk(:,:), qm1(:)
    !       gk(3,npwx),
@@ -497,20 +528,12 @@ SUBROUTINE dprojdepsilon_gamma ( wfcatom, spsi, ipol, jpol, dproj )
    ! <d\fi^{at}_{I,m1}/d\epsilon(ipol,jpol)|S|\psi_{k,v,s}>
    !
    ALLOCATE ( qm1(npwx), gk(3,npwx) )
-   ALLOCATE ( dwfc(npwx,natomwfc), aux(npwx,natomwfc) )
-
-   nworddw = 2*npwx*natomwfc
-   nworddb = 2*npwx*nkb
-
-   lmax_wfc = 0
-   DO nt=1, ntyp
-      lmax_wfc=MAX(lmax_wfc,MAXVAL(upf(nt)%lchi(1:upf(nt)%nwfc)))
-   END DO
+   ALLOCATE ( dwfc(npwx,nwfcU), aux(npwx,nwfcU) )
 
    ! here the derivative of the Bessel function
-   CALL gen_at_dj (ik,natomwfc,lmax_wfc,dwfc)
+   CALL gen_at_dj (ik,nwfcU,is_hubbard,hubbard_l,dwfc)
    ! and here the derivative of the spherical harmonic
-   CALL gen_at_dy (ik,natomwfc,lmax_wfc,xyz(1,ipol),aux)
+   CALL gen_at_dy (ik,nwfcU,is_hubbard,hubbard_l,xyz(1,ipol),aux)
 
    DO ig = 1,npw
       gk(1,ig) = (xk(1,ik)+g(1,igk(ig)))*tpiba
@@ -524,11 +547,11 @@ SUBROUTINE dprojdepsilon_gamma ( wfcatom, spsi, ipol, jpol, dproj )
       END IF
       a1 = -gk(jpol,ig)
       a2 = -gk(ipol,ig)*gk(jpol,ig)*qm1(ig)
-      DO iwf = 1,natomwfc
+      DO iwf = 1,nwfcU
          dwfc(ig,iwf) = aux(ig,iwf)*a1 + dwfc(ig,iwf)*a2
       END DO
    END DO
-   IF (ipol.EQ.jpol) dwfc(1:npw,:) = dwfc(1:npw,:) - wfcatom(1:npw,:)*0.5d0
+   IF (ipol.EQ.jpol) dwfc(1:npw,:) = dwfc(1:npw,:) - wfcU(1:npw,:)*0.5d0
 
    CALL calbec ( npw, dwfc, spsi, dproj )
 
@@ -537,68 +560,84 @@ SUBROUTINE dprojdepsilon_gamma ( wfcatom, spsi, ipol, jpol, dproj )
    ! Now the derivatives of the beta functions: we compute the term
    ! <\fi^{at}_{I,m1}|dS/d\epsilon(ipol,jpol)|\psi_{k,v,s}>
    !
-   ALLOCATE (dbeta(npwx,nkb), aux1(npwx,nkb), &
-             dbetapsi(nhm,nbnd), betapsi(nhm,nbnd), &
-             wfatbeta(natomwfc,nhm), wfatdbeta(natomwfc,nhm) )
+   ALLOCATE (aux0(npwx,nkb), aux1(npwx,nkb) )
 
    ! here the derivative of the Bessel function
-   CALL gen_us_dj (ik,dbeta)
+   CALL gen_us_dj (ik, aux0)
    ! and here the derivative of the spherical harmonic
-   CALL gen_us_dy (ik,xyz(1,ipol),aux1)
+   CALL gen_us_dy (ik, xyz(1,ipol), aux1)
 
-   jkb2 = 0
+   ijkb0 = 0
    DO nt=1,ntyp
+      ALLOCATE (dbeta(npwx,nh(nt)), dbetapsi(nh(nt),nbnd), betapsi(nh(nt),nbnd), &
+                wfatbeta(nwfcU,nh(nt)), wfatdbeta(nwfcU,nh(nt)) )
       DO na=1,nat
          IF ( ityp(na) .EQ. nt ) THEN
             DO ih=1,nh(nt)
-               jkb2 = jkb2 + 1
                ! now we compute the true dbeta function
                DO ig = 1,npw
-                  dbeta(ig,jkb2) = - aux1(ig,jkb2)*gk(jpol,ig) - &
-                        dbeta(ig,jkb2) * gk(ipol,ig) * gk(jpol,ig) * qm1(ig)
+                  dbeta(ig,ih) = - aux1(ig,ijkb0+ih)*gk(jpol,ig) - &
+                       aux0(ig,ijkb0+ih) * gk(ipol,ig) * gk(jpol,ig) * qm1(ig)
                   IF (ipol.EQ.jpol) &
-                     dbeta(ig,jkb2) = dbeta(ig,jkb2) - vkb(ig,jkb2)*0.5d0
-               END DO
-               DO ibnd = 1,nbnd
-                  betapsi(ih,ibnd)= becp%r(jkb2,ibnd)
-                  dbetapsi(ih,ibnd) = 2.0_dp * &
-                      ddot(2*npw,dbeta(1,jkb2),1,evc(1,ibnd),1)
-                  IF ( gstart == 2 ) dbetapsi(ih,ibnd) = &
-                        dbetapsi(ih,ibnd) - dbeta(1,jkb2)*evc(1,ibnd)
-               END DO
-               DO iwf=1,natomwfc
-                  wfatbeta(iwf,ih) = 2.0_dp * &
-                    ddot(2*npw,wfcatom(1,iwf),1,vkb(1,jkb2),1)
-                  IF ( gstart == 2 ) wfatbeta(iwf,ih) = &
-                      wfatbeta(iwf,ih) - wfcatom(1,iwf)*vkb(1,jkb2)
-                  wfatdbeta(iwf,ih)= 2.0_dp * &
-                    ddot(2*npw,wfcatom(1,iwf),1,dbeta(1,jkb2),1)
-                  IF ( gstart == 2 ) wfatdbeta(iwf,ih) = &
-                      wfatdbeta(iwf,ih) - wfcatom(1,iwf)*dbeta(1,jkb2)
+                       dbeta(ig,ih) = dbeta(ig,ih) - vkb(ig,ijkb0+ih)*0.5d0
                END DO
             END DO
             !
-            CALL mp_sum( dbetapsi, intra_bgrp_comm )
-            CALL mp_sum( wfatbeta, intra_bgrp_comm )
-            CALL mp_sum( wfatdbeta, intra_bgrp_comm )
+            CALL calbec(npw, dbeta, evc, dbetapsi )
+            CALL calbec(npw, wfcU, dbeta, wfatdbeta )
             !
-            DO ibnd = 1,nbnd
-               DO ih=1,nh(nt)
+            ! dbeta is now used as work space to store vkb
+            DO ih=1,nh(nt)
+               DO ig = 1,npw
+                  dbeta(ig,ih) = vkb(ig,ijkb0+ih)
+               END DO
+            END DO
+            CALL calbec(npw, wfcU, dbeta, wfatbeta )
+            !
+            ! here starts band parallelization
+            ! beta is here used as work space to calculate dbetapsi
+
+            betapsi(:,:) = 0.0_dp
+            DO ih=1,nh(nt)
+               DO ibnd = nb_s,nb_e
                   DO jh = 1,nh(nt)
-                     DO iwf=1,natomwfc
-                        dproj(iwf,ibnd) = dproj(iwf,ibnd) +               &
-                                          qq(ih,jh,nt) *                  &
-                               ( wfatdbeta(iwf,ih)*betapsi(jh,ibnd) +     &
-                                 wfatbeta(iwf,ih)*dbetapsi(jh,ibnd) )
-                     END DO
+                     betapsi(ih,ibnd) = betapsi(ih,ibnd) + &
+                          qq(ih,jh,nt)  * dbetapsi(jh,ibnd)
                   END DO
                END DO
             END DO
+            dbetapsi (:,:) =  betapsi(:,:)
+            !
+            DO ih=1,nh(nt)
+               DO ibnd = nb_s,nb_e
+                  betapsi(ih,ibnd)= 0.0_dp
+                  DO jh = 1,nh(nt)
+                     betapsi(ih,ibnd) = betapsi(ih,ibnd) + &
+                          qq(ih,jh,nt) * becp%r(ijkb0+jh,ibnd)
+                  END DO
+               END DO
+            END DO
+            !
+            ijkb0 = ijkb0 + nh(nt)
+            !
+            ! dproj(iwf,ibnd) = \sum_ih wfatdbeta(iwf,ih)*betapsi(ih,ibnd) +
+            !                           wfatbeta(iwf,ih)*dbetapsi(ih,ibnd) 
+            !
+            IF ( mykey == 0 ) THEN
+               CALL DGEMM('N','N',nwfcU, nb_e-nb_s+1, nh(nt), 1.0_dp,  &
+                    wfatdbeta, nwfcU, betapsi(1,nb_s), nh(nt), 1.0_dp,&
+                    dproj(1,nb_s), nwfcU)
+               CALL DGEMM('N','N',nwfcU, nb_e-nb_s+1, nh(nt), 1.0_dp,  &
+                    wfatbeta, nwfcU, dbetapsi(1,nb_s), nh(nt), 1.0_dp,&
+                    dproj(1,nb_s), nwfcU)
+            END IF
+            ! end band parallelization - only dproj(1,nb_s:nb_e) are calculated
          END IF
       END DO
+      DEALLOCATE (dbeta, dbetapsi, betapsi, wfatbeta, wfatdbeta )
    END DO
 
-   DEALLOCATE (dbeta, aux1, dbetapsi, betapsi, wfatbeta, wfatdbeta )
+   DEALLOCATE ( aux0, aux1 )
    DEALLOCATE ( qm1, gk )
 
    RETURN
