@@ -14,10 +14,10 @@ subroutine stres_nonloc_dft( rho, rho_core, nspin, sigma_nonloc_dft )
   USE kinds,            ONLY : DP
   use funct,            ONLY : gcxc, gcx_spin, gcc_spin, gcc_spin_more, &
                                dft_is_gradient, get_igcc, get_inlc 
-  USE mp_global,        ONLY : intra_pool_comm
   USE mp,               ONLY : mp_sum
   USE fft_base,         ONLY : dfftp
-  USE vdW_DF,           ONLY : stress_vdW_DF, print_sigma 
+  USE vdW_DF,           ONLY : stress_vdW_DF
+  USE rVV10,            ONLY : stress_rVV10
   !
   IMPLICIT NONE
   !
@@ -32,10 +32,13 @@ subroutine stres_nonloc_dft( rho, rho_core, nspin, sigma_nonloc_dft )
   inlc = get_inlc()
 
   if (inlc==1 .or. inlc==2) then
-     if (nspin>1) call errore('stres_vdW_DF', &
+     if (nspin>2) call errore('stres_vdW_DF', &
                   'vdW+DF non implemented in spin polarized calculations',1)
-     CALL stress_vdW_DF(rho, rho_core, sigma_nonloc_dft)
-
+     CALL stress_vdW_DF(rho, rho_core, nspin, sigma_nonloc_dft)
+  elseif (inlc == 3) then
+      if (nspin>2) call errore('stress_rVV10', &
+                   'rVV10 non implemented with nspin>2',1)
+      CALL stress_rVV10(rho, rho_core, nspin, sigma_nonloc_dft)
   end if
 
   return
