@@ -62,21 +62,20 @@ SUBROUTINE update_pot()
   !
   USE kinds,         ONLY : DP
   USE control_flags, ONLY : pot_order, wfc_order, history, alpha0, beta0
-  USE io_files,      ONLY : prefix, iunupdate, wfc_dir, tmp_dir, nd_nmbr, seqopn
+  USE io_files,      ONLY : prefix, iunupdate, tmp_dir, wfc_dir, nd_nmbr, seqopn
   USE io_global,     ONLY : ionode, ionode_id
   USE cell_base,     ONLY : bg
   USE ions_base,     ONLY : nat, tau, nsp, ityp
   USE gvect,         ONLY : ngm, g
   USE vlocal,        ONLY : strf
   USE mp,            ONLY : mp_bcast
-  USE mp_global,     ONLY : intra_image_comm
+  USE mp_images,     ONLY : intra_image_comm
   !
   IMPLICIT NONE
   !
   REAL(DP), ALLOCATABLE :: tauold(:,:,:)
   INTEGER               :: rho_extr, wfc_extr
   LOGICAL               :: exists
-  CHARACTER (LEN=256)   :: tmp_dir_saved
   !
   !
   CALL start_clock( 'update_pot' )
@@ -119,9 +118,6 @@ SUBROUTINE update_pot()
   CALL mp_bcast( beta0,  ionode_id, intra_image_comm )
   CALL mp_bcast( tauold, ionode_id, intra_image_comm )
   !
-  tmp_dir_saved = tmp_dir
-  IF ( wfc_dir /= 'undefined' ) tmp_dir = wfc_dir
-  !
   IF ( wfc_order > 0 ) THEN
      !
      ! ... determines the maximum effective order of the extrapolation on the
@@ -131,14 +127,14 @@ SUBROUTINE update_pot()
         !
         wfc_extr = MIN( 1, history, wfc_order )
         !
-        INQUIRE( FILE = TRIM( tmp_dir ) // &
+        INQUIRE( FILE = TRIM( wfc_dir ) // &
             & TRIM( prefix ) // '.oldwfc' // nd_nmbr, EXIST = exists )
         !
         IF ( exists ) THEN
            !
            wfc_extr = MIN( 2, history, wfc_order  )
            !
-           INQUIRE( FILE = TRIM( tmp_dir ) // &
+           INQUIRE( FILE = TRIM( wfc_dir ) // &
                & TRIM( prefix ) // '.old2wfc' // nd_nmbr , EXIST = exists )
            !
            IF ( exists ) wfc_extr = MIN( 3, history, wfc_order )
@@ -166,7 +162,6 @@ SUBROUTINE update_pot()
   END IF
   !
   DEALLOCATE( tauold )
-  tmp_dir = tmp_dir_saved
   !
   ! ... determines the maximum effective order of the extrapolation on the
   ! ... basis of the files that are really available (for the charge density)
@@ -237,6 +232,7 @@ SUBROUTINE extrapolate_charge( rho_extr )
   USE io_rho_xml,           ONLY : write_rho, read_rho
   USE paw_variables,        ONLY : okpaw, ddd_paw
   USE paw_onecenter,        ONLY : PAW_potential
+  USE funct,                ONLY : stop_exx
   !
   IMPLICIT NONE
   !
@@ -253,7 +249,10 @@ SUBROUTINE extrapolate_charge( rho_extr )
      !
      ! ... calculate structure factors for the new positions
      !
-     IF ( lmovecell ) CALL scale_h()
+     IF ( lmovecell ) THEN
+       CALL scale_h()
+       CALL stop_exx()
+     ENDIF
      !
      CALL struc_fact( nat, tau, nsp, ityp, ngm, g, bg, &
                       dfftp%nr1, dfftp%nr2, dfftp%nr3, strf, eigts1, eigts2, eigts3 )
@@ -473,7 +472,7 @@ SUBROUTINE extrapolate_wfcs( wfc_extr )
   USE control_flags,        ONLY : gamma_only
   USE becmod,               ONLY : allocate_bec_type, deallocate_bec_type, &
                                    bec_type, becp, calbec
-  USE mp_global,            ONLY : intra_image_comm
+  USE mp_images,            ONLY : intra_image_comm
   USE mp,                   ONLY : mp_barrier
   !
   IMPLICIT NONE
@@ -509,7 +508,7 @@ SUBROUTINE extrapolate_wfcs( wfc_extr )
         !
         ! ... "now"  -> "old"
         !
-        CALL get_buffer( evc, nwordwfc, iunwfc, ik )
+        IF ( nks > 1 ) CALL get_buffer( evc, nwordwfc, iunwfc, ik )
         CALL davcio( evc, 2*nwordwfc, iunoldwfc, ik, +1 )
         !
      END DO
@@ -565,7 +564,7 @@ SUBROUTINE extrapolate_wfcs( wfc_extr )
         ! ... read wavefcts as (t-dt), replace with wavefcts at (t)
         !
         CALL davcio( evcold, 2*nwordwfc, iunoldwfc, ik, -1 )
-        CALL get_buffer( evc, nwordwfc, iunwfc, ik )
+        IF ( nks > 1 ) CALL get_buffer( evc, nwordwfc, iunwfc, ik )
         CALL davcio(    evc, 2*nwordwfc, iunoldwfc, ik, +1 )
         !
         IF ( okvan ) THEN
@@ -665,7 +664,7 @@ SUBROUTINE extrapolate_wfcs( wfc_extr )
         !
         ! ... save interpolated wavefunctions to file iunwfc
         !
-        CALL save_buffer( evc, nwordwfc, iunwfc, ik )
+        IF ( nks > 1 ) CALL save_buffer( evc, nwordwfc, iunwfc, ik )
         !
      END DO
      !
