@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2009 Quantum ESPRESSO group
+! Copyright (C) 2001-2013 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -19,10 +19,13 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
   USE fft_base,         ONLY : dfftp
   USE gvect,            ONLY : ngm
   USE noncollin_module, ONLY : noncolin, nspin_lsda
-  USE ions_base,        ONLY : nat
+  USE ions_base,        ONLY : nat, tau
   USE ldaU,             ONLY : lda_plus_U 
   USE funct,            ONLY : dft_is_meta
   USE scf,              ONLY : scf_type
+  USE cell_base,        ONLY : alat
+  USE control_flags,    ONLY : ts_vdw
+  USE tsvdw_module,     ONLY : tsvdw_calculate, UtsvdW
   !
   IMPLICIT NONE
   !
@@ -44,7 +47,7 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
   REAL(DP), INTENT(INOUT) :: etotefield
     ! electric field energy - inout due to the screwed logic of add_efield
   ! ! 
-  INTEGER :: is
+  INTEGER :: is, ir
   !
   CALL start_clock( 'v_of_rho' )
   !
@@ -67,11 +70,11 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
   ! ... LDA+U: build up Hubbard potential 
   !
   if (lda_plus_u) then
-   if(noncolin) then
-    call v_hubbard_nc(rho%ns_nc,v%ns_nc,eth)
-   else
-    call v_hubbard(rho%ns,v%ns,eth)
-   endif
+     if(noncolin) then
+        call v_hubbard_nc(rho%ns_nc,v%ns_nc,eth)
+     else
+        call v_hubbard(rho%ns,v%ns,eth)
+     endif
   endif
   !
   ! ... add an electric field
@@ -80,6 +83,17 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
      CALL add_efield(v%of_r(1,is), etotefield, rho%of_r, .false. )
   END DO
   !
+  ! ... add Tkatchenko-Scheffler potential (factor 2: Ha -> Ry)
+  ! 
+  IF (ts_vdw) THEN
+     CALL tsvdw_calculate(tau*alat,rho%of_r)
+     DO is = 1, nspin_lsda
+        DO ir=1,dfftp%nnr
+           v%of_r(ir,is)=v%of_r(ir,is)+2.0d0*UtsvdW(ir)
+        END DO
+     END DO
+  END IF
+  !
   CALL stop_clock( 'v_of_rho' )
   !
   RETURN
@@ -103,7 +117,7 @@ SUBROUTINE v_xc_meta( rho, rho_core, rhog_core, etxc, vtxc, v, kedtaur )
                                get_igcx, get_igcc
   USE scf,              ONLY : scf_type
   USE mp,               ONLY : mp_sum
-  USE mp_global,        ONLY : intra_bgrp_comm
+  USE mp_bands,         ONLY : intra_bgrp_comm
   !
   IMPLICIT NONE
   !
@@ -294,7 +308,7 @@ SUBROUTINE v_xc_meta( rho, rho_core, rhog_core, etxc, vtxc, v, kedtaur )
   rhoneg(:) = rhoneg(:) * omega / ( dfftp%nr1*dfftp%nr2*dfftp%nr3 )
   !
   if ((rhoneg(1) > eps8) .or. (rhoneg(2) > eps8)) then
-    write (stdout, '(/,5x, "negative rho (up,down): ", 2e10.3)') rhoneg(:)
+    write (stdout, '(/,5x, "negative rho (up,down): ", 2es10.3)') rhoneg(:)
   end if
   !
   vtxc = omega * vtxc / ( dfftp%nr1*dfftp%nr2*dfftp%nr3 ) 
@@ -326,7 +340,7 @@ SUBROUTINE v_xc( rho, rho_core, rhog_core, etxc, vtxc, v )
   USE spin_orb,         ONLY : domag
   USE funct,            ONLY : xc, xc_spin
   USE scf,              ONLY : scf_type
-  USE mp_global,        ONLY : intra_pool_comm, intra_bgrp_comm, mpime
+  USE mp_bands,         ONLY : intra_bgrp_comm
   USE mp,               ONLY : mp_sum
 
   !
@@ -423,7 +437,7 @@ SUBROUTINE v_xc( rho, rho_core, rhog_core, etxc, vtxc, v )
            !
            etxc = etxc + e2*( ex + ec ) * rhox
            !
-           vtxc = vtxc + v(ir,1) * rho%of_r(ir,1) + v(ir,2) * rho%of_r(ir,2)
+           vtxc = vtxc + ( v(ir,1)*rho%of_r(ir,1) + v(ir,2)*rho%of_r(ir,2) )
            !
         END IF
         !
@@ -488,7 +502,7 @@ SUBROUTINE v_xc( rho, rho_core, rhog_core, etxc, vtxc, v )
   rhoneg(:) = rhoneg(:) * omega / ( dfftp%nr1*dfftp%nr2*dfftp%nr3 )
   !
   IF ( rhoneg(1) > eps8 .OR. rhoneg(2) > eps8 ) &
-     WRITE( stdout,'(/,5X,"negative rho (up, down): ",2E10.3)') rhoneg
+     WRITE( stdout,'(/,5X,"negative rho (up, down): ",2ES10.3)') rhoneg
   !
   ! ... energy terms, local-density contribution
   !
@@ -527,7 +541,7 @@ SUBROUTINE v_h( rhog, ehart, charge, v )
   USE lsda_mod,  ONLY : nspin
   USE cell_base, ONLY : omega, tpiba2
   USE control_flags, ONLY : gamma_only
-  USE mp_global, ONLY: intra_pool_comm, intra_bgrp_comm
+  USE mp_bands,  ONLY: intra_bgrp_comm
   USE mp,        ONLY: mp_sum
   USE martyna_tuckerman, ONLY : wg_corr_h, do_comp_mt
   USE esm,       ONLY: do_comp_esm, esm_hartree, esm_bc
