@@ -59,11 +59,13 @@ PROGRAM ev
       USE kinds, ONLY: DP
       USE constants, ONLY: bohr_radius_angs, ry_kbar
       USE ev_xml,    ONLY : write_evdata_xml
-      USE mp,        ONLY : mp_start
-      USE mp_global, ONLY : mp_global_end, nproc, mpime
+      USE mp_global, ONLY : mp_startup, mp_global_end
+      USE mp_world,  ONLY : world_comm
+      USE mp,        ONLY : mp_bcast
+      USE io_global, ONLY : ionode, ionode_id
       IMPLICIT NONE
       INTEGER, PARAMETER:: nmaxpar=4, nmaxpt=100, nseek=10000, nmin=4
-      INTEGER :: npar,npt,istat,ios,gid
+      INTEGER :: npar,npt,istat, ierr
       CHARACTER :: bravais*3, au_unit*3, filin*256
       REAL(DP) :: par(nmaxpar), deltapar(nmaxpar), parmin(nmaxpar), &
              parmax(nmaxpar), v0(nmaxpt), etot(nmaxpt), efit(nmaxpt), &
@@ -72,17 +74,16 @@ PROGRAM ev
       LOGICAL :: in_angstrom
       CHARACTER(LEN=256) :: fileout
   !
-  CALL mp_start( nproc, mpime, gid )
+  CALL mp_startup ( )
   !
-  IF ( mpime == 0 ) THEN
+  IF ( ionode ) THEN
 
       PRINT '(5x,"Lattice parameter or Volume are in (au, Ang) > ",$)'
       READ '(a)', au_unit
       in_angstrom = au_unit=='Ang' .or. au_unit=='ANG' .or. &
                     au_unit=='ang'
       IF (in_angstrom) PRINT '(5x,"Assuming Angstrom")'
-
-      PRINT '(5x,"Enter type of bravais lattice (fcc, bcc, sc, hex) > ",$)'
+      PRINT '(5x,"Enter type of bravais lattice (fcc, bcc, sc, noncubic) > ",$)'
       READ '(a)',bravais
 !
       IF(bravais=='fcc'.or.bravais=='FCC') THEN
@@ -91,7 +92,8 @@ PROGRAM ev
          fac = 0.50d0
       ELSEIF(bravais=='sc'.or.bravais=='SC') THEN
          fac = 1.0d0
-      ELSEIF(bravais=='hex'.or.bravais=='HEX') THEN
+      ELSEIF(bravais=='noncubic'.or.bravais=='NONCUBIC' .or.  &
+             bravais=='hex'.or.bravais=='HEX' ) THEN
 !         fac = sqrt(3d0)/2d0 ! not used
          fac = 0.0_DP ! not used
       ELSE
@@ -112,15 +114,16 @@ PROGRAM ev
       ENDIF
       PRINT '(5x,"Input file > ",$)'
       READ '(a)',filin
-      OPEN(unit=2,file=filin,status='old',form='formatted',iostat=ios)
-      IF (ios/=0) THEN
-         PRINT '(5x,"File ",A," cannot be opened, stopping")', trim(filin)
-         STOP
-      ENDIF
+      OPEN(unit=2,file=filin,status='old',form='formatted',iostat=ierr)
+      IF (ierr/=0) THEN
+         ierr= 1 
+         GO TO 99
+      END IF
   10  CONTINUE
       emin=1d10
       DO npt=1,nmaxpt
-         IF (bravais=='hex'.or.bravais=='HEX') THEN
+         IF (bravais=='noncubic'.or.bravais=='NONCUBIC' .or. &
+             bravais=='hex'.or.bravais=='HEX' ) THEN
             READ(2,*,err=10,END=20) v0(npt), etot(npt)
             IF (in_angstrom) v0(npt)=v0(npt)/bohr_radius_angs**3
          ELSE
@@ -169,9 +172,21 @@ PROGRAM ev
             fileout)
 !
       CALL write_evdata_xml  &
-           (npt,fac,v0,etot,efit,istat,par,npar,emin,chisq,fileout)
+           (npt,fac,v0,etot,efit,istat,par,npar,emin,chisq,fileout, ierr)
+
+      IF (ierr /= 0) GO TO 99
+    ENDIF
+99  CALL mp_bcast ( ierr, ionode_id, world_comm )
+    IF ( ierr == 1) THEN
+       CALL errore( 'ev', 'file '//trim(filin)//' cannot be opened', ierr )
+    ELSE IF ( ierr == 2 ) THEN
+       CALL errore( 'ev', 'file '//trim(fileout)//' cannot be opened', ierr )
+    ELSE IF ( ierr == 11 ) THEN
+       CALL errore( 'write_evdata_xml', 'no free units to write ', ierr )
+    ELSE IF ( ierr == 12 ) THEN
+       CALL errore( 'write_evdata_xml', 'error opening the xml file ', ierr )
+    ENDIF
 
-  ENDIF
   CALL mp_global_end()
 
       STOP
@@ -265,11 +280,11 @@ PROGRAM ev
          IF (exst) PRINT '(5x,"Beware: file ",A," will be overwritten")',&
                   trim(filout)
          OPEN(unit=iun,file=filout,form='formatted',status='unknown', &
-              iostat=ios)
-         IF (ios /= 0) THEN
-            PRINT '(5x,"Cannot open file ",A)',trim(filout)
-            STOP
-         ENDIF
+              iostat=ierr)
+         IF (ierr/=0) THEN
+            ierr= 2 
+            GO TO 99
+         END IF
       ELSE
          iun=6
       ENDIF
@@ -361,11 +376,9 @@ PROGRAM ev
                etot(i)-efit(i), p(i)/gpa_kbar, epv(i), i=1,npt )
          end if
 
-
       ENDIF
-
       IF(filout/=' ') CLOSE(unit=iun)
-      RETURN
+ 99   RETURN
     END SUBROUTINE write_results
 !
 !-----------------------------------------------------------------------
