@@ -24,12 +24,12 @@
 #include "iotk_auxmacros.h"
 
 
-#ifdef __IOTK_LOGICAL2
-#if 3 <= __IOTK_MAXRANK
+#ifdef __IOTK_LOGICAL1
+#if 6 <= __IOTK_MAXRANK
 
 
 
-subroutine iotk_write_attr_LOGICAL2_3(attr,name,val,dummy,first,newline,fmt,ierr)
+subroutine iotk_write_attr_LOGICAL1_6(attr,name,val,dummy,first,newline,fmt,ierr)
   use iotk_base
   use iotk_error_interf
   use iotk_attr_interf
@@ -38,7 +38,7 @@ subroutine iotk_write_attr_LOGICAL2_3(attr,name,val,dummy,first,newline,fmt,ierr
   implicit none
   character(*), intent(inout) :: attr
   character(*), intent(in)    :: name
-  LOGICAL(kind=iotk_LOGICAL2), intent(in)  :: val (:,:,:)
+  LOGICAL(kind=iotk_LOGICAL1), intent(in)  :: val (:,:,:,:,:,:)
   type(iotk_dummytype), optional :: dummy
   logical, optional, intent(in)  :: first
   logical, optional, intent(in)  :: newline
@@ -103,9 +103,9 @@ call iotk_error_msg(ierrl,'Attribute dummy argument is too short')
   else
     if(ierrl/=0) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_write_attr_LOGICAL2_3
+end subroutine iotk_write_attr_LOGICAL1_6
 
-subroutine iotk_scan_attr_LOGICAL2_3(attr,name,val,dummy,found,default,eos,ierr)
+subroutine iotk_scan_attr_LOGICAL1_6(attr,name,val,dummy,found,default,eos,ierr)
   use iotk_base
   use iotk_error_interf
   use iotk_attr_interf, only : iotk_read
@@ -115,13 +115,13 @@ subroutine iotk_scan_attr_LOGICAL2_3(attr,name,val,dummy,found,default,eos,ierr)
   character(*),             intent(in)  :: attr
   character(*),             intent(in)  :: name
 #ifdef __IOTK_WORKAROUND6
-  LOGICAL(kind=iotk_LOGICAL2)                        :: val (:,:,:)
+  LOGICAL(kind=iotk_LOGICAL1)                        :: val (:,:,:,:,:,:)
 #else
-  LOGICAL(kind=iotk_LOGICAL2), intent(out)           :: val (:,:,:)
+  LOGICAL(kind=iotk_LOGICAL1), intent(out)           :: val (:,:,:,:,:,:)
 #endif
   type(iotk_dummytype), optional :: dummy
   logical,        optional, intent(out) :: found
-  LOGICAL(kind=iotk_LOGICAL2), optional, intent(in)  :: default (:,:,:)
+  LOGICAL(kind=iotk_LOGICAL1), optional, intent(in)  :: default (:,:,:,:,:,:)
   logical,        optional, intent(in)  :: eos
   integer,        optional, intent(out) :: ierr
   integer :: ierrl
@@ -130,7 +130,7 @@ subroutine iotk_scan_attr_LOGICAL2_3(attr,name,val,dummy,found,default,eos,ierr)
   logical :: foundl
   character(iotk_vallenx) :: valc
   integer :: index
-  LOGICAL(kind=iotk_LOGICAL2), allocatable :: tmpval (:)
+  LOGICAL(kind=iotk_LOGICAL1), allocatable :: tmpval (:)
   ierrl = 0
   attlen=iotk_strlen(attr)
   namlen=iotk_strlen_trim(name)
@@ -219,14 +219,14 @@ call iotk_error_write(ierrl,"name",name)
   else
     if(ierrl/=0) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_scan_attr_LOGICAL2_3
+end subroutine iotk_scan_attr_LOGICAL1_6
 
 #endif
 #endif
 
-subroutine iotk_attr_dummy_LOGICAL2_3
+subroutine iotk_attr_dummy_LOGICAL1_6
   write(0,*)
-end subroutine iotk_attr_dummy_LOGICAL2_3
+end subroutine iotk_attr_dummy_LOGICAL1_6
 
 
 
@@ -239,12 +239,12 @@ end subroutine iotk_attr_dummy_LOGICAL2_3
 #include "iotk_auxmacros.h"
 
 
-#ifdef __IOTK_LOGICAL2
-#if 4 <= __IOTK_MAXRANK
+#ifdef __IOTK_LOGICAL1
+#if 7 <= __IOTK_MAXRANK
 
 
 
-subroutine iotk_write_attr_LOGICAL2_4(attr,name,val,dummy,first,newline,fmt,ierr)
+subroutine iotk_write_attr_LOGICAL1_7(attr,name,val,dummy,first,newline,fmt,ierr)
   use iotk_base
   use iotk_error_interf
   use iotk_attr_interf
@@ -253,7 +253,7 @@ subroutine iotk_write_attr_LOGICAL2_4(attr,name,val,dummy,first,newline,fmt,ierr
   implicit none
   character(*), intent(inout) :: attr
   character(*), intent(in)    :: name
-  LOGICAL(kind=iotk_LOGICAL2), intent(in)  :: val (:,:,:,:)
+  LOGICAL(kind=iotk_LOGICAL1), intent(in)  :: val (:,:,:,:,:,:,:)
   type(iotk_dummytype), optional :: dummy
   logical, optional, intent(in)  :: first
   logical, optional, intent(in)  :: newline
@@ -318,9 +318,9 @@ call iotk_error_msg(ierrl,'Attribute dummy argument is too short')
   else
     if(ierrl/=0) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_write_attr_LOGICAL2_4
+end subroutine iotk_write_attr_LOGICAL1_7
 
-subroutine iotk_scan_attr_LOGICAL2_4(attr,name,val,dummy,found,default,eos,ierr)
+subroutine iotk_scan_attr_LOGICAL1_7(attr,name,val,dummy,found,default,eos,ierr)
   use iotk_base
   use iotk_error_interf
   use iotk_attr_interf, only : iotk_read
@@ -330,13 +330,13 @@ subroutine iotk_scan_attr_LOGICAL2_4(attr,name,val,dummy,found,default,eos,ierr)
   character(*),             intent(in)  :: attr
   character(*),             intent(in)  :: name
 #ifdef __IOTK_WORKAROUND6
-  LOGICAL(kind=iotk_LOGICAL2)                        :: val (:,:,:,:)
+  LOGICAL(kind=iotk_LOGICAL1)                        :: val (:,:,:,:,:,:,:)
 #else
-  LOGICAL(kind=iotk_LOGICAL2), intent(out)           :: val (:,:,:,:)
+  LOGICAL(kind=iotk_LOGICAL1), intent(out)           :: val (:,:,:,:,:,:,:)
 #endif
   type(iotk_dummytype), optional :: dummy
   logical,        optional, intent(out) :: found
-  LOGICAL(kind=iotk_LOGICAL2), optional, intent(in)  :: default (:,:,:,:)
+  LOGICAL(kind=iotk_LOGICAL1), optional, intent(in)  :: default (:,:,:,:,:,:,:)
   logical,        optional, intent(in)  :: eos
   integer,        optional, intent(out) :: ierr
   integer :: ierrl
@@ -345,7 +345,7 @@ subroutine iotk_scan_attr_LOGICAL2_4(attr,name,val,dummy,found,default,eos,ierr)
   logical :: foundl
   character(iotk_vallenx) :: valc
   integer :: index
-  LOGICAL(kind=iotk_LOGICAL2), allocatable :: tmpval (:)
+  LOGICAL(kind=iotk_LOGICAL1), allocatable :: tmpval (:)
   ierrl = 0
   attlen=iotk_strlen(attr)
   namlen=iotk_strlen_trim(name)
@@ -434,228 +434,13 @@ call iotk_error_write(ierrl,"name",name)
   else
     if(ierrl/=0) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_scan_attr_LOGICAL2_4
+end subroutine iotk_scan_attr_LOGICAL1_7
 
 #endif
 #endif
 
-subroutine iotk_attr_dummy_LOGICAL2_4
+subroutine iotk_attr_dummy_LOGICAL1_7
   write(0,*)
-end subroutine iotk_attr_dummy_LOGICAL2_4
-
-
-
-
-!------------------------------------------------------------------------------!
-! Inclusion of configuration file
-#include "iotk_config.h"
-!------------------------------------------------------------------------------!
-
-#include "iotk_auxmacros.h"
-
-
-#ifdef __IOTK_LOGICAL2
-#if 5 <= __IOTK_MAXRANK
-
-
-
-subroutine iotk_write_attr_LOGICAL2_5(attr,name,val,dummy,first,newline,fmt,ierr)
-  use iotk_base
-  use iotk_error_interf
-  use iotk_attr_interf
-  use iotk_str_interf
-  use iotk_misc_interf
-  implicit none
-  character(*), intent(inout) :: attr
-  character(*), intent(in)    :: name
-  LOGICAL(kind=iotk_LOGICAL2), intent(in)  :: val (:,:,:,:,:)
-  type(iotk_dummytype), optional :: dummy
-  logical, optional, intent(in)  :: first
-  logical, optional, intent(in)  :: newline
-  character(*), optional, intent(in) :: fmt
-  integer, optional, intent(out) :: ierr
-  integer :: ierrl
-  integer :: attlen
-  integer :: vallen
-  integer :: namlen
-  character :: delim
-  character(len=300) :: usefmt
-  character(iotk_vallenx) :: tmpval
-  logical :: nl
-  if(present(newline)) then
-    nl = newline
-  else
-    nl = .false.
-  endif
-!-<
-  if (present(fmt)) then
-    usefmt = fmt
-  else
-    usefmt = "!"
-  end if
-!->
-  ierrl = 0
-  if(present(first)) then
-    if(first) attr(1:1) = iotk_eos
-  end if
-  attlen = iotk_strlen_trim(attr)
-  namlen = iotk_strlen_trim(name)
-  if(.not.iotk_check_name(name)) then
-    call iotk_error_issue(ierrl,"iotk_write_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierrl,'Wrong tag name')
-call iotk_error_write(ierrl,"name",name(1:namlen))
-    goto 1
-  end if
-  delim = '"'
-  call iotk_write(pack(val,mask=.true.),tmpval,usefmt,ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-    goto 1
-  end if
-  vallen = iotk_strlen(tmpval)
-  if(attlen+vallen+namlen+5>len(attr)) then
-    call iotk_error_issue(ierrl,"iotk_write_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierrl,'Attribute dummy argument is too short')
-    goto 1
-  end if
-  if(.not. nl) then
-    attr(attlen+1:attlen+vallen+namlen+5) = " "//name(1:namlen)//"="//delim//tmpval(1:vallen)//delim//iotk_eos
-  else
-    attr(attlen+1:attlen+vallen+namlen+len(iotk_newline)+5) &
-       = iotk_newline//" "//name(1:namlen)//"="//delim//tmpval(1:vallen)//delim//iotk_eos
-  endif
-1 continue
-  if(present(ierr)) then
-    ierr = ierrl
-  else
-    if(ierrl/=0) call iotk_error_handler(ierrl)
-  end if
-end subroutine iotk_write_attr_LOGICAL2_5
-
-subroutine iotk_scan_attr_LOGICAL2_5(attr,name,val,dummy,found,default,eos,ierr)
-  use iotk_base
-  use iotk_error_interf
-  use iotk_attr_interf, only : iotk_read
-  use iotk_str_interf
-  use iotk_misc_interf
-  implicit none
-  character(*),             intent(in)  :: attr
-  character(*),             intent(in)  :: name
-#ifdef __IOTK_WORKAROUND6
-  LOGICAL(kind=iotk_LOGICAL2)                        :: val (:,:,:,:,:)
-#else
-  LOGICAL(kind=iotk_LOGICAL2), intent(out)           :: val (:,:,:,:,:)
-#endif
-  type(iotk_dummytype), optional :: dummy
-  logical,        optional, intent(out) :: found
-  LOGICAL(kind=iotk_LOGICAL2), optional, intent(in)  :: default (:,:,:,:,:)
-  logical,        optional, intent(in)  :: eos
-  integer,        optional, intent(out) :: ierr
-  integer :: ierrl
-  integer :: attlen,pos,equal,namlen
-  character :: delim
-  logical :: foundl
-  character(iotk_vallenx) :: valc
-  integer :: index
-  LOGICAL(kind=iotk_LOGICAL2), allocatable :: tmpval (:)
-  ierrl = 0
-  attlen=iotk_strlen(attr)
-  namlen=iotk_strlen_trim(name)
-  foundl = .false.
-  equal = 0
-  do
-    pos = verify(attr(equal+1:attlen)," ")
-    if(pos<=0) exit
-    equal = equal + pos
-    pos = scan(attr(equal+1:attlen),"=")
-    if(pos<=0) then
-      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierrl,'')
-call iotk_error_write(ierrl,"attr",attr(equal+1:attlen))
-      goto 1
-    end if
-    equal = equal + pos
-    if(trim(attr(equal-pos:equal-1))==name(1:namlen)) foundl = .true.
-    pos = verify(attr(equal+1:attlen)," ")
-    if(pos<=0) then
-      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-      goto 1
-    end if
-    equal = equal + pos
-    delim = attr(equal:equal)
-    if(delim/="'" .and. delim/='"') then
-      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-      goto 1
-    end if
-    pos = scan(attr(equal+1:attlen),delim)
-    if(pos<=0) then
-      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-      goto 1
-    end if
-    if(foundl) exit
-    equal = equal + pos
-  end do
-  if(foundl) then
-    call iotk_strcpy(valc,attr(equal+1:equal+pos-1),ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-      goto 1
-    end if
-  else
-    goto 1
-  end if
-  allocate(tmpval(size(val)))
-  index = 0
-  call iotk_str_clean(valc)
-  call iotk_read(tmpval,valc(1:iotk_strlen(valc)),index,ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-    goto 1
-  end if
-  if(index/=size(val)) then
-    call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierrl,'Attribute size does not match')
-call iotk_error_write(ierrl,"attr",valc)
-call iotk_error_write(ierrl,"size",size(tmpval))
-    goto 1
-  end if
-  val = reshape (source=tmpval,shape=shape(val))
-  deallocate(tmpval)
-1 continue
-  if(ierrl/=0) foundl=.false.
-  if(present(found)) found = foundl
-  if(ierrl==0 .and. .not. present(found) .and. .not. present(default) .and. .not. foundl) then
-    call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierrl,'Attribute not found')
-call iotk_error_write(ierrl,"name",name)
-    ierrl = - ierrl
-  end if
-  if(present(default) .and. .not. foundl) then
-    val = default
-  end if
-  if(present(ierr)) then
-    ierr = ierrl
-  else
-    if(ierrl/=0) call iotk_error_handler(ierrl)
-  end if
-end subroutine iotk_scan_attr_LOGICAL2_5
-
-#endif
-#endif
-
-subroutine iotk_attr_dummy_LOGICAL2_5
-  write(0,*)
-end subroutine iotk_attr_dummy_LOGICAL2_5
+end subroutine iotk_attr_dummy_LOGICAL1_7
 
 
