@@ -17,7 +17,7 @@ PROGRAM uspp2upf
   CHARACTER(len=256) filein, fileout
   !
   !
-  IF ( TRIM(filein) == ' ') &
+  IF ( trim(filein) == ' ') &
        CALL errore ('uspp2upf', 'usage: uspp2upf "file-to-be-converted"', 1)
   CALL get_file ( filein )
   OPEN(unit=1,file=filein,status='old',form='unformatted')
@@ -33,7 +33,7 @@ PROGRAM uspp2upf
   PRINT '(''Output PP file in UPF format :  '',a)', fileout
 
   OPEN(unit=2,file=fileout,status='unknown',form='formatted')
-  CALL write_upf(2)
+  CALL write_upf_v1(2)
   CLOSE (unit=2)
 
   STOP
